@@ -9,6 +9,7 @@
 
 
 
+#include "Cursors32/CrosshairCursor.xpm"
 #include "Cursors32/DisabledCursor.xpm"
 #include "Cursors32/RearrangeCursor.xpm"
 #include "Cursors32/RearrangingCursor.xpm"
@@ -30,4 +31,5 @@
 #endif
 #include "Cursors32/SubViewsCursor.xpm"
 
+AUDACITY_DLL_API
 std::unique_ptr<wxCursor> MakeCursor(int WXUNUSED(CursorId), const char * const pXpm[36], int HotX, int HotY);
