@@ -203,9 +203,6 @@ void UndoManager::ModifyState(const TrackList &l,
 //   SonifyEndModifyState();
 
    EnqueueMessage({ UndoRedoMessage::Modified });
-
-   if (saved == current)
-      saved = -1;
 }
 
 void UndoManager::RenameState( int state,
@@ -355,6 +352,11 @@ void UndoManager::VisitStates(
    }
 }
 
+void UndoManager::MarkUnsaved()
+{
+   saved = -1;
+}
+
 bool UndoManager::UnsavedChanges() const
 {
    return (saved != current);
