@@ -13,12 +13,13 @@
 
 #include "CellularPanel.h"
 #include "widgets/Ruler.h" // member variable
+#include "Observer.h"
 #include "Prefs.h"
 #include "ViewInfo.h" // for PlayRegion
 
 class AudacityProject;
+struct AudioIOEvent;
 struct SelectedRegionEvent;
-class SnapManager;
 class TrackList;
 
 // This is an Audacity Specific ruler panel.
@@ -59,8 +60,6 @@ public:
 
    void SetPlayRegion(double playRegionStart, double playRegionEnd);
    void ClearPlayRegion();
-   void LockPlayRegion();
-   void UnlockPlayRegion();
    void TogglePinnedHead();
 
    void GetMaxSize(wxCoord *width, wxCoord *height);
@@ -70,7 +69,7 @@ public:
    void UpdatePrefs() override;
    void ReCreateButtons();
 
-   void UpdateQuickPlayPos(wxCoord &mousePosX, bool shiftDown);
+   void UpdateQuickPlayPos(wxCoord &mousePosX);
 
    bool ShowingScrubRuler() const;
    //void OnToggleScrubRulerFromMenu(wxCommandEvent& );
@@ -82,27 +81,35 @@ public:
 private:
    void DoIdle();
    void OnIdle( wxIdleEvent &evt );
-   void OnAudioStartStop(wxCommandEvent & evt);
+   void OnAudioStartStop(AudioIOEvent);
    void OnPaint(wxPaintEvent &evt);
    void OnSize(wxSizeEvent &evt);
-   void OnThemeChange(wxCommandEvent& evt);
-   void OnSelectionChange(SelectedRegionEvent& evt);
+   void OnLeave(wxMouseEvent &evt);
+   void OnThemeChange(struct ThemeChangeMessage);
+   void OnSelectionChange(Observer::Message);
    void DoSelectionChange( const SelectedRegion &selectedRegion );
    bool UpdateRects();
    void HandleQPClick(wxMouseEvent &event, wxCoord mousePosX);
    void HandleQPDrag(wxMouseEvent &event, wxCoord mousePosX);
    void HandleQPRelease(wxMouseEvent &event);
-   void StartQPPlay(bool looped, bool cutPreview);
+   void StartQPPlay(
+      bool newDefault, bool cutPreview, const double *pStartTime = nullptr);
 
    void DoDrawBackground(wxDC * dc);
    void DoDrawEdge(wxDC *dc);
    void DoDrawMarks(wxDC * dc, bool /*text */ );
-   void DoDrawSelection(wxDC * dc);
+   wxRect RegionRectangle(double t0, double t1) const;
+   wxRect PlayRegionRectangle() const;
+   wxRect SelectedRegionRectangle() const;
+   void DoDrawPlayRegion(wxDC * dc,
+      const wxRect &rectP, const wxRect &rectL, const wxRect &rectR);
+   void DoDrawPlayRegionLimits(wxDC * dc, const wxRect &rect);
+   void DoDrawOverlap(wxDC * dc, const wxRect &rect);
+   void DoDrawSelection(wxDC * dc,
+      const wxRect &rectS, const wxRect &rectL, const wxRect &rectR);
 
 public:
-   std::pair< wxPoint, wxBitmap >
-      GetIndicatorBitmap(wxCoord xx, bool playing) const;
-   void DoDrawIndicator(wxDC * dc, wxCoord xx, bool playing, int width, bool scrub, bool seek);
+   void DoDrawScrubIndicator(wxDC * dc, wxCoord xx, int width, bool scrub, bool seek);
    void UpdateButtonStates();
 
 private:
@@ -113,14 +120,14 @@ private:
 public:
    static TempAllowFocus TemporarilyAllowFocus();
 
-private:
-   void DoDrawPlayRegion(wxDC * dc);
+   void SetNumGuides(size_t nn);
 
+private:
    enum class MenuChoice { QuickPlay, Scrub };
    void ShowContextMenu( MenuChoice choice, const wxPoint *pPosition);
 
-   double Pos2Time(int p, bool ignoreFisheye = false);
-   int Time2Pos(double t, bool ignoreFisheye = false);
+   double Pos2Time(int p, bool ignoreFisheye = false) const;
+   int Time2Pos(double t, bool ignoreFisheye = false) const;
 
    bool IsWithinMarker(int mousePosX, double markerTime);
 
@@ -138,10 +145,13 @@ private:
 
 
    double mIndTime;
-   double mQuickPlayPosUnsnapped;
-   double mQuickPlayPos;
 
-   bool mIsSnapped;
+   static constexpr size_t MAX_GUIDES = 2;
+   double mQuickPlayOffset[MAX_GUIDES]{};
+   double mQuickPlayPosUnsnapped[MAX_GUIDES]{};
+   double mQuickPlayPos[MAX_GUIDES]{};
+   bool mIsSnapped[MAX_GUIDES]{};
+   size_t mNumGuides{ 1 };
 
    PlayRegion mOldPlayRegion;
 
@@ -152,20 +162,20 @@ private:
    //
    void ShowMenu(const wxPoint & pos);
    void ShowScrubMenu(const wxPoint & pos);
-   void DragSelection();
-   void HandleSnapping();
-   void OnToggleQuickPlay(wxCommandEvent &evt);
+   static void DragSelection(AudacityProject &project);
+   void HandleSnapping(size_t index);
    void OnSyncSelToQuickPlay(wxCommandEvent &evt);
    //void OnTimelineToolTips(wxCommandEvent &evt);
    void OnAutoScroll(wxCommandEvent &evt);
-   void OnLockPlayRegion(wxCommandEvent &evt);
+   void OnTogglePlayRegion(wxCommandEvent &evt);
+   void OnClearPlayRegion(wxCommandEvent &evt);
+   void OnSetPlayRegionToSelection(wxCommandEvent &evt);
 
    void OnPinnedButton(wxCommandEvent & event);
    void OnTogglePinnedState(wxCommandEvent & event);
 
    bool mPlayRegionDragsSelection;
    bool mTimelineToolTip;
-   bool mQuickPlayEnabled;
 
    enum MouseEventState {
       mesNone,
@@ -206,14 +216,18 @@ private:
    void CreateOverlays();
 
    // Cooperating objects
-   class QuickPlayIndicatorOverlay;
-   std::shared_ptr<QuickPlayIndicatorOverlay> mOverlay;
+   class TrackPanelGuidelineOverlay;
+   std::shared_ptr<TrackPanelGuidelineOverlay> mOverlay;
 
-   class QuickPlayRulerOverlay;
+   class ScrubbingRulerOverlay;
    
 private:
    class CommonRulerHandle;
    class QPHandle;
+   class PlayRegionAdjustingHandle;
+   class MovePlayRegionHandle;
+   class ResizePlayRegionHandle;
+   class NewPlayRegionHandle;
    class ScrubbingHandle;
 
    class CommonCell;
@@ -224,14 +238,19 @@ private:
    class ScrubbingCell;
    std::shared_ptr<ScrubbingCell> mScrubbingCell;
 
+   Observer::Subscription mAudioIOSubscription,
+      mPlayRegionSubscription,
+      mThemeChangeSubscription;
+
    // classes implementing subdivision for CellularPanel
    struct Subgroup;
    struct MainGroup;
 
    SelectedRegion mLastDrawnSelectedRegion;
+   std::pair<double, double> mLastDrawnPlayRegion{};
+   bool mLastPlayRegionActive = false;
    double mLastDrawnH{};
    double mLastDrawnZoom{};
-   bool mDirtySelectedRegion{};
 };
 
 #endif //define __AUDACITY_ADORNED_RULER_PANEL__
