@@ -11,7 +11,7 @@
 #ifndef __AUDACITY_CLIPBOARD__
 #define __AUDACITY_CLIPBOARD__
 
-#include "Audacity.h"
+
 
 #include <memory>
 #include <wx/event.h> // to inherit wxEvtHandler
@@ -23,7 +23,7 @@ class TrackList;
 wxDECLARE_EXPORTED_EVENT( AUDACITY_DLL_API,
                           EVT_CLIPBOARD_CHANGE, wxCommandEvent );
 
-class Clipboard final
+class AUDACITY_DLL_API Clipboard final
    : public wxEvtHandler
 {
 public:
@@ -35,20 +35,23 @@ public:
    double T1() const { return mT1; }
    double Duration() const { return mT1 - mT0; }
 
-   AudacityProject *Project() const { return mProject; }
+   const std::weak_ptr<AudacityProject> &Project() const { return mProject; }
 
    void Clear();
    
    void Assign(
      TrackList && newContents, double t0, double t1,
-     AudacityProject *pProject );
+     const std::weak_ptr<AudacityProject> &pProject );
 
-private:
    Clipboard();
    ~Clipboard();
 
+   void Swap( Clipboard &other );
+
+private:
+
    std::shared_ptr<TrackList> mTracks;
-   AudacityProject *mProject{};
+   std::weak_ptr<AudacityProject> mProject{};
    double mT0{ 0 };
    double mT1{ 0 };
 };
