@@ -15,13 +15,14 @@ information.
 *//*******************************************************************/
 
 
-#include "Audacity.h"
+
 #include "FileFormats.h"
 
 #include <wx/arrstr.h>
 #include <wx/intl.h>
 #include "sndfile.h"
 #include "Internat.h"
+#include "MemoryX.h"
 #include "widgets/AudacityMessageBox.h"
 #include "Prefs.h"
 
@@ -225,6 +226,17 @@ int sf_subtype_bytes_per_sample(unsigned int format){
    return 2;
 }
 
+sampleFormat sf_subtype_to_effective_format(unsigned int format)
+{
+   unsigned int subtype = format & SF_FORMAT_SUBMASK;
+   if (subtype == SF_FORMAT_PCM_24)
+      return int24Sample;
+   else if (sf_subtype_more_than_16_bits(format))
+      return widestSampleFormat;
+   else
+      return int16Sample;
+}
+
 
 FileExtensions sf_get_all_extensions()
 {
@@ -320,7 +332,7 @@ static OSType sf_header_mactype(int format)
 
 #endif // __WXMAC__
 
-ODLock libSndFileMutex;
+//std::mutex libSndFileMutex;
 
 int SFFileCloser::operator() (SNDFILE *sf) const
 {
