@@ -11,6 +11,8 @@
 #ifndef __AUDACITY_HISTORY_WINDOW__
 #define __AUDACITY_HISTORY_WINDOW__
 
+#include "Observer.h"
+#include "Prefs.h"
 #include "widgets/wxPanelWrapper.h" // to inherit
 
 class wxButton;
@@ -18,30 +20,47 @@ class wxListCtrl;
 class wxListEvent;
 class wxSpinCtrl;
 class wxTextCtrl;
+struct AudioIOEvent;
 class AudacityProject;
 class ShuttleGui;
 class UndoManager;
 
-class HistoryDialog final : public wxDialogWrapper {
+class HistoryDialog final : public wxDialogWrapper,
+                            public PrefsListener
+{
 
  public:
    HistoryDialog(AudacityProject * parent, UndoManager *manager);
 
-   void UpdateDisplay(wxEvent &e);
+   void UpdateDisplayForClipboard(wxEvent &);
+   void UpdateDisplay(struct UndoRedoMessage);
+   void DoUpdateDisplay();
    
    bool Show( bool show = true ) override;
 
  private:
-   void OnAudioIO(wxCommandEvent & evt);
+   void Populate(ShuttleGui & S);
+
+   void OnAudioIO(AudioIOEvent);
    void DoUpdate();
    void UpdateLevels();
 
-   void OnChar(wxEvent &event);
+   void OnShow(wxShowEvent & event);
    void OnSize(wxSizeEvent & event);
-   void OnCloseWindow(wxCloseEvent & WXUNUSED(event));
+   void OnCloseWindow(wxCloseEvent & event);
+   void OnListKeyDown(wxKeyEvent & event);
    void OnItemSelected(wxListEvent & event);
    void OnDiscard(wxCommandEvent & event);
    void OnDiscardClipboard(wxCommandEvent & event);
+   void OnCompact(wxCommandEvent & event);
+   void OnGetURL(wxCommandEvent & event);
+
+   // PrefsListener implementation
+   void UpdatePrefs() override;
+
+   Observer::Subscription mAudioIOSubscription
+      , mUndoSubscription
+   ;
 
    AudacityProject   *mProject;
    UndoManager       *mManager;
@@ -51,6 +70,7 @@ class HistoryDialog final : public wxDialogWrapper {
    wxTextCtrl        *mAvail;
    wxSpinCtrl        *mLevels;
    wxButton          *mDiscard;
+   wxButton          *mCompact;
 
    int               mSelected;
    bool              mAudioIOBusy;
