@@ -10,34 +10,32 @@
 #ifndef __AUDACITY_MENUS__
 #define __AUDACITY_MENUS__
 
-#include "audacity/Types.h"
+#include "Identifier.h"
 
 #include <wx/string.h> // member variable
 #include "Prefs.h"
 #include "ClientData.h"
 #include "commands/CommandFlag.h"
+#include "Observer.h"
 
 class wxArrayString;
 class wxCommandEvent;
 class AudacityProject;
 class CommandContext;
 class CommandManager;
-class LabelTrack;
-class PluginDescriptor;
 class Track;
 class TrackList;
 class ViewInfo;
-class WaveClip;
-class WaveTrack;
 
 enum EffectType : int;
 
 typedef wxString PluginID;
+typedef wxString MacroID;
 typedef wxArrayString PluginIDs;
 
 namespace Registry{ class Visitor; }
 
-class MenuCreator
+class AUDACITY_DLL_API MenuCreator
 {
 public:
    MenuCreator();
@@ -51,12 +49,29 @@ public:
    CommandFlag mLastFlags;
    
    // Last effect applied to this project
+   PluginID mLastGenerator{};
    PluginID mLastEffect{};
+   PluginID mLastAnalyzer{};
+   int mLastAnalyzerRegistration;
+   int mLastAnalyzerRegisteredId;
+   PluginID mLastTool{};
+   int mLastToolRegistration;
+   int mLastToolRegisteredId;
+   enum {
+      repeattypenone = 0,
+      repeattypeplugin = 1,
+      repeattypeunique = 2,
+      repeattypeapplymacro = 3
+   };
+   unsigned mRepeatGeneratorFlags;
+   unsigned mRepeatEffectFlags;
+   unsigned mRepeatAnalyzerFlags;
+   unsigned mRepeatToolFlags;
 };
 
 struct ToolbarMenuVisitor;
 
-class MenuManager final
+class AUDACITY_DLL_API MenuManager final
    : public MenuCreator
    , public ClientData::Base
    , private PrefsListener
@@ -99,8 +114,9 @@ private:
    void TellUserWhyDisallowed(const TranslatableString & Name, CommandFlag flagsGot,
       CommandFlag flagsRequired);
 
-   void OnUndoRedo( wxCommandEvent &evt );
+   void OnUndoRedo(struct UndoRedoMessage);
 
+   Observer::Subscription mUndoSubscription;
    AudacityProject &mProject;
 
 public:
