@@ -11,6 +11,7 @@ Paul Licameli split from AudacityProject.h
 #ifndef __AUDACITY_PROJECT_FILE_MANAGER__
 #define __AUDACITY_PROJECT_FILE_MANAGER__
 
+#include <functional>
 #include <memory>
 #include <vector>
 
@@ -20,54 +21,41 @@ Paul Licameli split from AudacityProject.h
 class wxString;
 class wxFileName;
 class AudacityProject;
-class ImportXMLTagHandler;
-class RecordingRecoveryHandler;
 class Track;
 class TrackList;
 class WaveTrack;
 class XMLTagHandler;
-namespace ProjectFileIORegistry{ struct Entry; }
 
 using WaveTrackArray = std::vector < std::shared_ptr < WaveTrack > >;
 using TrackHolders = std::vector< WaveTrackArray >;
 
-class ProjectFileManager final
+class AUDACITY_DLL_API ProjectFileManager final
    : public ClientData::Base
 {
 public:
    static ProjectFileManager &Get( AudacityProject &project );
    static const ProjectFileManager &Get( const AudacityProject &project );
 
+   // Open and close a file, invisibly, removing its Autosave blob
+   static void DiscardAutosave(const FilePath &filename);
+
    explicit ProjectFileManager( AudacityProject &project );
    ProjectFileManager( const ProjectFileManager & ) PROHIBITED;
    ProjectFileManager &operator=( const ProjectFileManager & ) PROHIBITED;
    ~ProjectFileManager();
 
-   struct ReadProjectResults
-   {
-      bool decodeError;
-      bool parseSuccess;
-      bool trackError;
-      TranslatableString errorString;
-      wxString helpUrl;
-   };
-   ReadProjectResults ReadProjectFile( const FilePath &fileName );
+   bool OpenProject();
+   void CloseProject();
+   bool OpenNewProject();
 
-   static unsigned int GetODFlags( const WaveTrack &track );
-   void EnqueueODTasks();
-
-   // To be called when closing a project that has been saved, so that
-   // block files are not erased
-   void CloseLock();
+   void CompactProjectOnClose();
 
    bool Save();
-   bool SaveAs(bool bWantSaveCopy = false, bool bLossless = false);
-   bool SaveAs(const wxString & newFileName, bool bWantSaveCopy = false,
-      bool addToHistory = true);
+   bool SaveAs(bool allowOverwrite = false);
+   bool SaveAs(const FilePath &newFileName, bool addToHistory = true);
    // strProjectPathName is full path for aup except extension
    bool SaveFromTimerRecording( wxFileName fnFile );
-
-   void Reset();
+   bool SaveCopy(const FilePath &fileName = wxT(""));
 
    /** @brief Show an open dialogue for opening audio files, and possibly other
     * sorts of files.
@@ -89,50 +77,60 @@ public:
     * @return Array of file paths which the user selected to open (multiple
     * selections allowed).
     */
-   static wxArrayString ShowOpenDialog(
+   static wxArrayString ShowOpenDialog(FileNames::Operation op,
       const FileNames::FileType &extraType = {});
 
    static bool IsAlreadyOpen(const FilePath &projPathName);
 
-   void OpenFile(const FilePath &fileName, bool addtohistory = true);
+   //! A function that returns a project to use for opening a file; argument is true if opening a project file
+   using ProjectChooserFn = std::function<AudacityProject&(bool)>;
 
-   // If pNewTrackList is passed in non-NULL, it gets filled with the pointers to NEW tracks.
-   bool Import(const FilePath &fileName, WaveTrackArray *pTrackArray = NULL);
+   /*!
+    Opens files of many kinds.  In case of import (sound, MIDI, or .aup), the undo history is pushed.
+    @param chooser told whether opening a project file; decides which project to open into
+    @param fileName the name and contents are examined to decide a type and open appropriately
+    @param addtohistory whether to add .aup3 files to the MRU list (but always done for imports)
+    @return if something was successfully opened, the project containing it; else null
+    */
+   static AudacityProject *OpenFile( const ProjectChooserFn &chooser,
+      const FilePath &fileName, bool addtohistory = true);
 
-   // Takes array of unique pointers; returns array of shared
-   std::vector< std::shared_ptr<Track> >
-   AddImportedTracks(const FilePath &fileName,
+   bool Import(const FilePath &fileName,
+               bool addToHistory = true);
+
+   void Compact();
+
+   void AddImportedTracks(const FilePath &fileName,
                      TrackHolders &&newTracks);
 
    bool GetMenuClose() const { return mMenuClose; }
    void SetMenuClose(bool value) { mMenuClose = value; }
 
-private:   
-   void SetImportedDependencies( bool value ) { mImportedDependencies = value; }
-   
-   // Push names of NEW export files onto the path list
-   bool SaveCopyWaveTracks(const FilePath & strProjectPathName,
-      bool bLossless, FilePaths &strOtherNamesArray);
-   bool DoSave(bool fromSaveAs, bool bWantSaveCopy, bool bLossless = false);
+private:
+   /*!
+    @param fileName a path assumed to exist and contain an .aup3 project
+    @param addtohistory whether to add the file to the MRU list
+    @return if something was successfully opened, the project containing it; else null
+    */
+   AudacityProject *OpenProjectFile(
+      const FilePath &fileName, bool addtohistory);
 
-   // Declared in this class so that they can have access to private members
-   static XMLTagHandler *RecordingRecoveryFactory( AudacityProject &project );
-   static ProjectFileIORegistry::Entry sRecoveryFactory;
-   static XMLTagHandler *ImportHandlerFactory( AudacityProject &project );
-   static ProjectFileIORegistry::Entry sImportHandlerFactory;
+   struct ReadProjectResults
+   {
+      bool parseSuccess;
+      bool trackError;
+      const TranslatableString errorString;
+      wxString helpUrl;
+   };
+   ReadProjectResults ReadProjectFile(
+      const FilePath &fileName, bool discardAutosave = false );
+
+   bool DoSave(const FilePath & fileName, bool fromSaveAs);
 
    AudacityProject &mProject;
 
    std::shared_ptr<TrackList> mLastSavedTracks;
    
-   // The handler that handles recovery of <recordingrecovery> tags
-   std::unique_ptr<RecordingRecoveryHandler> mRecordingRecoveryHandler;
-   
-   std::unique_ptr<ImportXMLTagHandler> mImportXMLTagHandler;
-   
-   // Dependencies have been imported and a warning should be shown on save
-   bool mImportedDependencies{ false };
-   
    // Are we currently closing as the result of a menu command?
    bool mMenuClose{ false };
 };
