@@ -10,17 +10,14 @@ Paul Licameli split from AudacityProject.cpp
 
 #include "ProjectManager.h"
 
-#include "Experimental.h"
 
+
+#include "ActiveProject.h"
 #include "AdornedRulerPanel.h"
 #include "AudioIO.h"
-#include "AutoRecovery.h"
-#include "BlockFile.h"
 #include "Clipboard.h"
-#include "DirManager.h"
 #include "FileNames.h"
 #include "Menus.h"
-#include "MissingAliasFileDialog.h"
 #include "ModuleManager.h"
 #include "Project.h"
 #include "ProjectAudioIO.h"
@@ -29,6 +26,8 @@ Paul Licameli split from AudacityProject.cpp
 #include "ProjectFileManager.h"
 #include "ProjectHistory.h"
 #include "ProjectSelectionManager.h"
+#include "ProjectWindows.h"
+#include "ProjectRate.h"
 #include "ProjectSettings.h"
 #include "ProjectStatus.h"
 #include "ProjectWindow.h"
@@ -40,20 +39,17 @@ Paul Licameli split from AudacityProject.cpp
 #include "wxFileNameWrapper.h"
 #include "import/Import.h"
 #include "import/ImportMIDI.h"
-#include "ondemand/ODManager.h"
-#include "prefs/QualityPrefs.h"
-#include "toolbars/MixerToolBar.h"
+#include "QualitySettings.h"
+#include "toolbars/MeterToolBar.h"
 #include "toolbars/SelectionBar.h"
 #include "toolbars/SpectralSelectionBar.h"
 #include "toolbars/TimeToolBar.h"
 #include "toolbars/ToolManager.h"
 #include "widgets/AudacityMessageBox.h"
 #include "widgets/FileHistory.h"
-#include "widgets/ErrorDialog.h"
 #include "widgets/WindowAccessible.h"
 
-#include <wx/dataobj.h>
-#include <wx/dnd.h>
+#include <wx/app.h>
 #include <wx/scrolbar.h>
 #include <wx/sizer.h>
 
@@ -61,6 +57,22 @@ Paul Licameli split from AudacityProject.cpp
 #include "../images/AudacityLogoAlpha.xpm"
 #endif
 
+namespace {
+   void SaveWindowPreferences(const wxRect& windowRect, const wxRect& normalRect,
+                              bool isMaximized, bool isIconized) {
+      ProjectWindowX.Write(windowRect.GetX());
+      ProjectWindowY.Write(windowRect.GetY());
+      ProjectWindowWidth.Write(windowRect.GetWidth());
+      ProjectWindowHeight.Write(windowRect.GetHeight());
+      ProjectWindowMaximized.Write(isMaximized);
+      ProjectWindowNormalX.Write(normalRect.GetX());
+      ProjectWindowNormalY.Write(normalRect.GetY());
+      ProjectWindowNormalWidth.Write(normalRect.GetWidth());
+      ProjectWindowNormalHeight.Write(normalRect.GetHeight());
+      ProjectWindowIconized.Write(isIconized);
+   }
+}
+
 const int AudacityProjectTimerID = 5200;
 
 static AudacityProject::AttachedObjects::RegisteredFactory sProjectManagerKey {
@@ -85,8 +97,10 @@ ProjectManager::ProjectManager( AudacityProject &project )
 {
    auto &window = ProjectWindow::Get( mProject );
    window.Bind( wxEVT_CLOSE_WINDOW, &ProjectManager::OnCloseWindow, this );
-   mProject.Bind(EVT_PROJECT_STATUS_UPDATE,
-      &ProjectManager::OnStatusChange, this);
+   mSubscription = ProjectStatus::Get(mProject)
+      .Subscribe(*this, &ProjectManager::OnStatusChange);
+   project.Bind( EVT_RECONNECTION_FAILURE,
+      &ProjectManager::OnReconnectionFailure, this );
 }
 
 ProjectManager::~ProjectManager() = default;
@@ -126,23 +140,15 @@ void ProjectManager::SaveWindowSize()
       }
       else
          foundIconizedProject =  TRUE;
-
    }
+
    if (validWindowForSaveWindowSize)
    {
       wxRect windowRect = validProject->GetRect();
       wxRect normalRect = validProject->GetNormalizedWindowState();
       bool wndMaximized = validProject->IsMaximized();
-      gPrefs->Write(wxT("/Window/X"), windowRect.GetX());
-      gPrefs->Write(wxT("/Window/Y"), windowRect.GetY());
-      gPrefs->Write(wxT("/Window/Width"), windowRect.GetWidth());
-      gPrefs->Write(wxT("/Window/Height"), windowRect.GetHeight());
-      gPrefs->Write(wxT("/Window/Maximized"), wndMaximized);
-      gPrefs->Write(wxT("/Window/Normal_X"), normalRect.GetX());
-      gPrefs->Write(wxT("/Window/Normal_Y"), normalRect.GetY());
-      gPrefs->Write(wxT("/Window/Normal_Width"), normalRect.GetWidth());
-      gPrefs->Write(wxT("/Window/Normal_Height"), normalRect.GetHeight());
-      gPrefs->Write(wxT("/Window/Iconized"), FALSE);
+
+      SaveWindowPreferences(windowRect, normalRect, wndMaximized, false);
    }
    else
    {
@@ -150,18 +156,10 @@ void ProjectManager::SaveWindowSize()
          validProject = &ProjectWindow::Get( **AllProjects{}.begin() );
          bool wndMaximized = validProject->IsMaximized();
          wxRect normalRect = validProject->GetNormalizedWindowState();
+
          // store only the normal rectangle because the itemized rectangle
          // makes no sense for an opening project window
-         gPrefs->Write(wxT("/Window/X"), normalRect.GetX());
-         gPrefs->Write(wxT("/Window/Y"), normalRect.GetY());
-         gPrefs->Write(wxT("/Window/Width"), normalRect.GetWidth());
-         gPrefs->Write(wxT("/Window/Height"), normalRect.GetHeight());
-         gPrefs->Write(wxT("/Window/Maximized"), wndMaximized);
-         gPrefs->Write(wxT("/Window/Normal_X"), normalRect.GetX());
-         gPrefs->Write(wxT("/Window/Normal_Y"), normalRect.GetY());
-         gPrefs->Write(wxT("/Window/Normal_Width"), normalRect.GetWidth());
-         gPrefs->Write(wxT("/Window/Normal_Height"), normalRect.GetHeight());
-         gPrefs->Write(wxT("/Window/Iconized"), TRUE);
+         SaveWindowPreferences(normalRect, normalRect, wndMaximized, true);
       }
       else {
          // this would be a very strange case that might possibly occur on the Mac
@@ -169,196 +167,18 @@ void ProjectManager::SaveWindowSize()
          // in this case we are going to write only the default values
          wxRect defWndRect;
          GetDefaultWindowRect(&defWndRect);
-         gPrefs->Write(wxT("/Window/X"), defWndRect.GetX());
-         gPrefs->Write(wxT("/Window/Y"), defWndRect.GetY());
-         gPrefs->Write(wxT("/Window/Width"), defWndRect.GetWidth());
-         gPrefs->Write(wxT("/Window/Height"), defWndRect.GetHeight());
-         gPrefs->Write(wxT("/Window/Maximized"), FALSE);
-         gPrefs->Write(wxT("/Window/Normal_X"), defWndRect.GetX());
-         gPrefs->Write(wxT("/Window/Normal_Y"), defWndRect.GetY());
-         gPrefs->Write(wxT("/Window/Normal_Width"), defWndRect.GetWidth());
-         gPrefs->Write(wxT("/Window/Normal_Height"), defWndRect.GetHeight());
-         gPrefs->Write(wxT("/Window/Iconized"), FALSE);
+         SaveWindowPreferences(defWndRect, defWndRect, false, false);
       }
    }
-   gPrefs->Flush();
    sbWindowRectAlreadySaved = true;
 }
 
-#if wxUSE_DRAG_AND_DROP
-class FileObject final : public wxFileDataObject
-{
-public:
-   FileObject()
-   {
-   }
-
-   bool IsSupportedFormat(const wxDataFormat & format, Direction WXUNUSED(dir = Get)) const
-      // PRL:  This function does NOT override any inherited virtual!  What does it do?
-   {
-      if (format.GetType() == wxDF_FILENAME) {
-         return true;
-      }
-
-#if defined(__WXMAC__)
-#if !wxCHECK_VERSION(3, 0, 0)
-      if (format.GetFormatId() == kDragPromisedFlavorFindFile) {
-         return true;
-      }
-#endif
-#endif
-
-      return false;
-   }
-};
-
-class DropTarget final : public wxFileDropTarget
-{
-public:
-   DropTarget(AudacityProject *proj)
-   {
-      mProject = proj;
-
-      // SetDataObject takes ownership
-      SetDataObject(safenew FileObject());
-   }
-
-   ~DropTarget()
-   {
-   }
-
-#if defined(__WXMAC__)
-#if !wxCHECK_VERSION(3, 0, 0)
-   bool GetData() override
-   {
-      bool foundSupported = false;
-      bool firstFileAdded = false;
-      OSErr result;
-
-      UInt16 items = 0;
-      CountDragItems((DragReference)m_currentDrag, &items);
-
-      for (UInt16 index = 1; index <= items; index++) {
-
-         DragItemRef theItem = 0;
-         GetDragItemReferenceNumber((DragReference)m_currentDrag, index, &theItem);
-
-         UInt16 flavors = 0;
-         CountDragItemFlavors((DragReference)m_currentDrag, theItem , &flavors ) ;
-
-         for (UInt16 flavor = 1 ;flavor <= flavors; flavor++) {
-
-            FlavorType theType = 0;
-            result = GetFlavorType((DragReference)m_currentDrag, theItem, flavor, &theType);
-            if (theType != kDragPromisedFlavorFindFile && theType != kDragFlavorTypeHFS) {
-               continue;
-            }
-            foundSupported = true;
-
-            Size dataSize = 0;
-            GetFlavorDataSize((DragReference)m_currentDrag, theItem, theType, &dataSize);
-
-            ArrayOf<char> theData{ dataSize };
-            GetFlavorData((DragReference)m_currentDrag, theItem, theType, (void*) theData.get(), &dataSize, 0L);
-
-            wxString name;
-            if (theType == kDragPromisedFlavorFindFile) {
-               name = wxMacFSSpec2MacFilename((FSSpec *)theData.get());
-            }
-            else if (theType == kDragFlavorTypeHFS) {
-               name = wxMacFSSpec2MacFilename(&((HFSFlavor *)theData.get())->fileSpec);
-            }
-
-            if (!firstFileAdded) {
-               // reset file list
-               ((wxFileDataObject*)GetDataObject())->SetData(0, "");
-               firstFileAdded = true;
-            }
-
-            ((wxFileDataObject*)GetDataObject())->AddFile(name);
-
-            // We only want to process one flavor
-            break;
-         }
-      }
-      return foundSupported;
-   }
-#endif
-
-   bool OnDrop(wxCoord x, wxCoord y) override
-   {
-      // bool foundSupported = false;
-#if !wxCHECK_VERSION(3, 0, 0)
-      bool firstFileAdded = false;
-      OSErr result;
-
-      UInt16 items = 0;
-      CountDragItems((DragReference)m_currentDrag, &items);
-
-      for (UInt16 index = 1; index <= items; index++) {
-
-         DragItemRef theItem = 0;
-         GetDragItemReferenceNumber((DragReference)m_currentDrag, index, &theItem);
-
-         UInt16 flavors = 0;
-         CountDragItemFlavors((DragReference)m_currentDrag, theItem , &flavors ) ;
-
-         for (UInt16 flavor = 1 ;flavor <= flavors; flavor++) {
-
-            FlavorType theType = 0;
-            result = GetFlavorType((DragReference)m_currentDrag, theItem, flavor, &theType);
-            if (theType != kDragPromisedFlavorFindFile && theType != kDragFlavorTypeHFS) {
-               continue;
-            }
-            return true;
-         }
-      }
-#endif
-      return CurrentDragHasSupportedFormat();
-   }
-
-#endif
-
-   bool OnDropFiles(wxCoord WXUNUSED(x), wxCoord WXUNUSED(y), const wxArrayString& filenames) override
-   {
-      // Experiment shows that this function can be reached while there is no
-      // catch block above in wxWidgets.  So stop all exceptions here.
-      return GuardedCall< bool > ( [&] {
-         //sort by OD non OD.  load Non OD first so user can start editing asap.
-         wxArrayString sortednames(filenames);
-         sortednames.Sort(CompareNoCaseFileName);
-
-         ODManager::Pauser pauser;
-
-         auto cleanup = finally( [&] {
-            ProjectWindow::Get( *mProject ).HandleResize(); // Adjust scrollers for NEW track sizes.
-         } );
-
-         for (const auto &name : sortednames) {
-#ifdef USE_MIDI
-            if (FileNames::IsMidi(name))
-               DoImportMIDI( *mProject, name );
-            else
-#endif
-               ProjectFileManager::Get( *mProject ).Import(name);
-         }
-
-         auto &window = ProjectWindow::Get( *mProject );
-         window.ZoomAfterImport(nullptr);
-
-         return true;
-      } );
-   }
-
-private:
-   AudacityProject *mProject;
-};
-
-#endif
-
 void InitProjectWindow( ProjectWindow &window )
 {
-   auto &project = window.GetProject();
+   auto pProject = window.FindProject();
+   if (!pProject)
+      return;
+   auto &project = *pProject;
 
 #ifdef EXPERIMENTAL_DA2
    SetBackgroundColour(theTheme.Colour( clrMedium ));
@@ -389,18 +209,12 @@ void InitProjectWindow( ProjectWindow &window )
    // Near as I can tell, this is only a problem under Windows.
    //
 
-
    //
    // Create the ToolDock
    //
    ToolManager::Get( project ).CreateWindows();
    ToolManager::Get( project ).LayoutToolBars();
 
-   //
-   // Create the horizontal ruler
-   //
-   auto &ruler = AdornedRulerPanel::Get( project );
-
    //
    // Create the TrackPanel and the scrollbars
    //
@@ -410,22 +224,15 @@ void InitProjectWindow( ProjectWindow &window )
    {
       auto ubs = std::make_unique<wxBoxSizer>(wxVERTICAL);
       ubs->Add( ToolManager::Get( project ).GetTopDock(), 0, wxEXPAND | wxALIGN_TOP );
-      ubs->Add(&ruler, 0, wxEXPAND);
       topPanel->SetSizer(ubs.release());
    }
 
-   // Ensure that the topdock comes before the ruler in the tab order,
-   // irrespective of the order in which they were created.
-   ToolManager::Get(project).GetTopDock()->MoveBeforeInTabOrder(&ruler);
-
-   const auto pPage = window.GetMainPage();
-
    wxBoxSizer *bs;
    {
       auto ubs = std::make_unique<wxBoxSizer>(wxVERTICAL);
       bs = ubs.get();
       bs->Add(topPanel, 0, wxEXPAND | wxALIGN_TOP);
-      bs->Add(pPage, 1, wxEXPAND);
+      bs->Add(window.GetContainerWindow(), 1, wxEXPAND);
       bs->Add( ToolManager::Get( project ).GetBotDock(), 0, wxEXPAND );
       window.SetAutoLayout(true);
       window.SetSizer(ubs.release());
@@ -439,9 +246,16 @@ void InitProjectWindow( ProjectWindow &window )
    //      will be given the focus even if we try to SetFocus().  By
    //      making the TrackPanel that first window, we resolve several
    //      keyboard focus problems.
-   pPage->MoveBeforeInTabOrder(topPanel);
+   window.GetContainerWindow()->MoveAfterInTabOrder(topPanel);
 
-   bs = (wxBoxSizer *)pPage->GetSizer();
+   const auto trackListWindow = window.GetTrackListWindow();
+
+   //
+   // Create the horizontal ruler
+   //
+   auto &ruler = AdornedRulerPanel::Get( project );
+
+   bs = static_cast<wxBoxSizer*>(trackListWindow->GetSizer());
 
    auto vsBar = &window.GetVerticalScrollBar();
    auto hsBar = &window.GetHorizontalScrollBar();
@@ -462,6 +276,7 @@ void InitProjectWindow( ProjectWindow &window )
          hs->Add(vs.release(), 0, wxEXPAND | wxALIGN_TOP);
       }
 
+      bs->Add(&ruler, 0, wxEXPAND | wxALIGN_TOP);
       bs->Add(hs.release(), 1, wxEXPAND | wxALIGN_LEFT | wxALIGN_TOP);
    }
 
@@ -477,16 +292,8 @@ void InitProjectWindow( ProjectWindow &window )
    }
 
    // Lay it out
-   pPage->SetAutoLayout(true);
-   pPage->Layout();
-
-#ifdef EXPERIMENTAL_NOTEBOOK
-   AddPages(this, Factory, pNotebook);
-#endif
-
-   auto mainPanel = window.GetMainPanel();
-
-   mainPanel->Layout();
+   trackListWindow->SetAutoLayout(true);
+   trackListWindow->Layout();
 
    wxASSERT( trackPanel.GetProject() == &project );
 
@@ -534,26 +341,33 @@ AudacityProject *ProjectManager::New()
    
    // Create and show a NEW project
    // Use a non-default deleter in the smart pointer!
-   auto sp = std::make_shared< AudacityProject >();
+   auto sp = AudacityProject::Create();
    AllProjects{}.Add( sp );
    auto p = sp.get();
    auto &project = *p;
    auto &projectHistory = ProjectHistory::Get( project );
    auto &projectManager = Get( project );
    auto &window = ProjectWindow::Get( *p );
+
+   // Issue #2569
+   // There is a dependency on the order of initialisation.
+   // The menus must be created, and registered, before
+   // InitProjectWindows can UpdateMenus.
+   MenuManager::Get(project).CreateMenusAndCommands(project);
+
    InitProjectWindow( window );
 
-   ProjectFileIO::Get( *p ).SetProjectTitle();
-   
-   MissingAliasFilesDialog::SetShouldShow(true);
-   MenuManager::Get( project ).CreateMenusAndCommands( project );
-   
-   projectHistory.InitialState();
-   projectManager.RestartTimer();
-   
    // wxGTK3 seems to need to require creating the window using default position
    // and then manually positioning it.
    window.SetPosition(wndRect.GetPosition());
+
+   auto &projectFileManager = ProjectFileManager::Get( *p );
+
+   // This may report an error.
+   projectFileManager.OpenNewProject();
+
+   projectHistory.InitialState();
+   projectManager.RestartTimer();
    
    if(bMaximized) {
       window.Maximize(true);
@@ -573,16 +387,7 @@ AudacityProject *ProjectManager::New()
    SpectralSelectionBar::Get( project ).SetListener( &projectSelectionManager );
 #endif
    TimeToolBar::Get( project ).SetListener( &projectSelectionManager );
-   
-#if wxUSE_DRAG_AND_DROP
-   // We can import now, so become a drag target
-   //   SetDropTarget(safenew AudacityDropTarget(this));
-   //   mTrackPanel->SetDropTarget(safenew AudacityDropTarget(this));
-   
-   // SetDropTarget takes ownership
-   TrackPanel::Get( project ).SetDropTarget( safenew DropTarget( &project ) );
-#endif
-   
+      
    //Set the NEW project as active:
    SetActiveProject(p);
    
@@ -597,10 +402,21 @@ AudacityProject *ProjectManager::New()
    return p;
 }
 
-// LL: All objects that have a reference to the DirManager should
-//     be deleted before the final mDirManager->Deref() in this
-//     routine.  Failing to do so can cause unwanted recursion
-//     and/or attempts to DELETE objects twice.
+void ProjectManager::OnReconnectionFailure(wxCommandEvent & event)
+{
+   event.Skip();
+   wxTheApp->CallAfter([this]{
+      ProjectWindow::Get(mProject).Close(true);
+   });
+}
+
+static bool sbClosingAll = false;
+
+void ProjectManager::SetClosingAll(bool closing)
+{
+   sbClosingAll = closing;
+}
+
 void ProjectManager::OnCloseWindow(wxCloseEvent & event)
 {
    auto &project = mProject;
@@ -681,7 +497,6 @@ void ProjectManager::OnCloseWindow(wxCloseEvent & event)
          }
       }
    }
-
 #ifdef __WXMAC__
    // Fix bug apparently introduced into 2.1.2 because of wxWidgets 3:
    // closing a project that was made full-screen (as by clicking the green dot
@@ -694,15 +509,14 @@ void ProjectManager::OnCloseWindow(wxCloseEvent & event)
    window.ShowFullScreen(false);
 #endif
 
+   // This achieves auto save on close of project before other important
+   // project state is destroyed
+   window.Publish(ProjectWindowDestroyedMessage {});
    ModuleManager::Get().Dispatch(ProjectClosing);
 
    // Stop the timer since there's no need to update anything anymore
    mTimer.reset();
 
-   // The project is now either saved or the user doesn't want to save it,
-   // so there's no need to keep auto save info around anymore
-   projectFileIO.DeleteCurrentAutoSaveFile();
-
    // DMM: Save the size of the last window the user closes
    //
    // LL: Save before doing anything else to the window that might make
@@ -722,17 +536,48 @@ void ProjectManager::OnCloseWindow(wxCloseEvent & event)
 #endif
 
    // DanH: If we're definitely about to quit, clear the clipboard.
-   //       Doing this after Deref'ing the DirManager causes problems.
+   auto &clipboard = Clipboard::Get();
    if ((AllProjects{}.size() == 1) &&
-      (quitOnClose || AllProjects::Closing()))
-      Clipboard::Get().Clear();
+      (quitOnClose || sbClosingAll))
+      clipboard.Clear();
+   else {
+      auto clipboardProject = clipboard.Project().lock();
+      if ( clipboardProject.get() == &mProject ) {
+         // Closing the project from which content was cut or copied.
+         // For 3.0.0, clear the clipboard, because accessing clipboard contents
+         // would depend on a database connection to the closing project, but
+         // that connection should closed now so that the project file can be
+         // freely moved.
+         // Notes:
+         // 1) maybe clipboard contents could be saved by migrating them to
+         // another temporary database, but that extra effort is beyond the
+         // scope of 3.0.0.
+         // 2) strictly speaking this is necessary only when the clipboard
+         // contains WaveTracks.
+         clipboard.Clear();
+      }
+   }
 
    // JKC: For Win98 and Linux do not detach the menu bar.
    // We want wxWidgets to clean it up for us.
    // TODO: Is there a Mac issue here??
    // SetMenuBar(NULL);
 
-   projectFileManager.CloseLock();
+   // Compact the project.
+   projectFileManager.CompactProjectOnClose();
+
+   // Set (or not) the bypass flag to indicate that deletes that would happen during
+   // the UndoManager::ClearStates() below are not necessary.
+   projectFileIO.SetBypass();
+
+   {
+      // This can reduce reference counts of sample blocks in the project's
+      // tracks.
+      UndoManager::Get( project ).ClearStates();
+
+      // Delete all the tracks to free up memory
+      tracks.Clear();
+   }
 
    // Some of the AdornedRulerPanel functions refer to the TrackPanel, so destroy this
    // before the TrackPanel is destroyed. This change was needed to stop Audacity
@@ -744,42 +589,30 @@ void ProjectManager::OnCloseWindow(wxCloseEvent & event)
    // Check validity of mTrackPanel per bug 584 Comment 1.
    // Deeper fix is in the Import code, but this failsafes against crash.
    TrackPanel::Destroy( project );
-
    // Finalize the tool manager before the children since it needs
    // to save the state of the toolbars.
    ToolManager::Get( project ).Destroy();
 
    window.DestroyChildren();
 
-   TrackFactory::Destroy( project );
+   // Close project only now, because TrackPanel might have been holding
+   // some shared_ptr to WaveTracks keeping SampleBlocks alive.
+   // We're all done with the project file, so close it now
+   projectFileManager.CloseProject();
 
-   // Delete all the tracks to free up memory and DirManager references.
-   tracks.Clear();
-
-   // This must be done before the following Deref() since it holds
-   // references to the DirManager.
-   UndoManager::Get( project ).ClearStates();
-
-   // MM: Tell the DirManager it can now DELETE itself
-   // if it finds it is no longer needed. If it is still
-   // used (f.e. by the clipboard), it will recognize this
-   // and will destroy itself later.
-   //
-   // LL: All objects with references to the DirManager should
-   //     have been deleted before this.
-   DirManager::Destroy( project );
+   WaveTrackFactory::Destroy( project );
 
    // Remove self from the global array, but defer destruction of self
    auto pSelf = AllProjects{}.Remove( project );
    wxASSERT( pSelf );
 
-   if (GetActiveProject() == &project) {
+   if (GetActiveProject().lock().get() == &project) {
       // Find a NEW active project
       if ( !AllProjects{}.empty() ) {
          SetActiveProject(AllProjects{}.begin()->get());
       }
       else {
-         SetActiveProject(NULL);
+         SetActiveProject(nullptr);
       }
    }
 
@@ -788,7 +621,7 @@ void ProjectManager::OnCloseWindow(wxCloseEvent & event)
    // PRL:  Maybe all this is unnecessary now that the listener is managed
    // by a weak pointer.
    if ( gAudioIO->GetListener().get() == &ProjectAudioManager::Get( project ) ) {
-      auto active = GetActiveProject();
+      auto active = GetActiveProject().lock();
       gAudioIO->SetListener(
          active
             ? ProjectAudioManager::Get( *active ).shared_from_this()
@@ -796,7 +629,7 @@ void ProjectManager::OnCloseWindow(wxCloseEvent & event)
       );
    }
 
-   if (AllProjects{}.empty() && !AllProjects::Closing()) {
+   if (AllProjects{}.empty() && !sbClosingAll) {
 
 #if !defined(__WXMAC__)
       if (quitOnClose) {
@@ -820,93 +653,124 @@ void ProjectManager::OnCloseWindow(wxCloseEvent & event)
 
 // PRL: I preserve this handler function for an event that was never sent, but
 // I don't know the intention.
-
 void ProjectManager::OnOpenAudioFile(wxCommandEvent & event)
 {
-   auto &project = mProject;
-   auto &window = GetProjectFrame( project );
    const wxString &cmd = event.GetString();
-
-   if (!cmd.empty())
-      ProjectFileManager::Get( mProject ).OpenFile(cmd);
-
-   window.RequestUserAttention();
+   if (!cmd.empty()) {
+      ProjectChooser chooser{ &mProject, true };
+      if (auto project = ProjectFileManager::OpenFile(
+            std::ref(chooser), cmd)) {
+         auto &window = GetProjectFrame( *project );
+         window.RequestUserAttention();
+         chooser.Commit();
+      }
+   }
 }
 
 // static method, can be called outside of a project
 void ProjectManager::OpenFiles(AudacityProject *proj)
 {
    auto selectedFiles =
-      ProjectFileManager::ShowOpenDialog( FileNames::AudacityProjects );
+      ProjectFileManager::ShowOpenDialog(FileNames::Operation::Open);
    if (selectedFiles.size() == 0) {
       Importer::SetLastOpenType({});
       return;
    }
 
-   //sort selected files by OD status.
-   //For the open menu we load OD first so user can edit asap.
    //first sort selectedFiles.
-   selectedFiles.Sort(CompareNoCaseFileName);
-   ODManager::Pauser pauser;
+   selectedFiles.Sort(FileNames::CompareNoCase);
 
    auto cleanup = finally( [] {
       Importer::SetLastOpenType({});
    } );
 
-   for (size_t ff = 0; ff < selectedFiles.size(); ff++) {
-      const wxString &fileName = selectedFiles[ff];
-
+   for (const auto &fileName : selectedFiles) {
       // Make sure it isn't already open.
       if (ProjectFileManager::IsAlreadyOpen(fileName))
          continue; // Skip ones that are already open.
 
-      FileNames::UpdateDefaultPath(FileNames::Operation::Open, fileName);
-
-      // DMM: If the project is dirty, that means it's been touched at
-      // all, and it's not safe to open a NEW project directly in its
-      // place.  Only if the project is brand-NEW clean and the user
-      // hasn't done any action at all is it safe for Open to take place
-      // inside the current project.
-      //
-      // If you try to Open a NEW project inside the current window when
-      // there are no tracks, but there's an Undo history, etc, then
-      // bad things can happen, including data files moving to the NEW
-      // project directory, etc.
-      if ( proj && (
-         ProjectHistory::Get( *proj ).GetDirty() ||
-         !TrackList::Get( *proj ).empty()
-      ) )
-         proj = nullptr;
-
-      // This project is clean; it's never been touched.  Therefore
-      // all relevant member variables are in their initial state,
-      // and it's okay to open a NEW project inside this window.
-      proj = OpenProject( proj, fileName );
+      proj = OpenProject( proj, fileName,
+         true /* addtohistory */, false /* reuseNonemptyProject */ );
    }
 }
 
-AudacityProject *ProjectManager::OpenProject(
-   AudacityProject *pProject, const FilePath &fileNameArg, bool addtohistory)
+bool ProjectManager::SafeToOpenProjectInto(AudacityProject &proj)
 {
-   AudacityProject *pNewProject = nullptr;
-   if ( ! pProject )
-      pProject = pNewProject = New();
-   auto cleanup = finally( [&] {
-      if( pNewProject )
-         GetProjectFrame( *pNewProject ).Close(true);
-   } );
-   ProjectFileManager::Get( *pProject ).OpenFile( fileNameArg, addtohistory );
-   pNewProject = nullptr;
-   auto &projectFileIO = ProjectFileIO::Get( *pProject );
-   if( projectFileIO.IsRecovered() ) {
-      auto &window = ProjectWindow::Get( *pProject );
-      window.Zoom( window.GetZoomOfToFit() );
-      // "Project was recovered" replaces "Create new project" in Undo History.
-      auto &undoManager = UndoManager::Get( *pProject );
-      undoManager.RemoveStates(1);
-   }
+   // DMM: If the project is dirty, that means it's been touched at
+   // all, and it's not safe to open a fresh project directly in its
+   // place.  Only if the project is brandnew clean and the user
+   // hasn't done any action at all is it safe for Open to take place
+   // inside the current project.
+   //
+   // If you try to Open a fresh project inside the current window when
+   // there are no tracks, but there's an Undo history, etc, then
+   // bad things can happen, including orphan blocks, or tracks
+   // referring to non-existent blocks
+   if (
+      ProjectHistory::Get( proj ).GetDirty() ||
+      !TrackList::Get( proj ).empty()
+   )
+      return false;
+   // This project is clean; it's never been touched.  Therefore
+   // all relevant member variables are in their initial state,
+   // and it's okay to open a NEW project inside its window.
+   return true;
+}
 
-   return pProject;
+ProjectManager::ProjectChooser::~ProjectChooser()
+{
+   if (mpUsedProject) {
+      if (mpUsedProject == mpGivenProject) {
+         // Ensure that it happens here: don't wait for the application level
+         // exception handler, because the exception may be intercepted
+         ProjectHistory::Get(*mpGivenProject).RollbackState();
+         // Any exception now continues propagating
+      }
+      else
+         GetProjectFrame( *mpUsedProject ).Close(true);
+   }
+}
+
+AudacityProject &
+ProjectManager::ProjectChooser::operator() ( bool openingProjectFile )
+{
+   if (mpGivenProject) {
+      // Always check before opening a project file (for safety);
+      // May check even when opening other files
+      // (to preserve old behavior; as with the File > Open command specifying
+      // multiple files of whatever types, so that each gets its own window)
+      bool checkReuse = (openingProjectFile || !mReuseNonemptyProject);
+      if (!checkReuse || SafeToOpenProjectInto(*mpGivenProject))
+         return *(mpUsedProject = mpGivenProject);
+   }
+   return *(mpUsedProject = New());
+}
+
+void ProjectManager::ProjectChooser::Commit()
+{
+   mpUsedProject = nullptr;
+}
+
+AudacityProject *ProjectManager::OpenProject(
+   AudacityProject *pGivenProject, const FilePath &fileNameArg,
+   bool addtohistory, bool reuseNonemptyProject)
+{
+   ProjectManager::ProjectChooser chooser{ pGivenProject, reuseNonemptyProject };
+   if (auto pProject = ProjectFileManager::OpenFile(
+      std::ref(chooser), fileNameArg, addtohistory )) {
+      chooser.Commit();
+
+      auto &projectFileIO = ProjectFileIO::Get( *pProject );
+      if( projectFileIO.IsRecovered() ) {
+         auto &window = ProjectWindow::Get( *pProject );
+         window.Zoom( window.GetZoomOfToFit() );
+         // "Project was recovered" replaces "Create new project" in Undo History.
+         auto &undoManager = UndoManager::Get( *pProject );
+         undoManager.RemoveStates(0, 1);
+      }
+      return pProject;
+   }
+   return nullptr;
 }
 
 // This is done to empty out the tracks, but without creating a new project.
@@ -920,15 +784,14 @@ void ProjectManager::ResetProjectToEmpty() {
    SelectUtilities::DoSelectAll( project );
    TrackUtilities::DoRemoveTracks( project );
 
-   // A new DirManager.
-   DirManager::Reset( project );
-   TrackFactory::Reset( project );
+   WaveTrackFactory::Reset( project );
 
-   projectFileManager.Reset();
+   // InitialState will reset UndoManager
+   projectHistory.InitialState();
+   projectHistory.SetDirty(false);
 
-   projectHistory.SetDirty( false );
-   auto &undoManager = UndoManager::Get( project );
-   undoManager.ClearStates();
+   projectFileManager.CloseProject();
+   projectFileManager.OpenProject();
 }
 
 void ProjectManager::RestartTimer()
@@ -943,15 +806,16 @@ void ProjectManager::OnTimer(wxTimerEvent& WXUNUSED(event))
 {
    auto &project = mProject;
    auto &projectAudioIO = ProjectAudioIO::Get( project );
-   auto &dirManager = DirManager::Get( project );
-   auto mixerToolBar = &MixerToolBar::Get( project );
-   mixerToolBar->UpdateControls();
+   auto meterToolBars = MeterToolBar::GetToolBars( project );
+
+   for (auto& meterToolBar : meterToolBars)
+      meterToolBar.get().UpdateControls();
    
    auto gAudioIO = AudioIO::Get();
    // gAudioIO->GetNumCaptureChannels() should only be positive
    // when we are recording.
    if (projectAudioIO.GetAudioIOToken() > 0 && gAudioIO->GetNumCaptureChannels() > 0) {
-      wxLongLong freeSpace = dirManager.GetFreeDiskSpace();
+      wxLongLong freeSpace = ProjectFileIO::Get(project).GetFreeDiskSpace();
       if (freeSpace >= 0) {
 
          int iRecordingMins = GetEstimatedRecordingMinsLeftOnDisk(gAudioIO->GetNumCaptureChannels());
@@ -962,46 +826,14 @@ void ProjectManager::OnTimer(wxTimerEvent& WXUNUSED(event))
          SetStatusText(sMessage, mainStatusBarField);
       }
    }
-   else if(ODManager::IsInstanceCreated())
-   {
-      //if we have some tasks running, we should say something about it.
-      int numTasks = ODManager::Instance()->GetTotalNumTasks();
-      if(numTasks)
-      {
-         TranslatableString msg;
-         float ratioComplete= ODManager::Instance()->GetOverallPercentComplete();
-
-         if(ratioComplete>=1.0f)
-         {
-            //if we are 100 percent complete and there is still a task in the queue, we should wake the ODManager
-            //so it can clear it.
-            //signal the od task queue loop to wake up so it can remove the tasks from the queue and the queue if it is empty.
-            ODManager::Instance()->SignalTaskQueueLoop();
-
-            msg = XO("On-demand import and waveform calculation complete.");
-         }
-         else if(numTasks>1)
-            msg = XO(
-"Import(s) complete. Running %d on-demand waveform calculations. Overall %2.0f%% complete.")
-               .Format( numTasks, ratioComplete * 100.0 );
-         else
-            msg = XO(
-"Import complete. Running an on-demand waveform calculation. %2.0f%% complete.")
-               .Format( ratioComplete * 100.0 );
-
-         SetStatusText(msg, mainStatusBarField);
-      }
-   }
 
    // As also with the TrackPanel timer:  wxTimer may be unreliable without
    // some restarts
    RestartTimer();
 }
 
-void ProjectManager::OnStatusChange( wxCommandEvent &evt )
+void ProjectManager::OnStatusChange(StatusBarField field)
 {
-   evt.Skip();
-
    auto &project = mProject;
 
    // Be careful to null-check the window.  We might get to this function
@@ -1014,7 +846,6 @@ void ProjectManager::OnStatusChange( wxCommandEvent &evt )
 
    window.UpdateStatusWidths();
 
-   auto field = static_cast<StatusBarField>( evt.GetInt() );
    const auto &msg = ProjectStatus::Get( project ).Get( field );
    SetStatusText( msg, field );
    
@@ -1061,13 +892,12 @@ int ProjectManager::GetEstimatedRecordingMinsLeftOnDisk(long lCaptureChannels) {
    auto &project = mProject;
 
    // Obtain the current settings
-   auto oCaptureFormat = QualityPrefs::SampleFormatChoice();
-   if (lCaptureChannels == 0) {
-      gPrefs->Read(wxT("/AudioIO/RecordChannels"), &lCaptureChannels, 2L);
-   }
+   auto oCaptureFormat = QualitySettings::SampleFormatChoice();
+   if (lCaptureChannels == 0)
+      lCaptureChannels = AudioIORecordChannels.Read();
 
    // Find out how much free space we have on disk
-   wxLongLong lFreeSpace = DirManager::Get( project ).GetFreeDiskSpace();
+   wxLongLong lFreeSpace = ProjectFileIO::Get( project ).GetFreeDiskSpace();
    if (lFreeSpace < 0) {
       return 0;
    }
@@ -1078,7 +908,7 @@ int ProjectManager::GetEstimatedRecordingMinsLeftOnDisk(long lCaptureChannels) {
    dRecTime = lFreeSpace.GetHi() * 4294967296.0 + lFreeSpace.GetLo();
    dRecTime /= bytesOnDiskPerSample;   
    dRecTime /= lCaptureChannels;
-   dRecTime /= ProjectSettings::Get( project ).GetRate();
+   dRecTime /= ProjectRate::Get( project ).GetRate();
 
    // Convert to minutes before returning
    int iRecMins = (int)round(dRecTime / 60.0);
