@@ -10,20 +10,26 @@ Paul Licameli split from AudacityProject.cpp
 
 #include "ProjectSettings.h"
 
-#include "Experimental.h"
+
 
 #include "AudioIOBase.h"
 #include "Project.h"
+#include "QualitySettings.h"
 #include "widgets/NumericTextCtrl.h"
 #include "prefs/TracksBehaviorsPrefs.h"
+#include "XMLWriter.h"
+#include "XMLTagHandler.h"
 
 wxDEFINE_EVENT(EVT_PROJECT_SETTINGS_CHANGE, wxCommandEvent);
 
 namespace {
-   void Notify( AudacityProject &project, ProjectSettings::EventCode code )
+   void Notify(
+      AudacityProject &project, ProjectSettings::EventCode code,
+      long previousValue )
    {
       wxCommandEvent e{ EVT_PROJECT_SETTINGS_CHANGE };
       e.SetInt( static_cast<int>( code ) );
+      e.SetExtraLong( previousValue );
       project.ProcessEvent( e );
    }
 }
@@ -53,10 +59,6 @@ ProjectSettings::ProjectSettings(AudacityProject &project)
       NumericConverter::TIME,
       gPrefs->Read(wxT("/SelectionFormat"), wxT("")))
 }
-, mAudioTimeFormat{ NumericTextCtrl::LookupFormat(
-   NumericConverter::TIME,
-   gPrefs->Read(wxT("/AudioTimeFormat"), wxT("hh:mm:ss")))
-}
 , mFrequencySelectionFormatName{ NumericTextCtrl::LookupFormat(
    NumericConverter::FREQUENCY,
    gPrefs->Read(wxT("/FrequencySelectionFormatName"), wxT("")) )
@@ -65,17 +67,13 @@ ProjectSettings::ProjectSettings(AudacityProject &project)
    NumericConverter::BANDWIDTH,
    gPrefs->Read(wxT("/BandwidthSelectionFormatName"), wxT("")) )
 }
+, mAudioTimeFormat{ NumericTextCtrl::LookupFormat(
+   NumericConverter::TIME,
+   gPrefs->Read(wxT("/AudioTimeFormat"), wxT("hh:mm:ss")))
+}
 , mSnapTo( gPrefs->Read(wxT("/SnapTo"), SNAP_OFF) )
+, mCurrentBrushRadius ( 5 )
 {
-   if (!gPrefs->Read(wxT("/SamplingRate/DefaultProjectSampleRate"), &mRate,
-         AudioIOBase::GetOptimalSupportedSampleRate())) {
-      // The default given above can vary with host/devices. So unless there is
-      // an entry for the default sample rate in audacity.cfg, Audacity can open
-      // with a rate which is different from the rate with which it closed.
-      // See bug 1879.
-      gPrefs->Write(wxT("/SamplingRate/DefaultProjectSampleRate"), mRate);
-      gPrefs->Flush();
-   }
    gPrefs->Read(wxT("/GUI/SyncLockTracks"), &mIsSyncLocked, false);
 
    bool multiToolActive = false;
@@ -92,7 +90,7 @@ ProjectSettings::ProjectSettings(AudacityProject &project)
 void ProjectSettings::UpdatePrefs()
 {
    gPrefs->Read(wxT("/AudioFiles/ShowId3Dialog"), &mShowId3Dialog, true);
-   gPrefs->Read(wxT("/GUI/EmptyCanBeDirty"), &mEmptyCanBeDirty, true );
+   gPrefs->Read(wxT("/GUI/EmptyCanBeDirty"), &mEmptyCanBeDirty, true);
    gPrefs->Read(wxT("/GUI/ShowSplashScreen"), &mShowSplashScreen, true);
    mSoloPref = TracksBehaviorsSolo.Read();
    // Update the old default to the NEW default.
@@ -110,8 +108,7 @@ void ProjectSettings::UpdatePrefs()
    // The DefaultProjectSample rate is the rate for new projects.
    // Do not change this project's rate, unless there are no tracks.
    if( TrackList::Get( *this ).size() == 0){
-      gPrefs->Read(wxT("/SamplingRate/DefaultProjectSampleRate"), &mRate,
-         AudioIOBase::GetOptimalSupportedSampleRate());
+      mRate = QualityDefaultSampleRate.Read();
       // If necessary, we change this rate in the selection toolbar too.
       auto bar = SelectionBar::Get( *this );
       bar.SetRate( mRate );
@@ -163,20 +160,6 @@ const NumericFormatSymbol & ProjectSettings::GetAudioTimeFormat() const
    return mAudioTimeFormat;
 }
 
-double ProjectSettings::GetRate() const
-{
-   return mRate;
-}
-
-void ProjectSettings::SetRate(double rate)
-{
-   auto &project = mProject;
-   if (rate != mRate) {
-      mRate = rate;
-      Notify( project, ChangedProjectRate );
-   }
-}
-
 void ProjectSettings::SetSnapTo(int snap)
 {
    mSnapTo = snap;
@@ -187,6 +170,13 @@ int ProjectSettings::GetSnapTo() const
    return mSnapTo;
 }
 
+void ProjectSettings::SetTool(int tool) {
+   if (auto oldValue = mCurrentTool; oldValue != tool) {
+      mCurrentTool = tool;
+      Notify( mProject, ChangedTool, oldValue );
+   }
+}
+
 bool ProjectSettings::IsSyncLocked() const
 {
 #ifdef EXPERIMENTAL_SYNC_LOCK
@@ -199,9 +189,46 @@ bool ProjectSettings::IsSyncLocked() const
 void ProjectSettings::SetSyncLock(bool flag)
 {
    auto &project = mProject;
-   if (flag != mIsSyncLocked) {
+   if (auto oldValue = mIsSyncLocked; flag != oldValue) {
       mIsSyncLocked = flag;
-      Notify( project, ChangedSyncLock );
+      Notify( project, ChangedSyncLock, oldValue );
    }
 }
 
+static ProjectFileIORegistry::AttributeWriterEntry entry {
+[](const AudacityProject &project, XMLWriter &xmlFile){
+   auto &settings = ProjectSettings::Get(project);
+   xmlFile.WriteAttr(wxT("snapto"), settings.GetSnapTo() ? wxT("on") : wxT("off"));
+   xmlFile.WriteAttr(wxT("selectionformat"),
+                     settings.GetSelectionFormat().Internal());
+   xmlFile.WriteAttr(wxT("frequencyformat"),
+                     settings.GetFrequencySelectionFormatName().Internal());
+   xmlFile.WriteAttr(wxT("bandwidthformat"),
+                     settings.GetBandwidthSelectionFormatName().Internal());
+}
+};
+
+static ProjectFileIORegistry::AttributeReaderEntries entries {
+// Just a pointer to function, but needing overload resolution as non-const:
+(ProjectSettings& (*)(AudacityProject &)) &ProjectSettings::Get, {
+   // PRL:  The following have persisted as per-project settings for long.
+   // Maybe that should be abandoned.  Enough to save changes in the user
+   // preference file.
+   { "snapto", [](auto &settings, auto value){
+      settings.SetSnapTo(value.ToWString() == wxT("on") ? true : false);
+   } },
+   { "selectionformat", [](auto &settings, auto value){
+      settings.SetSelectionFormat(NumericConverter::LookupFormat(
+              NumericConverter::TIME, value.ToWString()));
+   } },
+   { "frequencyformat", [](auto &settings, auto value){
+      settings.SetFrequencySelectionFormatName(
+              NumericConverter::LookupFormat(
+                 NumericConverter::FREQUENCY, value.ToWString()));
+   } },
+   { "bandwidthformat", [](auto &settings, auto value){
+      settings.SetBandwidthSelectionFormatName(
+              NumericConverter::LookupFormat(
+                 NumericConverter::BANDWIDTH, value.ToWString()));
+   } },
+} };
