@@ -16,10 +16,13 @@ Paul Licameli split from AudacityProject.h
 
 #include "ClientData.h" // to inherit
 #include "Prefs.h" // to inherit
+#include "audacity/Types.h"
 
 class AudacityProject;
 
 // Sent to the project when certain settings change
+// See enum EventCode below for values of GetInt() identifying changed setting
+// The previous value of that setting can also be found using GetExtraLong()
 wxDECLARE_EXPORTED_EVENT(AUDACITY_DLL_API,
    EVT_PROJECT_SETTINGS_CHANGE, wxCommandEvent);
 
@@ -32,22 +35,25 @@ enum
 
 namespace ToolCodes {
 enum {
+   // The buttons that are in the Tools toolbar must be in correspondence
+   // with the first few
    selectTool,
    envelopeTool,
    drawTool,
-   zoomTool,
-   slideTool,
    multiTool,
+
+#ifdef EXPERIMENTAL_BRUSH_TOOL
+   brushTool,
+#endif
+
    numTools,
-   
    firstTool = selectTool,
-   lastTool = multiTool,
 };
 }
 
-///\brief Holds various per-project settings values, including the sample rate,
+///\brief Holds various per-project settings values,
 /// and sends events to the project when certain values change
-class ProjectSettings final
+class AUDACITY_DLL_API ProjectSettings final
    : public ClientData::Base
    , private PrefsListener
 {
@@ -58,7 +64,7 @@ public:
    // Values retrievable from GetInt() of the event for settings change
    enum EventCode : int {
       ChangedSyncLock,
-      ChangedProjectRate
+      ChangedTool,
    };
 
    explicit ProjectSettings( AudacityProject &project );
@@ -75,11 +81,6 @@ public:
    bool IsSyncLocked() const;
    void SetSyncLock(bool flag);
    
-   // Rate
-
-   void SetRate(double rate);
-   double GetRate() const;
-
    // Snap To
 
    void SetSnapTo(int snap);
@@ -87,15 +88,19 @@ public:
 
    // Current tool
 
-   void SetTool(int tool) { mCurrentTool = tool; }
+   void SetTool(int tool);
    int GetTool() const { return mCurrentTool; }
 
-   // Speed play
-   double GetPlaySpeed() const {
-      return mPlaySpeed.load( std::memory_order_relaxed ); }
-   void SetPlaySpeed( double value ) {
-      mPlaySpeed.store( value, std::memory_order_relaxed ); }
+   // Current brush radius
+   void SetBrushRadius(int brushRadius) { mCurrentBrushRadius = brushRadius; }
+   int GetBrushRadius() const { return mCurrentBrushRadius; }
 
+   void SetSmartSelection(bool isSelected) { mbSmartSelection = isSelected; }
+   bool IsSmartSelection() const { return mbSmartSelection; }
+
+   void SetOvertones(bool isSelected) { mbOvertones = isSelected; }
+   bool IsOvertones() const { return mbOvertones; }
+   
    // Selection Format
    void SetSelectionFormat(const NumericFormatSymbol & format);
    const NumericFormatSymbol & GetSelectionFormat() const;
@@ -130,15 +135,13 @@ private:
 
    wxString mSoloPref;
 
-   double mRate;
-
-   // This is atomic because scrubber may read it in a separate thread from
-   // the main
-   std::atomic<double> mPlaySpeed{};
-
    int mSnapTo;
 
    int mCurrentTool;
+   int mCurrentBrushRadius;
+   int mCurrentBrushHop;
+   bool mbSmartSelection { false };
+   bool mbOvertones { false };
    
    bool mTracksFitVerticallyZoomed{ false };  //lda
    bool mShowId3Dialog{ true }; //lda
