@@ -6,10 +6,11 @@
 
  **********************************************************************/
 
-#include "Audacity.h"
+
 #include "SelectionState.h"
 
 #include "ViewInfo.h"
+#include "SyncLock.h"
 #include "Track.h"
 #include "Project.h"
 
@@ -36,7 +37,7 @@ void SelectionState::SelectTrackLength
    auto trackRange = syncLocked
    // If we have a sync-lock group and sync-lock linking is on,
    // check the sync-lock group tracks.
-   ? TrackList::SyncLockGroup(&track)
+   ? SyncLock::Group(&track)
 
    // Otherwise, check for a stereo pair
    : TrackList::Channels(&track);
