@@ -11,44 +11,53 @@
 #ifndef __AUDACITY_SEQUENCE__
 #define __AUDACITY_SEQUENCE__
 
+
 #include <vector>
+#include <functional>
 
 #include "SampleFormat.h"
-#include "xml/XMLTagHandler.h"
-#include "ondemand/ODTaskThread.h"
+#include "XMLTagHandler.h"
 
-#include "audacity/Types.h"
+#include "SampleCount.h"
 
-class BlockFile;
-using BlockFilePtr = std::shared_ptr<BlockFile>;
-
-class DirManager;
-class wxFileNameWrapper;
+class SampleBlock;
+class SampleBlockFactory;
+using SampleBlockFactoryPtr = std::shared_ptr<SampleBlockFactory>;
 
 // This is an internal data structure!  For advanced use only.
 class SeqBlock {
  public:
-   BlockFilePtr f;
+   using SampleBlockPtr = std::shared_ptr<SampleBlock>;
+   SampleBlockPtr sb;
    ///the sample in the global wavetrack that this block starts at.
    sampleCount start;
 
    SeqBlock()
-      : f{}, start(0)
+      : sb{}, start(0)
    {}
 
-   SeqBlock(const BlockFilePtr &f_, sampleCount start_)
-      : f(f_), start(start_)
+   SeqBlock(const SampleBlockPtr &sb_, sampleCount start_)
+      : sb(sb_), start(start_)
    {}
 
    // Construct a SeqBlock with changed start, same file
    SeqBlock Plus(sampleCount delta) const
    {
-      return SeqBlock(f, start + delta);
+      return SeqBlock(sb, start + delta);
    }
 };
 class BlockArray : public std::vector<SeqBlock> {};
 using BlockPtrArray = std::vector<SeqBlock*>; // non-owning pointers
 
+// Put extra symbol information in the release build, for the purpose of gathering
+// profiling information (as from Windows Process Monitor), when there otherwise
+// isn't a need for AUDACITY_DLL_API.
+#ifdef IS_ALPHA
+   #define PROFILE_DLL_API AUDACITY_DLL_API
+#else
+   #define PROFILE_DLL_API
+#endif
+
 class PROFILE_DLL_API Sequence final : public XMLTagHandler{
  public:
 
@@ -59,19 +68,18 @@ class PROFILE_DLL_API Sequence final : public XMLTagHandler{
    static void SetMaxDiskBlockSize(size_t bytes);
    static size_t GetMaxDiskBlockSize();
 
+   //! true if nValue is one of the sampleFormat enum values
+   static bool IsValidSampleFormat(const int nValue);
+
    //
    // Constructor / Destructor / Duplicator
    //
 
-   Sequence(const std::shared_ptr<DirManager> &projDirManager, sampleFormat format);
+   Sequence(const SampleBlockFactoryPtr &pFactory, sampleFormat format);
 
-   // The copy constructor and duplicate operators take a
-   // DirManager as a parameter, because you might be copying
-   // from one project to another...
-   Sequence(const Sequence &orig, const std::shared_ptr<DirManager> &projDirManager);
+   Sequence(const Sequence &orig, const SampleBlockFactoryPtr &pFactory);
 
-   // Sequence cannot be copied without specifying a DirManager
-   Sequence(const Sequence&) PROHIBITED;
+   Sequence( const Sequence& ) = delete;
    Sequence& operator= (const Sequence&) PROHIBITED;
 
    ~Sequence();
@@ -87,69 +95,47 @@ class PROFILE_DLL_API Sequence final : public XMLTagHandler{
 
    // Note that len is not size_t, because nullptr may be passed for buffer, in
    // which case, silence is inserted, possibly a large amount.
-   void SetSamples(samplePtr buffer, sampleFormat format,
-            sampleCount start, sampleCount len);
-
-   // where is input, assumed to be nondecreasing, and its size is len + 1.
-   // min, max, rms, bl are outputs, and their lengths are len.
-   // Each position in the output arrays corresponds to one column of pixels.
-   // The column for pixel p covers samples from
-   // where[p] up to (but excluding) where[p + 1].
-   // bl is negative wherever data are not yet available.
-   // Return true if successful.
-   bool GetWaveDisplay(float *min, float *max, float *rms, int* bl,
-                       size_t len, const sampleCount *where) const;
+   void SetSamples(constSamplePtr buffer, sampleFormat format,
+                   sampleCount start, sampleCount len);
 
    // Return non-null, or else throw!
-   std::unique_ptr<Sequence> Copy(sampleCount s0, sampleCount s1) const;
+   // Must pass in the correct factory for the result.  If it's not the same
+   // as in this, then block contents must be copied.
+   std::unique_ptr<Sequence> Copy( const SampleBlockFactoryPtr &pFactory,
+      sampleCount s0, sampleCount s1) const;
    void Paste(sampleCount s0, const Sequence *src);
 
    size_t GetIdealAppendLen() const;
-   void Append(samplePtr buffer, sampleFormat format, size_t len,
-               XMLWriter* blockFileLog=NULL);
+   void Append(constSamplePtr buffer, sampleFormat format, size_t len);
+
+   //! Append data, not coalescing blocks, returning a pointer to the new block.
+   SeqBlock::SampleBlockPtr AppendNewBlock(
+      constSamplePtr buffer, sampleFormat format, size_t len);
+   //! Append a complete block, not coalescing
+   void AppendSharedBlock(const SeqBlock::SampleBlockPtr &pBlock);
    void Delete(sampleCount start, sampleCount len);
 
-   using BlockFileFactory =
-      std::function< BlockFilePtr( wxFileNameWrapper, size_t /* len */ ) >;
-   // An overload of AppendBlockFile that passes the factory to DirManager
-   // which supplies it with a file name
-   void AppendBlockFile( const BlockFileFactory &factory, size_t len );
-
-   // Append a blockfile. The blockfile pointer is then "owned" by the
-   // sequence. This function is used by the recording log crash recovery
-   // code, but may be useful for other purposes. The blockfile must already
-   // be registered within the dir manager hash. This is the case
-   // when the blockfile is created using SimpleBlockFile or
-   // loaded from an XML file via DirManager::HandleXMLTag
-   void AppendBlockFile(const BlockFilePtr &blockFile);
-
    void SetSilence(sampleCount s0, sampleCount len);
    void InsertSilence(sampleCount s0, sampleCount len);
 
-   const std::shared_ptr<DirManager> &GetDirManager() { return mDirManager; }
+   const SampleBlockFactoryPtr &GetFactory() { return mpFactory; }
 
    //
    // XMLTagHandler callback methods for loading and saving
    //
 
-   bool HandleXMLTag(const wxChar *tag, const wxChar **attrs) override;
-   void HandleXMLEndTag(const wxChar *tag) override;
-   XMLTagHandler *HandleXMLChild(const wxChar *tag) override;
+   bool HandleXMLTag(const std::string_view& tag, const AttributesList& attrs) override;
+   void HandleXMLEndTag(const std::string_view& tag) override;
+   XMLTagHandler *HandleXMLChild(const std::string_view& tag) override;
    void WriteXML(XMLWriter &xmlFile) const /* not override */;
 
    bool GetErrorOpening() { return mErrorOpening; }
 
    //
-   // Lock/Unlock all of this sequence's BlockFiles, keeping them
-   // from being moved.  Call this if you want to copy a
-   // track to a different DirManager.  See BlockFile.h
-   // for details.
-   //
+   // Lock all of this sequence's sample blocks, keeping them
+   // from being destroyed when closing.
 
-   bool Lock();
-   bool Unlock();
-
-   bool CloseLock();//similar to Lock but should be called upon project close.
+   bool CloseLock();//should be called upon project close.
    // not balanced by unlocking calls.
 
    //
@@ -159,7 +145,8 @@ class PROFILE_DLL_API Sequence final : public XMLTagHandler{
    sampleFormat GetSampleFormat() const;
 
    // Return true iff there is a change
-   bool ConvertToSampleFormat(sampleFormat format);
+   bool ConvertToSampleFormat(sampleFormat format, 
+      const std::function<void(size_t)> & progressReport = {});
 
    //
    // Retrieving summary info
@@ -189,25 +176,6 @@ class PROFILE_DLL_API Sequence final : public XMLTagHandler{
    BlockArray &GetBlockArray() { return mBlock; }
    const BlockArray &GetBlockArray() const { return mBlock; }
 
-   ///
-   void LockDeleteUpdateMutex(){mDeleteUpdateMutex.Lock();}
-   void UnlockDeleteUpdateMutex(){mDeleteUpdateMutex.Unlock();}
-
-   // RAII idiom wrapping the functions above
-   struct DeleteUpdateMutexLocker {
-      DeleteUpdateMutexLocker(Sequence &sequence)
-         : mSequence(sequence)
-      {
-         mSequence.LockDeleteUpdateMutex();
-      }
-      ~DeleteUpdateMutexLocker()
-      {
-         mSequence.UnlockDeleteUpdateMutex();
-      }
-   private:
-      Sequence &mSequence;
-   };
-
  private:
 
    //
@@ -220,7 +188,7 @@ class PROFILE_DLL_API Sequence final : public XMLTagHandler{
    // Private variables
    //
 
-   std::shared_ptr<DirManager> mDirManager;
+   SampleBlockFactoryPtr mpFactory;
 
    BlockArray    mBlock;
    sampleFormat  mSampleFormat;
@@ -233,32 +201,35 @@ class PROFILE_DLL_API Sequence final : public XMLTagHandler{
 
    bool          mErrorOpening{ false };
 
-   ///To block the Delete() method against the ODCalcSummaryTask::Update() method
-   ODLock   mDeleteUpdateMutex;
-
    //
    // Private methods
    //
 
-   int FindBlock(sampleCount pos) const;
+   SeqBlock::SampleBlockPtr DoAppend(
+      constSamplePtr buffer, sampleFormat format, size_t len, bool coalesce);
 
-   static void AppendBlock
-      (DirManager &dirManager,
-       BlockArray &blocks, sampleCount &numSamples, const SeqBlock &b);
-
-   static bool Read(samplePtr buffer, sampleFormat format,
-             const SeqBlock &b,
-             size_t blockRelativeStart, size_t len, bool mayThrow);
+   static void AppendBlock(SampleBlockFactory *pFactory, sampleFormat format,
+                           BlockArray &blocks,
+                           sampleCount &numSamples,
+                           const SeqBlock &b);
 
    // Accumulate NEW block files onto the end of a block array.
    // Does not change this sequence.  The intent is to use
    // CommitChangesIfConsistent later.
-   static void Blockify
-      (DirManager &dirManager, size_t maxSamples, sampleFormat format,
-       BlockArray &list, sampleCount start, samplePtr buffer, size_t len);
+   static void Blockify(SampleBlockFactory &factory,
+                        size_t maxSamples,
+                        sampleFormat format,
+                        BlockArray &list,
+                        sampleCount start,
+                        constSamplePtr buffer,
+                        size_t len);
 
-   bool Get(int b, samplePtr buffer, sampleFormat format,
-      sampleCount start, size_t len, bool mayThrow) const;
+   bool Get(int b,
+            samplePtr buffer,
+            sampleFormat format,
+            sampleCount start,
+            size_t len,
+            bool mayThrow) const;
 
 public:
 
@@ -266,6 +237,12 @@ public:
    // Public methods
    //
 
+   int FindBlock(sampleCount pos) const;
+
+   static bool Read(samplePtr buffer, sampleFormat format,
+             const SeqBlock &b,
+             size_t blockRelativeStart, size_t len, bool mayThrow);
+
    // This function throws if the track is messed up
    // because of inconsistent block starts & lengths
    void ConsistencyCheck (const wxChar *whereStr, bool mayThrow = true) const;
