@@ -14,15 +14,18 @@
 #ifndef SHUTTLE_GUI
 #define SHUTTLE_GUI
 
-#include "Audacity.h"
-#include "audacity/Types.h"
+
+#include "Identifier.h"
 
 #include <vector>
 #include <wx/slider.h> // to inherit
-#include "MemoryX.h"
 #include <wx/listbase.h> // for wxLIST_FORMAT_LEFT
 
+#include "Prefs.h"
 #include "WrappedType.h"
+#include "ComponentInterfaceSymbol.h"
+
+#include <optional>
 
 class ChoiceSetting;
 
@@ -70,6 +73,7 @@ class wxSpinCtrl;
 class wxListBox;
 class wxGrid;
 class Shuttle;
+class ReadOnlyText;
 
 class WrappedType;
 
@@ -102,20 +106,6 @@ using wxStaticBoxWrapper = wxStaticBox;
 using wxSliderWrapper = wxSlider;
 #endif
 
-template< typename T > class SettingSpec {
-public:
-   SettingSpec( const RegistryPath &path, const T &defaultValue = {} )
-      : mPath{ path }, mDefaultValue{ defaultValue }
-   {}
-
-   const RegistryPath &GetPath() const { return mPath; }
-   const T &GetDefault() const { return mDefaultValue; }
-
-private:
-   RegistryPath mPath;
-   T mDefaultValue;
-};
-
 namespace DialogDefinition {
 
 struct Item {
@@ -186,10 +176,9 @@ struct Item {
       wxEventTypeTag<Tag> eventType,
       void (Handler::*func)(Argument&)
    ) &&
-        -> typename std::enable_if<
-            std::is_base_of<Argument, Tag>::value,
-            Item&&
-        >::type
+        -> std::enable_if_t<
+            std::is_base_of_v<Argument, Tag>,
+            Item&& >
    {
       mRootConnections.push_back({
          eventType,
@@ -267,7 +256,7 @@ public:
    void AddPrompt(const TranslatableString &Prompt, int wrapWidth = 0);
    void AddUnits(const TranslatableString &Prompt, int wrapWidth = 0);
    void AddTitle(const TranslatableString &Prompt, int wrapWidth = 0);
-   wxWindow * AddWindow(wxWindow * pWindow);
+   wxWindow * AddWindow(wxWindow* pWindow, int PositionFlags = wxALIGN_CENTRE);
    wxSlider * AddSlider(
       const TranslatableString &Prompt, int pos, int Max, int Min = 0);
    wxSlider * AddVSlider(const TranslatableString &Prompt, int pos, int Max);
@@ -300,6 +289,9 @@ public:
    wxStaticText * AddVariableText(
       const TranslatableString &Str, bool bCenter = false,
       int PositionFlags = 0, int wrapWidth = 0);
+   ReadOnlyText * AddReadOnlyText(
+      const TranslatableString &Caption,
+      const wxString &Value);
    wxTextCtrl * AddTextBox(
       const TranslatableString &Caption,
       const wxString &Value, const int nChars);
@@ -356,10 +348,14 @@ public:
 //   and create the appropriate widget.
    void StartHorizontalLay(int PositionFlags=wxALIGN_CENTRE, int iProp=1);
    void EndHorizontalLay();
+
    void StartVerticalLay(int iProp=1);
    void StartVerticalLay(int PositionFlags, int iProp);
-
    void EndVerticalLay();
+
+   void StartWrapLay(int PositionFlags=wxEXPAND, int iProp = 0);
+   void EndWrapLay();
+
    wxScrolledWindow * StartScroller(int iStyle=0);
    void EndScroller();
    wxPanel * StartPanel(int iStyle=0);
@@ -387,11 +383,10 @@ public:
 
    void EndNotebookPage();
 
-   wxPanel * StartInvisiblePanel();
+   wxPanel * StartInvisiblePanel(int border = 0);
    void EndInvisiblePanel();
 
-   // SettingName is a key in Preferences.
-   void StartRadioButtonGroup( const ChoiceSetting &Setting );
+   void StartRadioButtonGroup(ChoiceSetting &Setting);
    void EndRadioButtonGroup();
 
    bool DoStep( int iStep );
@@ -446,51 +441,50 @@ public:
 // so it doesn't need an argument that is writeable.
    virtual wxCheckBox * TieCheckBox(
       const TranslatableString &Prompt,
-      const SettingSpec< bool > &Setting);
+      const BoolSetting &Setting);
    virtual wxCheckBox * TieCheckBoxOnRight(
       const TranslatableString &Prompt,
-      const SettingSpec< bool > &Setting);
+      const BoolSetting &Setting);
 
-   virtual wxChoice *TieChoice(
-      const TranslatableString &Prompt,
-      const ChoiceSetting &choiceSetting );
+   virtual wxChoice *TieChoice(const TranslatableString &Prompt,
+      ChoiceSetting &choiceSetting);
 
    // This overload presents what is really a numerical setting as a choice among
    // commonly used values, but the choice is not necessarily exhaustive.
    // This behaves just like the previous for building dialogs, but the
    // behavior is different when the call is intercepted for purposes of
    // emitting scripting information about Preferences.
-   virtual wxChoice * TieNumberAsChoice(
-      const TranslatableString &Prompt,
-      const SettingSpec< int > &Setting,
+   virtual wxChoice * TieNumberAsChoice(const TranslatableString &Prompt,
+      IntSetting &Setting,
       const TranslatableStrings & Choices,
       const std::vector<int> * pInternalChoices = nullptr,
-      int iNoMatchSelector = 0 );
+      int iNoMatchSelector = 0);
 
    virtual wxTextCtrl * TieTextBox(
       const TranslatableString &Prompt,
-      const SettingSpec< wxString > &Setting,
+      const StringSetting &Setting,
       const int nChars);
    virtual wxTextCtrl * TieIntegerTextBox(
       const TranslatableString & Prompt,
-      const SettingSpec< int > &Setting,
+      const IntSetting &Setting,
       const int nChars);
    virtual wxTextCtrl * TieNumericTextBox(
       const TranslatableString & Prompt,
-      const SettingSpec< double > &Setting,
+      const DoubleSetting &Setting,
       const int nChars);
    virtual wxSlider * TieSlider(
       const TranslatableString & Prompt,
-      const SettingSpec< int > &Setting,
+      const IntSetting &Setting,
       const int max,
       const int min = 0);
    virtual wxSpinCtrl * TieSpinCtrl(
       const TranslatableString &Prompt,
-      const SettingSpec< int > &Setting,
+      const IntSetting &Setting,
       const int max,
       const int min);
 //-- End of variants.
    void SetBorder( int Border ) {miBorder = Border;};
+   int GetBorder() const noexcept;
    void SetSizerProportion( int iProp ) {miSizerProp = iProp;};
    void SetStretchyCol( int i );
    void SetStretchyRow( int i );
@@ -575,7 +569,7 @@ private:
 
    std::vector<EnumValueSymbol> mRadioSymbols;
    wxString mRadioSettingName; /// The setting controlled by a group.
-   Optional<WrappedType> mRadioValue;  /// The wrapped value associated with the active radio button.
+   std::optional<WrappedType> mRadioValue;  /// The wrapped value associated with the active radio button.
    int mRadioCount;       /// The index of this radio item.  -1 for none.
    wxString mRadioValueString; /// Unwrapped string value.
    wxRadioButton * DoAddRadioButton(
@@ -703,10 +697,9 @@ public:
       wxEventTypeTag<Tag> eventType,
       void (Handler::*func)(Argument&)
    )
-        -> typename std::enable_if<
-            std::is_base_of<Argument, Tag>::value,
-            ShuttleGui&
-        >::type
+        -> std::enable_if_t<
+            std::is_base_of_v<Argument, Tag>,
+            ShuttleGui& >
    {
       std::move( mItem ).ConnectRoot( eventType, func );
       return *this;
@@ -755,4 +748,11 @@ public:
    teShuttleMode GetMode() { return  mShuttleMode; };
 };
 
+//! Convenience function often useful when adding choice controls
+AUDACITY_DLL_API TranslatableStrings Msgids(
+   const EnumValueSymbol strings[], size_t nStrings);
+
+//! Convenience function often useful when adding choice controls
+AUDACITY_DLL_API TranslatableStrings Msgids( const std::vector<EnumValueSymbol> &strings );
+
 #endif
