@@ -18,11 +18,12 @@
 
 *//*******************************************************************/
 
-#include "Audacity.h"
+
 #include "TimerRecordDialog.h"
 
 #include "FileNames.h"
 
+#include <thread>
 #include <wx/setup.h> // for wxUSE_* macros
 
 #include <wx/wxcrtvararg.h>
@@ -40,11 +41,11 @@
 #include <wx/timer.h>
 #include <wx/dynlib.h> //<! For windows.h
 
-#include "DirManager.h"
+#include "AudioIO.h"
+#include "SelectFile.h"
 #include "ShuttleGui.h"
-#include "MissingAliasFileDialog.h"
-#include "Project.h"
 #include "ProjectAudioManager.h"
+#include "ProjectFileIO.h"
 #include "ProjectFileManager.h"
 #include "ProjectManager.h"
 #include "Prefs.h"
@@ -52,8 +53,8 @@
 #include "widgets/NumericTextCtrl.h"
 #include "widgets/HelpSystem.h"
 #include "widgets/AudacityMessageBox.h"
-#include "widgets/ErrorDialog.h"
 #include "widgets/ProgressDialog.h"
+#include "widgets/wxTextCtrlWrapper.h"
 
 #if wxUSE_ACCESSIBILITY
 #include "widgets/WindowAccessible.h"
@@ -85,7 +86,7 @@ enum {
 const int kSlowTimerInterval = 1000; // ms
 
 // This timer interval is used in some busy-wait loops and is much shorter.
-const int kTimerInterval = 50; // ms
+constexpr auto kTimerInterval = std::chrono::milliseconds{50};
 
 static double wxDateTime_to_AudacityTime(wxDateTime& dateTime)
 {
@@ -186,10 +187,6 @@ TimerRecordDialog::TimerRecordDialog(
 
    m_timer.SetOwner(this, TIMER_ID);
    m_timer.Start(kSlowTimerInterval);
-
-   // Do we need to tidy up when the timer recording has been completed?
-   m_bProjectCleanupRequired = !(this->HaveFilesToRecover());
-
 }
 
 TimerRecordDialog::~TimerRecordDialog()
@@ -305,11 +302,13 @@ void TimerRecordDialog::OnTimeText_Duration(wxCommandEvent& WXUNUSED(event))
 // New events for timer recording automation
 void TimerRecordDialog::OnAutoSavePathButton_Click(wxCommandEvent& WXUNUSED(event))
 {
-   wxString fName = FileNames::SelectFile(FileNames::Operation::Export,
+   auto &projectFileIO = ProjectFileIO::Get(mProject);
+
+   wxString fName = SelectFile(FileNames::Operation::Export,
       XO("Save Timer Recording As"),
       m_fnAutoSaveFile.GetPath(),
       m_fnAutoSaveFile.GetFullName(),
-      wxT("aup"),
+      wxT("aup3"),
       { FileNames::AudacityProjects },
       wxFD_SAVE | wxRESIZE_BORDER,
       this);
@@ -317,11 +316,9 @@ void TimerRecordDialog::OnAutoSavePathButton_Click(wxCommandEvent& WXUNUSED(even
    if (fName.empty())
       return;
 
-   AudacityProject* pProject = &mProject;
-
    // If project already exists then abort - we do not allow users to overwrite an existing project
    // unless it is the current project.
-   if (wxFileExists(fName) && (pProject->GetFileName() != fName)) {
+   if (wxFileExists(fName) && (projectFileIO.GetFileName() != fName)) {
       AudacityMessageDialog m(
          nullptr,
          XO("The selected file name could not be used\nfor Timer Recording because it \
@@ -334,10 +331,10 @@ would overwrite another project.\nPlease try again and select an original name."
 
    // Set this boolean to false so we now do a SaveAs at the end of the recording
    // unless we're saving the current project.
-   m_bProjectAlreadySaved = pProject->GetFileName() == fName? true : false;
+   m_bProjectAlreadySaved = projectFileIO.GetFileName() == fName? true : false;
 
    m_fnAutoSaveFile = fName;
-   m_fnAutoSaveFile.SetExt(wxT("aup"));
+   m_fnAutoSaveFile.SetExt(wxT("aup3"));
    this->UpdateTextBoxControls();
 }
 
@@ -368,7 +365,7 @@ void TimerRecordDialog::OnAutoExportCheckBox_Change(wxCommandEvent& WXUNUSED(eve
 
 void TimerRecordDialog::OnHelpButtonClick(wxCommandEvent& WXUNUSED(event))
 {
-   HelpSystem::ShowHelp(this, wxT("Timer_Record"), true);
+   HelpSystem::ShowHelp(this, L"Timer_Record", true);
 }
 
 void TimerRecordDialog::OnOK(wxCommandEvent& WXUNUSED(event))
@@ -406,13 +403,12 @@ void TimerRecordDialog::OnOK(wxCommandEvent& WXUNUSED(event))
 
    // MY: Estimate here if we have enough disk space to
    // complete this Timer Recording.
-   // If we dont think there is enough space then ask the user
+   // If we don't think there is enough space then ask the user
    // if they want to continue.
    // We don't stop the user from starting the recording 
    // as its possible that they plan to free up some
    // space before the recording begins
-   AudacityProject* pProject = &mProject;
-   auto &projectManager = ProjectManager::Get( *pProject );
+   auto &projectManager = ProjectManager::Get( mProject );
 
    // How many minutes do we have left on the disc?
    int iMinsLeft = projectManager.GetEstimatedRecordingMinsLeftOnDisk();
@@ -481,113 +477,77 @@ void TimerRecordDialog::UpdateTextBoxControls() {
    }
 }
 
-// Copied from AutoRecovery.cpp - for use with Timer Recording Improvements
-bool TimerRecordDialog::HaveFilesToRecover()
-{
-   wxDir dir(FileNames::AutoSaveDir());
-   if (!dir.IsOpened()) {
-      AudacityMessageBox(
-         XO("Could not enumerate files in auto save directory."),
-         XO("Error"),
-         wxICON_STOP);
-      return false;
-   }
-
-   wxString filename;
-   bool c = dir.GetFirst(&filename, wxT("*.autosave"), wxDIR_FILES);
-
-   return c;
-}
-
-bool TimerRecordDialog::RemoveAllAutoSaveFiles()
-{
-   FilePaths files;
-   wxDir::GetAllFiles(FileNames::AutoSaveDir(), &files,
-      wxT("*.autosave"), wxDIR_FILES);
-
-   for (unsigned int i = 0; i < files.size(); i++)
-   {
-      if (!wxRemoveFile(files[i]))
-      {
-         // I don't think this error message is actually useful.
-         // -dmazzoni
-         //AudacityMessageBox(
-         //   XO("Could not remove auto save file: %s)".Format( files[i] ),
-         //   XO("Error"),
-         //   wxICON_STOP);
-         return false;
-      }
-   }
-
-   return true;
-}
-
 /// Runs the wait for start dialog.  Returns -1 if the user clicks stop while we are recording
 /// or if the post recording actions fail.
 int TimerRecordDialog::RunWaitDialog()
 {
-   AudacityProject* pProject = &mProject;
-   
    auto updateResult = ProgressResult::Success;
 
-   if (m_DateTime_Start > wxDateTime::UNow())
-      updateResult = this->WaitForStart();
+   const auto gAudioIO = AudioIO::Get();
+   gAudioIO->DelayActions(true);
+   {
+      auto cleanup = finally([gAudioIO]{ gAudioIO->DelayActions(false); });
 
-   if (updateResult != ProgressResult::Success)  {
-      // Don't proceed, but don't treat it as canceled recording. User just canceled waiting.
-      return POST_TIMER_RECORD_CANCEL_WAIT;
-   } else {
-      // Record for specified time.
-      ProjectAudioManager::Get( *pProject ).OnRecord(false);
-      bool bIsRecording = true;
+      if (m_DateTime_Start > wxDateTime::UNow())
+         updateResult = this->WaitForStart();
 
-      auto sPostAction = Verbatim(
-         m_pTimerAfterCompleteChoiceCtrl->GetStringSelection() );
+      if (updateResult != ProgressResult::Success)  {
+         // Don't proceed, but don't treat it as canceled recording. User just canceled waiting.
+         return POST_TIMER_RECORD_CANCEL_WAIT;
+      } else {
+         // Record for specified time.
+         ProjectAudioManager::Get( mProject ).OnRecord(false);
+         bool bIsRecording = true;
 
-      // Two column layout.
-      TimerProgressDialog::MessageTable columns{
-         {
-            XO("Recording start:") ,
-            XO("Duration:") ,
-            XO("Recording end:") ,
-            {} ,
-            XO("Automatic Save enabled:") ,
-            XO("Automatic Export enabled:") ,
-            XO("Action after Timer Recording:") ,
-         },
-         {
-            GetDisplayDate(m_DateTime_Start) ,
-            Verbatim( m_TimeSpan_Duration.Format() ),
-            GetDisplayDate(m_DateTime_End) ,
-            {} ,
-            (m_bAutoSaveEnabled ? XO("Yes") : XO("No")) ,
-            (m_bAutoExportEnabled ? XO("Yes") : XO("No")) ,
-            sPostAction ,
+         auto sPostAction = Verbatim(
+            m_pTimerAfterCompleteChoiceCtrl->GetStringSelection() );
+
+         // Two column layout.
+         TimerProgressDialog::MessageTable columns{
+            {
+               XO("Recording start:") ,
+               XO("Duration:") ,
+               XO("Recording end:") ,
+               {} ,
+               XO("Automatic Save enabled:") ,
+               XO("Automatic Export enabled:") ,
+               XO("Action after Timer Recording:") ,
+            },
+            {
+               GetDisplayDate(m_DateTime_Start) ,
+               Verbatim( m_TimeSpan_Duration.Format() ),
+               GetDisplayDate(m_DateTime_End) ,
+               {} ,
+               (m_bAutoSaveEnabled ? XO("Yes") : XO("No")) ,
+               (m_bAutoExportEnabled ? XO("Yes") : XO("No")) ,
+               sPostAction ,
+            }
+         };
+
+         TimerProgressDialog
+            progress(m_TimeSpan_Duration.GetMilliseconds().GetValue(),
+                     XO("Audacity Timer Record Progress"),
+                     columns,
+                     pdlgHideCancelButton | pdlgConfirmStopCancel);
+
+         // Make sure that start and end time are updated, so we always get the full
+         // duration, even if there's some delay getting here.
+         wxTimerEvent dummyTimerEvent;
+         this->OnTimer(dummyTimerEvent);
+
+         // Loop for progress display during recording.
+         while (bIsRecording && (updateResult == ProgressResult::Success)) {
+            updateResult = progress.UpdateProgress();
+            using namespace std::chrono;
+            std::this_thread::sleep_for(kTimerInterval);
+            bIsRecording = (wxDateTime::UNow() <= m_DateTime_End); // Call UNow() again for extra accuracy...
          }
-      };
-
-      TimerProgressDialog
-         progress(m_TimeSpan_Duration.GetMilliseconds().GetValue(),
-                  XO("Audacity Timer Record Progress"),
-                  columns,
-                  pdlgHideCancelButton | pdlgConfirmStopCancel);
-
-      // Make sure that start and end time are updated, so we always get the full
-      // duration, even if there's some delay getting here.
-      wxTimerEvent dummyTimerEvent;
-      this->OnTimer(dummyTimerEvent);
-
-      // Loop for progress display during recording.
-      while (bIsRecording && (updateResult == ProgressResult::Success)) {
-         updateResult = progress.UpdateProgress();
-         wxMilliSleep(kTimerInterval);
-         bIsRecording = (wxDateTime::UNow() <= m_DateTime_End); // Call UNow() again for extra accuracy...
       }
    }
 
    // Must do this AFTER the timer project dialog has been deleted to ensure the application
    // responds to the AUDIOIO events...see not about bug #334 in the ProgressDialog constructor.
-   ProjectAudioManager::Get( *pProject ).Stop();
+   ProjectAudioManager::Get( mProject ).Stop();
 
    // Let the caller handle cancellation or failure from recording progress.
    if (updateResult == ProgressResult::Cancelled || updateResult == ProgressResult::Failed)
@@ -607,8 +567,6 @@ int TimerRecordDialog::ExecutePostRecordActions(bool bWasStopped) {
    // Finally, if there is no post-record action selected then we output
    // a dialog detailing what has been carried out instead.
 
-   AudacityProject* pProject = &mProject;
-
    bool bSaveOK = false;
    bool bExportOK = false;
    int iPostRecordAction = m_pTimerAfterCompleteChoiceCtrl->GetSelection();
@@ -618,7 +576,7 @@ int TimerRecordDialog::ExecutePostRecordActions(bool bWasStopped) {
    // Do Automatic Save?
    if (m_bAutoSaveEnabled) {
 
-      auto &projectFileManager = ProjectFileManager::Get( *pProject );
+      auto &projectFileManager = ProjectFileManager::Get( mProject );
       // MY: If this project has already been saved then simply execute a Save here
       if (m_bProjectAlreadySaved) {
          bSaveOK = projectFileManager.Save();
@@ -630,9 +588,8 @@ int TimerRecordDialog::ExecutePostRecordActions(bool bWasStopped) {
    // Do Automatic Export?
    if (m_bAutoExportEnabled) {
       Exporter e{ mProject };
-      MissingAliasFilesDialog::SetShouldShow(true);
       bExportOK = e.ProcessFromTimerRecording(
-         false, 0.0, TrackList::Get( *pProject ).GetEndTime(),
+         false, 0.0, TrackList::Get( mProject ).GetEndTime(),
             m_fnAutoExportFile, m_iAutoExportFormat,
             m_iAutoExportSubFormat, m_iAutoExportFilterIndex);
    }
@@ -716,31 +673,11 @@ int TimerRecordDialog::ExecutePostRecordActions(bool bWasStopped) {
          if (iDelayOutcome != ProgressResult::Success) {
             // Cancel the action!
             iPostRecordAction = POST_TIMER_RECORD_NOTHING;
-            // Set this to true to avoid any chance of the temp files being deleted
-            bErrorOverride = true;
             break;
          }
-
-
-         // If we have simply recorded, exported and then plan to Exit/Restart/Shutdown
-         // then we will have a temporary project setup.  Let's get rid of that!
-         if (m_bAutoExportEnabled && !m_bAutoSaveEnabled) {
-            // PRL:  Move the following cleanup into a finally?
-            // No, I think you would want to skip this, in case recording
-            // succeeded but then save or export threw an exception.
-            DirManager::CleanTempDir();
-         }
       } while (false);
    }
 
-   // Do we need to cleanup the orphaned temporary project?
-   if (m_bProjectCleanupRequired && !bErrorOverride) {
-      // PRL:  Move the following cleanup into a finally?
-      // No, I think you would want to skip this, in case recording
-      // succeeded but then save or export threw an exception.
-      RemoveAllAutoSaveFiles();
-   }
-
    // Return the action as required
    return iPostRecordAction;
 }
@@ -794,13 +731,13 @@ wxPrintf(wxT("%s\n"), dt.Format());
    return Verbatim( dt.FormatDate() + wxT(" ") + dt.FormatTime() );
 }
 
-TimerRecordPathCtrl * TimerRecordDialog::NewPathControl(
+wxTextCtrlWrapper * TimerRecordDialog::NewPathControl(
    wxWindow *wParent, const int iID,
    const TranslatableString &sCaption, const TranslatableString &sValue)
 {
-   TimerRecordPathCtrl * pTextCtrl;
+   wxTextCtrlWrapper * pTextCtrl;
    wxASSERT(wParent); // to justify safenew
-   pTextCtrl = safenew TimerRecordPathCtrl(wParent, iID, sValue);
+   pTextCtrl = safenew wxTextCtrlWrapper(wParent, iID, sValue.Translation());
    pTextCtrl->SetName(sCaption.Translation());
    return pTextCtrl;
 }
@@ -917,8 +854,7 @@ void TimerRecordDialog::PopulateOrExchange(ShuttleGui& S)
             S.StartMultiColumn(3, wxEXPAND);
             {
                TranslatableString sInitialValue;
-               AudacityProject* pProject = &mProject;
-               auto sSaveValue = pProject->GetFileName();
+               auto sSaveValue = ProjectFileIO::Get(mProject).GetFileName();
                if (!sSaveValue.empty()) {
                   m_fnAutoSaveFile.Assign(sSaveValue);
                   sInitialValue = XO("Current Project");
@@ -927,7 +863,7 @@ void TimerRecordDialog::PopulateOrExchange(ShuttleGui& S)
                m_pTimerSavePathTextCtrl = NewPathControl(
                   S.GetParent(), ID_AUTOSAVEPATH_TEXT,
                   XO("Save Project As:"), sInitialValue);
-               m_pTimerSavePathTextCtrl->SetEditable(false);
+               m_pTimerSavePathTextCtrl->SetReadOnly(true);
                S.AddWindow(m_pTimerSavePathTextCtrl);
                m_pTimerSavePathButtonCtrl = S.Id(ID_AUTOSAVEPATH_BUTTON).AddButton(XXO("Select..."));
                }
@@ -944,7 +880,7 @@ void TimerRecordDialog::PopulateOrExchange(ShuttleGui& S)
                m_pTimerExportPathTextCtrl = NewPathControl(
                   S.GetParent(), ID_AUTOEXPORTPATH_TEXT,
                   XO("Export Project As:"), {});
-               m_pTimerExportPathTextCtrl->SetEditable(false);
+               m_pTimerExportPathTextCtrl->SetReadOnly(true);
                S.AddWindow(m_pTimerExportPathTextCtrl);
                m_pTimerExportPathButtonCtrl = S.Id(ID_AUTOEXPORTPATH_BUTTON).AddButton(XXO("Select..."));
             }
@@ -1102,7 +1038,8 @@ ProgressResult TimerRecordDialog::WaitForStart()
    while (updateResult == ProgressResult::Success && !bIsRecording)
    {
       updateResult = progress.UpdateProgress();
-      wxMilliSleep(kTimerInterval);
+      using namespace std::chrono;
+      std::this_thread::sleep_for(kTimerInterval);
       bIsRecording = (m_DateTime_Start <= wxDateTime::UNow());
    }
    return updateResult;
@@ -1152,7 +1089,8 @@ ProgressResult TimerRecordDialog::PreActionDelay(int iActionIndex, TimerRecordCo
    while (iUpdateResult == ProgressResult::Success && !bIsTime)
    {
       iUpdateResult = dlgAction.UpdateProgress();
-      wxMilliSleep(kTimerInterval);
+      using namespace std::chrono;
+      std::this_thread::sleep_for(kTimerInterval);
       bIsTime = (dtActionTime <= wxDateTime::UNow());
    }
    return iUpdateResult;
