@@ -18,8 +18,8 @@
 #ifndef __AUDACITY_TRACKARTIST__
 #define __AUDACITY_TRACKARTIST__
 
-#include "Audacity.h" // for USE_* macros
-#include "Experimental.h"
+
+
 
 #include <wx/brush.h> // member variable
 #include <wx/pen.h> // member variables
@@ -36,22 +36,6 @@ class TrackPanel;
 struct TrackPanelDrawingContext;
 class ZoomInfo;
 
-namespace TrackArt {
-
-   // Helper: draws the "sync-locked" watermark tiled to a rectangle
-   void DrawSyncLockTiles(
-      TrackPanelDrawingContext &context, const wxRect &rect );
-
-   // Helper: draws background with selection rect
-   void DrawBackgroundWithSelection(TrackPanelDrawingContext &context,
-         const wxRect &rect, const Track *track,
-         const wxBrush &selBrush, const wxBrush &unselBrush,
-         bool useSelection = true);
-
-   void DrawNegativeOffsetTrackArrows( TrackPanelDrawingContext &context,
-                                       const wxRect & rect );
-}
-
 class AUDACITY_DLL_API TrackArtist final : private PrefsListener {
 
 public:
@@ -134,14 +118,8 @@ public:
    bool drawEnvelope{ false };
    bool bigPoints{ false };
    bool drawSliders{ false };
+   bool onBrushTool{ false };
    bool hasSolo{ false };
 };
 
-extern int GetWaveYPos(float value, float min, float max,
-                       int height, bool dB, bool outer, float dBr,
-                       bool clip);
-extern float FromDB(float value, double dBRange);
-extern float ValueOfPixel(int yy, int height, bool offset,
-                          bool dB, double dBRange, float zoomMin, float zoomMax);
-
 #endif                          // define __AUDACITY_TRACKARTIST__
