@@ -25,11 +25,10 @@ Paul Licameli split from TrackPanel.cpp
 
 **********************************************************************/
 
-#include "Audacity.h"
+
 #include "TrackInfo.h"
 
-#include "Experimental.h"
-
+#include <wx/app.h>
 #include <wx/dc.h>
 #include <wx/frame.h>
 
@@ -37,6 +36,7 @@ Paul Licameli split from TrackPanel.cpp
 #include "AllThemeResources.h"
 #include "Prefs.h"
 #include "Project.h"
+#include "SyncLock.h"
 #include "Theme.h"
 #include "Track.h"
 #include "TrackPanelDrawingContext.h"
@@ -202,7 +202,7 @@ unsigned TrackInfo::MinimumTrackHeight()
       height += commonTrackTCPBottomLines.front().height;
    // + 1 prevents the top item from disappearing for want of enough space,
    // according to the rules in HideTopItem.
-   return height + kTopMargin + kBottomMargin + 1;
+   return height + kVerticalPadding + 1;
 }
 
 bool TrackInfo::HideTopItem( const wxRect &rect, const wxRect &subRect,
@@ -377,7 +377,7 @@ void TrackInfo::MinimizeSyncLockDrawFunction
 {
    auto dc = &context.dc;
    bool selected = pTrack ? pTrack->GetSelected() : true;
-   bool syncLockSelected = pTrack ? pTrack->IsSyncLockSelected() : true;
+   bool syncLockSelected = pTrack ? SyncLock::IsSyncLockSelected(pTrack) : true;
    bool minimized =
       pTrack ? TrackView::Get( *pTrack ).GetMinimized() : false;
    {
@@ -566,7 +566,7 @@ void TrackInfo::SetTrackInfoFont(wxDC * dc)
 unsigned TrackInfo::DefaultTrackHeight( const TCPLines &topLines )
 {
    int needed =
-      kTopMargin + kBottomMargin +
+      kVerticalPadding +
       totalTCPLines( topLines, true ) +
       totalTCPLines( commonTrackTCPBottomLines, false ) + 1;
    return (unsigned) std::max( needed, (int) TrackView::DefaultHeight );
