@@ -11,12 +11,11 @@
 #ifndef __AUDACITY_TRACK_PANEL_ACCESSIBILITY__
 #define __AUDACITY_TRACK_PANEL_ACCESSIBILITY__
 
-#include "Audacity.h"
+
 
 #include <functional>
 #include <memory>
 
-#include <wx/event.h> // to declare custom event types
 #include <wx/setup.h> // for wxUSE_* macros
 
 #include <wx/string.h> // member variable
@@ -26,17 +25,15 @@
 #endif
 
 #include "ClientData.h" // to inherit
+#include "Observer.h"
 
 class wxRect;
+class wxWindow;
 
 class AudacityProject;
 class Track;
 class TrackList;
 
-// An event sent to the project
-wxDECLARE_EXPORTED_EVENT(AUDACITY_DLL_API,
-                         EVT_TRACK_FOCUS_CHANGE, wxCommandEvent);
-
 class TrackPanelAx final
 #if wxUSE_ACCESSIBILITY
    : public WindowAccessible
@@ -156,8 +153,12 @@ private:
    int mMessageCount;
 };
 
-class TrackFocus final
+struct TrackFocusChangeMessage {};
+
+class AUDACITY_DLL_API TrackFocus final
    : public ClientData::Base
+   , public Observer::Publisher<TrackFocusChangeMessage>
+   , public std::enable_shared_from_this<TrackFocus>
 {
 public:
    static TrackFocus &Get( AudacityProject &project );
@@ -191,6 +192,7 @@ public:
    void UpdateAccessibility();
 
 private:
+   friend TrackPanelAx; // so it can Publish()
 
    AudacityProject &mProject;
 
