@@ -8,7 +8,7 @@ Paul Licameli
 
 **********************************************************************/
 
-#include "Audacity.h"
+
 #include "UIHandle.h"
 
 #include "RefreshCode.h"
@@ -31,7 +31,7 @@ bool UIHandle::Rotate(bool)
    return false;
 }
 
-bool UIHandle::HasEscape() const
+bool UIHandle::HasEscape(AudacityProject *) const
 {
    return false;
 }
@@ -41,6 +41,11 @@ bool UIHandle::Escape(AudacityProject *)
    return false;
 }
 
+bool UIHandle::HandlesRightClick()
+{
+   return false;
+}
+
 bool UIHandle::StopsOnKeystroke()
 {
    return false;
