@@ -11,7 +11,7 @@
 #ifndef __AUDACITY_COMMAND__
 #define __AUDACITY_COMMAND__
 
-#include "../Audacity.h"
+
 
 #include <set>
 
@@ -20,10 +20,11 @@
 
 #include "../widgets/wxPanelWrapper.h" // to inherit
 
-#include "../include/audacity/ComponentInterface.h"
-#include "../include/audacity/EffectAutomationParameters.h" // for command automation
+#include "ComponentInterface.h"
+#include "EffectAutomationParameters.h"
+#include "EffectInterface.h" // for SettingsVisitor type alias
 
-#include "../Registrar.h"
+#include "Registrar.h"
 
 class ShuttleGui;
 
@@ -32,7 +33,6 @@ class ShuttleGui;
 class AudacityCommand;
 class AudacityProject;
 class CommandContext;
-class EffectUIHostInterface;
 class ProgressDialog;
 
 
@@ -46,38 +46,32 @@ class AUDACITY_DLL_API AudacityCommand /* not final */ : public wxEvtHandler,
    AudacityCommand();
    virtual ~AudacityCommand();
    
-   // Type of a registered function that, if it returns true,
-   // causes ShowInterface to return early without making any dialog
-   using VetoDialogHook = bool (*) ( wxDialog* );
-   static VetoDialogHook SetVetoDialogHook( VetoDialogHook hook );
-
    // ComponentInterface implementation
 
    //These four can be defaulted....
-   PluginPath GetPath() override;
-   VendorSymbol GetVendor() override;
-   wxString GetVersion() override;
+   PluginPath GetPath() const override;
+   VendorSymbol GetVendor() const override;
+   wxString GetVersion() const override;
    //  virtual wxString GetFamily();
 
    //These two must be implemented by instances.
-   ComponentInterfaceSymbol GetSymbol() override = 0;
-   virtual TranslatableString GetDescription() override
+   ComponentInterfaceSymbol GetSymbol() const override = 0;
+   virtual TranslatableString GetDescription() const override
    {wxFAIL_MSG( "Implement a Description for this command");return XO("FAIL");};
 
    // Name of page in the Audacity alpha manual
-   virtual wxString ManualPage(){ return wxEmptyString;};
-   virtual bool IsBatchProcessing(){ return mIsBatch;}
-   virtual void SetBatchProcessing(bool start){ mIsBatch = start;};
+   virtual ManualPageID ManualPage() { return {}; }
+   virtual bool IsBatchProcessing() const { return mIsBatch; }
+   virtual void SetBatchProcessing(bool start) { mIsBatch = start; }
    
-   virtual bool Apply(const CommandContext & WXUNUSED(context) ) {return false;};
+   virtual bool Apply(const CommandContext & WXUNUSED(context) ) { return false; }
 
    bool ShowInterface(wxWindow *parent, bool forceModal = false);
-   virtual void SetHostUI(EffectUIHostInterface * WXUNUSED(host)){;};
 
    wxDialog *CreateUI(wxWindow *parent, AudacityCommand *client);
 
-   virtual bool GetAutomationParameters(wxString & parms);
-   virtual bool SetAutomationParameters(const wxString & parms);
+   bool SaveSettingsAsString(wxString & parms);
+   bool LoadSettingsFromString(const wxString & parms);
 
    bool DoAudacityCommand(wxWindow *parent, const CommandContext & context,bool shouldPrompt = true);
 
@@ -109,7 +103,7 @@ class AUDACITY_DLL_API AudacityCommand /* not final */ : public wxEvtHandler,
    virtual bool PromptUser(wxWindow *parent);
 
    // Check whether command should be skipped
-   // Typically this is only useful in automation, for example
+   // Typically this is only useful in macros, for example
    // detecting that zero noise reduction is to be done,
    // or that normalisation is being done without Dc bias shift
    // or amplitude modification
@@ -123,17 +117,22 @@ class AUDACITY_DLL_API AudacityCommand /* not final */ : public wxEvtHandler,
    virtual bool TransferDataToWindow();
    virtual bool TransferDataFromWindow();
 
+   //! Visit settings, if defined.  false means no defined settings.
+   //! Default implementation returns false
+   virtual bool VisitSettings( SettingsVisitor & );
+   //! Visit settings, if defined.  false means no defined settings.
+   //! Default implementation returns false
+   virtual bool VisitSettings( ConstSettingsVisitor & );
+
 protected:
 
    ProgressDialog *mProgress; // Temporary pointer, NOT deleted in destructor.
    // UI
    wxDialog       *mUIDialog;
    wxWindow       *mUIParent;
-   int             mUIResultID;
 
 private:
    bool mIsBatch;
-   bool mUIDebug;
    bool mNeedsInit;
 };
 
