@@ -25,14 +25,15 @@ class wxString;
 class wxVariant;
 class Validator;
 
-class CommandSignature
+class AUDACITY_DLL_API CommandSignature
 {
 private:
    ParamValueMap mDefaults;
    ValidatorMap mValidators;
-   explicit CommandSignature(const CommandSignature & WXUNUSED(other)) { }
+   CommandSignature(const CommandSignature &) PROHIBITED;
+   CommandSignature& operator=(const CommandSignature &) PROHIBITED;
 public:
-   explicit CommandSignature() { }
+   explicit CommandSignature();
    ~CommandSignature();
 
    // Add a parameter to the signature.
