@@ -19,7 +19,7 @@ parameters.  It is for development purposes.
 
 *//*******************************************************************/
 
-#include "../Audacity.h"
+
 #include "Demo.h"
 #include "LoadCommands.h"
 
@@ -39,12 +39,19 @@ const ComponentInterfaceSymbol DemoCommand::Symbol
 //Don't register the demo command.  
 //namespace{ BuiltinCommandsModule::Registration< DemoCommand > reg; }
 
-bool DemoCommand::DefineParams( ShuttleParams & S ){
+template<bool Const>
+bool DemoCommand::VisitSettings( SettingsVisitorBase<Const> & S ){
    S.Define( delay, wxT("Delay"), 1.0f, 0.001f,  FLT_MAX, 1.0f );
    S.Define( decay, wxT("Decay"), 0.5f, 0.0f,    FLT_MAX, 1.0f  );
    return true;
 }
 
+bool DemoCommand::VisitSettings( SettingsVisitor & S )
+   { return VisitSettings<false>(S); }
+
+bool DemoCommand::VisitSettings( ConstSettingsVisitor & S )
+   { return VisitSettings<true>(S); }
+
 bool DemoCommand::Apply(const CommandContext & context){
    context.Status( "A Message");
    return true;
