@@ -20,11 +20,12 @@ This class now lists
 
 *//*******************************************************************/
 
-#include "../Audacity.h" // for USE_* macros
+
 #include "GetInfoCommand.h"
 
 #include "LoadCommands.h"
-#include "../Project.h"
+#include "Project.h"
+#include "../ProjectWindows.h"
 #include "CommandManager.h"
 #include "CommandTargets.h"
 #include "../effects/EffectManager.h"
@@ -32,10 +33,12 @@ This class now lists
 #include "../TrackPanelAx.h"
 #include "../TrackPanel.h"
 #include "../WaveClip.h"
-#include "../ViewInfo.h"
+#include "ViewInfo.h"
 #include "../WaveTrack.h"
 #include "../LabelTrack.h"
-#include "../Envelope.h"
+#include "../NoteTrack.h"
+#include "../TimeTrack.h"
+#include "Envelope.h"
 
 #include "SelectCommand.h"
 #include "../ShuttleGui.h"
@@ -43,11 +46,12 @@ This class now lists
 
 #include "../prefs/PrefsDialog.h"
 #include "../Shuttle.h"
-#include "../PluginManager.h"
+#include "PluginManager.h"
 #include "../tracks/ui/TrackView.h"
 #include "../ShuttleGui.h"
 
 #include <wx/frame.h>
+#include <wx/log.h>
 #include <wx/menu.h>
 
 const ComponentInterfaceSymbol GetInfoCommand::Symbol
@@ -98,23 +102,28 @@ static const EnumValueSymbol kFormats[nFormats] =
    { XO("Brief") }
 };
 
-
-
-bool GetInfoCommand::DefineParams( ShuttleParams & S ){
+template<bool Const>
+bool GetInfoCommand::VisitSettings( SettingsVisitorBase<Const> & S ){
    S.DefineEnum( mInfoType, wxT("Type"), 0, kTypes, nTypes );
    S.DefineEnum( mFormat, wxT("Format"), 0, kFormats, nFormats );
    return true;
 }
 
+bool GetInfoCommand::VisitSettings( SettingsVisitor & S )
+   { return VisitSettings<false>(S); }
+
+bool GetInfoCommand::VisitSettings( ConstSettingsVisitor & S )
+   { return VisitSettings<true>(S); }
+
 void GetInfoCommand::PopulateOrExchange(ShuttleGui & S)
 {
    S.AddSpace(0, 5);
 
    S.StartMultiColumn(2, wxALIGN_CENTER);
    {
-      S.TieChoice( XO("Type:"),
+      S.TieChoice( XXO("Type:"),
          mInfoType, Msgids( kTypes, nTypes ));
-      S.TieChoice( XO("Format:"),
+      S.TieChoice( XXO("Format:"),
          mFormat, Msgids( kFormats, nFormats ));
    }
    S.EndMultiColumn();
@@ -191,6 +200,8 @@ bool GetInfoCommand::SendMenus(const CommandContext &context)
    return true;
 }
 
+#include "Prefs.h"
+
 namespace {
 
 /**************************************************************************//**
@@ -204,41 +215,39 @@ public:
 
    wxCheckBox * TieCheckBox(
       const TranslatableString &Prompt,
-      const SettingSpec< bool > &Setting) override;
+      const BoolSetting &Setting) override;
    wxCheckBox * TieCheckBoxOnRight(
       const TranslatableString &Prompt,
-      const SettingSpec< bool > &Setting) override;
+      const BoolSetting &Setting) override;
 
-   wxChoice *TieChoice(
-      const TranslatableString &Prompt,
-      const ChoiceSetting &choiceSetting ) override;
+   wxChoice *TieChoice(const TranslatableString &Prompt,
+      ChoiceSetting &choiceSetting) override;
 
-   wxChoice * TieNumberAsChoice(
-      const TranslatableString &Prompt,
-      const SettingSpec< int > &Setting,
+   wxChoice * TieNumberAsChoice(const TranslatableString &Prompt,
+      IntSetting &Setting,
       const TranslatableStrings & Choices,
-      const std::vector<int> * pInternalChoices, int iNoMatchSelector ) override;
+      const std::vector<int> * pInternalChoices, int iNoMatchSelector) override;
 
    wxTextCtrl * TieTextBox(
       const TranslatableString &Prompt,
-      const SettingSpec< wxString > &Setting,
+      const StringSetting &Setting,
       const int nChars) override;
    wxTextCtrl * TieIntegerTextBox(
       const TranslatableString & Prompt,
-      const SettingSpec< int > &Setting,
+      const IntSetting &Setting,
       const int nChars) override;
    wxTextCtrl * TieNumericTextBox(
       const TranslatableString & Prompt,
-      const SettingSpec< double > &Setting,
+      const DoubleSetting &Setting,
       const int nChars) override;
    wxSlider * TieSlider(
       const TranslatableString & Prompt,
-      const SettingSpec< int > &Setting,
+      const IntSetting &Setting,
       const int max,
       const int min = 0) override;
    wxSpinCtrl * TieSpinCtrl(
       const TranslatableString &Prompt,
-      const SettingSpec< int > &Setting,
+      const IntSetting &Setting,
       const int max,
       const int min) override;
 };
@@ -256,7 +265,7 @@ ShuttleGuiGetDefinition::~ShuttleGuiGetDefinition(void)
 
 wxCheckBox * ShuttleGuiGetDefinition::TieCheckBox(
    const TranslatableString &Prompt,
-   const SettingSpec< bool > &Setting)
+   const BoolSetting &Setting)
 {
    StartStruct();
    AddItem( Setting.GetPath(), "id" );
@@ -269,7 +278,7 @@ wxCheckBox * ShuttleGuiGetDefinition::TieCheckBox(
 
 wxCheckBox * ShuttleGuiGetDefinition::TieCheckBoxOnRight(
    const TranslatableString &Prompt,
-   const SettingSpec< bool > &Setting)
+   const BoolSetting &Setting)
 {
    StartStruct();
    AddItem( Setting.GetPath(), "id" );
@@ -280,9 +289,8 @@ wxCheckBox * ShuttleGuiGetDefinition::TieCheckBoxOnRight(
    return ShuttleGui::TieCheckBoxOnRight( Prompt, Setting );
 }
 
-wxChoice * ShuttleGuiGetDefinition::TieChoice(
-   const TranslatableString &Prompt,
-   const ChoiceSetting &choiceSetting  )
+wxChoice * ShuttleGuiGetDefinition::TieChoice(const TranslatableString &Prompt,
+   ChoiceSetting &choiceSetting)
 {
    StartStruct();
    AddItem( choiceSetting.Key(), "id" );
@@ -301,7 +309,7 @@ wxChoice * ShuttleGuiGetDefinition::TieChoice(
 
 wxChoice * ShuttleGuiGetDefinition::TieNumberAsChoice(
    const TranslatableString &Prompt,
-   const SettingSpec< int > &Setting,
+   IntSetting &Setting,
    const TranslatableStrings & Choices,
    const std::vector<int> * pInternalChoices, int iNoMatchSelector)
 {
@@ -320,7 +328,7 @@ wxChoice * ShuttleGuiGetDefinition::TieNumberAsChoice(
 
 wxTextCtrl * ShuttleGuiGetDefinition::TieTextBox(
    const TranslatableString &Prompt,
-   const SettingSpec< wxString > &Setting,
+   const StringSetting &Setting,
    const int nChars)
 {
    StartStruct();
@@ -334,7 +342,7 @@ wxTextCtrl * ShuttleGuiGetDefinition::TieTextBox(
 
 wxTextCtrl * ShuttleGuiGetDefinition::TieIntegerTextBox(
    const TranslatableString & Prompt,
-   const SettingSpec< int > &Setting,
+   const IntSetting &Setting,
    const int nChars)
 {
    StartStruct();
@@ -348,7 +356,7 @@ wxTextCtrl * ShuttleGuiGetDefinition::TieIntegerTextBox(
 
 wxTextCtrl * ShuttleGuiGetDefinition::TieNumericTextBox(
    const TranslatableString & Prompt,
-   const SettingSpec< double > &Setting,
+   const DoubleSetting &Setting,
    const int nChars)
 {
    StartStruct();
@@ -362,7 +370,7 @@ wxTextCtrl * ShuttleGuiGetDefinition::TieNumericTextBox(
 
 wxSlider * ShuttleGuiGetDefinition::TieSlider(
    const TranslatableString & Prompt,
-   const SettingSpec< int > &Setting,
+   const IntSetting &Setting,
    const int max,
    const int min) 
 {
@@ -377,7 +385,7 @@ wxSlider * ShuttleGuiGetDefinition::TieSlider(
 
 wxSpinCtrl * ShuttleGuiGetDefinition::TieSpinCtrl(
    const TranslatableString &Prompt,
-   const SettingSpec< int > &Setting,
+   const IntSetting &Setting,
    const int max,
    const int min) 
 {
@@ -414,14 +422,12 @@ bool GetInfoCommand::SendCommands(const CommandContext &context, int flags )
    PluginManager & pm = PluginManager::Get();
    EffectManager & em = EffectManager::Get();
    {
-      const PluginDescriptor *plug = pm.GetFirstPlugin(PluginTypeEffect | PluginTypeAudacityCommand);
-      while (plug)
-      {
-         auto command = em.GetCommandIdentifier(plug->GetID());
+      for (auto &plug
+           : pm.PluginsOfType(PluginTypeEffect | PluginTypeAudacityCommand)) {
+         auto command = em.GetCommandIdentifier(plug.GetID());
          if (!command.empty()){
-            em.GetCommandDefinition( plug->GetID(), context, flags );
+            em.GetCommandDefinition( plug.GetID(), context, flags );
          }
-         plug = pm.GetNextPlugin(PluginTypeEffect | PluginTypeAudacityCommand );
       }
    }
    context.EndArray();
@@ -510,16 +516,19 @@ bool GetInfoCommand::SendClips(const CommandContext &context)
    auto &tracks = TrackList::Get( context.project );
    int i=0;
    context.StartArray();
-   for (auto waveTrack : tracks.Leaders<WaveTrack>()) {
-      WaveClipPointers ptrs( waveTrack->SortedClipArray());
-      for(WaveClip * pClip : ptrs ) {
-         context.StartStruct();
-         context.AddItem( (double)i, "track" );
-         context.AddItem( pClip->GetStartTime(), "start" );
-         context.AddItem( pClip->GetEndTime(), "end" );
-         context.AddItem( pClip->GetColourIndex(), "color" );
-         context.EndStruct();
-      }
+   for (auto t : tracks.Leaders()) {
+      t->TypeSwitch([&](WaveTrack *waveTrack) {
+         WaveClipPointers ptrs(waveTrack->SortedClipArray());
+         for (WaveClip * pClip : ptrs) {
+            context.StartStruct();
+            context.AddItem((double)i, "track");
+            context.AddItem(pClip->GetPlayStartTime(), "start");
+            context.AddItem(pClip->GetPlayEndTime(), "end");
+            context.AddItem(pClip->GetColourIndex(), "color");
+            context.EndStruct();
+         }
+      });
+      // Per track numbering counts all tracks
       i++;
    }
    context.EndArray();
@@ -533,30 +542,35 @@ bool GetInfoCommand::SendEnvelopes(const CommandContext &context)
    int i=0;
    int j=0;
    context.StartArray();
-   for (auto waveTrack : tracks.Leaders<WaveTrack>()) {
-      WaveClipPointers ptrs( waveTrack->SortedClipArray());
-      for(WaveClip * pClip : ptrs ) {
-         context.StartStruct();
-         context.AddItem( (double)i, "track" );
-         context.AddItem( (double)j, "clip" );
-         context.AddItem( pClip->GetStartTime(), "start" );
-         Envelope * pEnv = pClip->GetEnvelope();
-         context.StartField( "points" );
-         context.StartArray();
-         double offset = pEnv->GetOffset();
-         for( size_t k = 0; k < pEnv->GetNumberOfPoints(); k++)
-         {
-            context.StartStruct( );
-            context.AddItem( (*pEnv)[k].GetT()+offset, "t" );
-            context.AddItem( (*pEnv)[k].GetVal(), "y" );
+   for (auto t : tracks.Leaders()) {
+      t->TypeSwitch([&](WaveTrack *waveTrack) {
+         WaveClipPointers ptrs(waveTrack->SortedClipArray());
+         j = 0;
+         for (WaveClip * pClip : ptrs) {
+            context.StartStruct();
+            context.AddItem((double)i, "track");
+            context.AddItem((double)j, "clip");
+            context.AddItem(pClip->GetPlayStartTime(), "start");
+            Envelope * pEnv = pClip->GetEnvelope();
+            context.StartField("points");
+            context.StartArray();
+            double offset = pEnv->GetOffset();
+            for (size_t k = 0; k < pEnv->GetNumberOfPoints(); k++)
+            {
+               context.StartStruct();
+               context.AddItem((*pEnv)[k].GetT() + offset, "t");
+               context.AddItem((*pEnv)[k].GetVal(), "y");
+               context.EndStruct();
+            }
+            context.EndArray();
+            context.EndField();
+            context.AddItem(pClip->GetPlayEndTime(), "end");
             context.EndStruct();
+            j++;
          }
-         context.EndArray();
-         context.EndField();
-         context.AddItem( pClip->GetEndTime(), "end" );
-         context.EndStruct();
-         j++;
-      }
+      });
+      // Per track numbering counts all tracks
+      i++;
    }
    context.EndArray();
 
@@ -601,8 +615,6 @@ bool GetInfoCommand::SendLabels(const CommandContext &context)
    }
    context.EndArray();
 
-
-
    return true;
 }
 
@@ -688,80 +700,20 @@ void GetInfoCommand::ExploreAdornments( const CommandContext &context,
 }
 
 void GetInfoCommand::ExploreTrackPanel( const CommandContext &context,
-   wxPoint P, wxWindow * pWin, int WXUNUSED(Id), int depth )
+   wxPoint P, int depth )
 {
    AudacityProject * pProj = &context.project;
    auto &tp = TrackPanel::Get( *pProj );
-   auto &viewInfo = ViewInfo::Get( *pProj );
-
-   wxRect trackRect = pWin->GetRect();
-
-   for ( auto t : TrackList::Get( *pProj ).Any() + IsVisibleTrack{ pProj } ) {
-      auto &view = TrackView::Get( *t );
-      trackRect.y = view.GetY() - viewInfo.vpos;
-      trackRect.height = view.GetHeight();
-
-#if 0
-      // Work in progress on getting the TCP button positions and sizes.
-      wxRect rect = trackRect;
-      Track *l = t->GetLink();
-
-      if (t->GetLinked()) {
-         rect.height += l->GetHeight();
-      }
-
-      switch (t->GetKind()) {
-         case Track::Wave:
-         {
-            break;
-         }
-#ifdef USE_MIDI
-         case Track::Note:
-         {
-            break;
-         }
-#endif // USE_MIDI
-         case Track::Label:
-            break;
-         case Track::Time:
-            break;
-      }
-      {
-         // Start with whole track rect
-         wxRect R = trackRect;
-
-         // Now exclude left, right, and top insets
-         R.x += kLeftInset;
-         R.y += kTopInset;
-         R.width -= kLeftInset * 2;
-         R.height -= kTopInset;
-
-         int labelw = viewInfo.GetLabelWidth();
-         //int vrul = viewInfo.GetVRulerOffset();
-         bool bIsWave = true;
-         //mTrackInfo.DrawBackground(dc, R, t->GetSelected(), bIsWave, labelw, vrul);
-
-
-         for (Overlay * pOverlay : pTP->mOverlays) {
-            auto R2(pOverlay->GetRectangle(trackRect.GetSize()).first);
-            context.Status( wxString::Format("  [ %2i, %3i, %3i, %3i, %3i, \"%s\" ],", 
-               depth, R2.GetLeft(), R2.GetTop(), R2.GetRight(), R2.GetBottom(), "Otherthing" )); 
-         }
-      }
-#endif
-
-      // The VRuler.
-      {  
-         wxRect R = trackRect;
-         R.x += viewInfo.GetVRulerOffset();
-         R.y += kTopMargin;
-         R.width = viewInfo.GetVRulerWidth();
-         R.height -= (kTopMargin + kBottomMargin);
+   wxRect panelRect{ {}, tp.GetSize() };
+   for ( auto t : TrackList::Get( *pProj ).Any() ) {
+      auto rulers = tp.FindRulerRects(t);
+      for (auto &R : rulers) {
+         if (!R.Intersects(panelRect))
+            continue;
          R.SetPosition( R.GetPosition() + P );
-
          context.StartStruct();
          context.AddItem( depth, "depth" );
-         context.AddItem( "VRuler", "label" ); 
+         context.AddItem( "VRuler", "label" );
          context.StartField("box");
          context.StartArray();
          context.AddItem( R.GetLeft() );
@@ -784,7 +736,7 @@ void GetInfoCommand::ExploreWindows( const CommandContext &context,
    if( pWin->GetName() == "Track Panel" )
    {
       wxRect R = pWin->GetScreenRect();
-      ExploreTrackPanel(  context, R.GetPosition()-P, pWin, Id, depth );
+      ExploreTrackPanel(  context, R.GetPosition()-P, depth );
       return;
    }
    wxWindowList list = pWin->GetChildren();
