@@ -12,8 +12,7 @@
 #ifndef __AUDACITY_KEYBOARD__
 #define __AUDACITY_KEYBOARD__
 
-#include <audacity/Types.h>
-#include <wx/defs.h>
+#include "Identifier.h"
 
 class wxKeyEvent;
 
@@ -21,7 +20,7 @@ struct NormalizedKeyStringTag;
 // Case insensitive comparisons
 using NormalizedKeyStringBase = TaggedIdentifier<NormalizedKeyStringTag, false>;
 
-struct NormalizedKeyString : NormalizedKeyStringBase
+struct AUDACITY_DLL_API NormalizedKeyString : NormalizedKeyStringBase
 {
    NormalizedKeyString() = default;
    explicit NormalizedKeyString( const wxString &key );
@@ -35,6 +34,7 @@ namespace std
       : hash< NormalizedKeyStringBase > {};
 }
 
+AUDACITY_DLL_API
 NormalizedKeyString KeyEventToKeyString(const wxKeyEvent & keyEvent);
 
 #endif
