@@ -13,7 +13,7 @@
 
 *//*******************************************************************/
 
-#include "../Audacity.h"
+
 #include "MessageCommand.h"
 
 #include "LoadCommands.h"
@@ -26,11 +26,18 @@ const ComponentInterfaceSymbol MessageCommand::Symbol
 
 namespace{ BuiltinCommandsModule::Registration< MessageCommand > reg; }
 
-bool MessageCommand::DefineParams( ShuttleParams & S ){
-   S.Define( mMessage, wxT("Text"),  "Some message" );
+template<bool Const>
+bool MessageCommand::VisitSettings( SettingsVisitorBase<Const> & S ){
+   S.Define( mMessage, wxT("Text"), wxString{"Some message"} );
    return true;
 }
 
+bool MessageCommand::VisitSettings( SettingsVisitor & S )
+   { return VisitSettings<false>(S); }
+
+bool MessageCommand::VisitSettings( ConstSettingsVisitor & S )
+   { return VisitSettings<true>(S); }
+
 void MessageCommand::PopulateOrExchange(ShuttleGui & S)
 {
    S.AddSpace(0, 5);
