@@ -16,7 +16,7 @@
 
 *//*******************************************************************/
 
-#include "../Audacity.h"
+
 #include "SetClipCommand.h"
 
 #include "LoadCommands.h"
@@ -52,7 +52,8 @@ static const EnumValueSymbol kColourStrings[nColours] =
 };
 
 
-bool SetClipCommand::DefineParams( ShuttleParams & S ){ 
+template<bool Const>
+bool SetClipCommand::VisitSettings( SettingsVisitorBase<Const> & S ){
    S.OptionalY( bHasContainsTime   ).Define(     mContainsTime,   wxT("At"),         0.0, 0.0, 100000.0 );
    S.OptionalN( bHasColour         ).DefineEnum( mColour,         wxT("Color"),      kColour0, kColourStrings, nColours );
    // Allowing a negative start time is not a mistake.
@@ -60,6 +61,11 @@ bool SetClipCommand::DefineParams( ShuttleParams & S ){
    S.OptionalN( bHasT0             ).Define(     mT0,             wxT("Start"),      0.0, -5.0, 1000000.0);
    return true;
 };
+bool SetClipCommand::VisitSettings( SettingsVisitor & S )
+   { return VisitSettings<false>(S); }
+
+bool SetClipCommand::VisitSettings( ConstSettingsVisitor & S )
+   { return VisitSettings<true>(S); }
 
 void SetClipCommand::PopulateOrExchange(ShuttleGui & S)
 {
@@ -71,6 +77,7 @@ void SetClipCommand::PopulateOrExchange(ShuttleGui & S)
       S.Optional( bHasColour      ).TieChoice(          XXO("Color:"),         mColour,
          Msgids( kColourStrings, nColours ) );
       S.Optional( bHasT0          ).TieNumericTextBox(  XXO("Start:"),         mT0 );
+      S.Optional( bHasName        ).TieTextBox(         XXO("Name:"),          mName );
    }
    S.EndMultiColumn();
 }
@@ -84,8 +91,8 @@ bool SetClipCommand::ApplyInner( const CommandContext &, Track * t )
          WaveClip * pClip = *it;
          bool bFound =
             !bHasContainsTime || (
-               ( pClip->GetStartTime() <= mContainsTime ) &&
-               ( pClip->GetEndTime() >= mContainsTime )
+               ( pClip->GetPlayStartTime() <= mContainsTime ) &&
+               ( pClip->GetPlayEndTime() >= mContainsTime )
             );
          if( bFound )
          {
@@ -95,8 +102,10 @@ bool SetClipCommand::ApplyInner( const CommandContext &, Track * t )
                pClip->SetColourIndex(mColour);
             // No validation of overlap yet.  We assume the user is sensible!
             if( bHasT0 )
-               pClip->SetOffset(mT0);
+               pClip->SetPlayStartTime(mT0);
             // \todo Use SetClip to move a clip between tracks too.
+            if( bHasName )
+               pClip->SetName(mName);
 
          }
       }
