@@ -25,24 +25,28 @@ public:
 
    SetClipCommand();
    // ComponentInterface overrides
-   ComponentInterfaceSymbol GetSymbol() override {return Symbol;};
-   TranslatableString GetDescription() override {return XO("Sets various values for a clip.");};
-   bool DefineParams( ShuttleParams & S ) override;
+   ComponentInterfaceSymbol GetSymbol() const override {return Symbol;};
+   TranslatableString GetDescription() const override {return XO("Sets various values for a clip.");};
+   template<bool Const> bool VisitSettings( SettingsVisitorBase<Const> &S );
+   bool VisitSettings( SettingsVisitor & S ) override;
+   bool VisitSettings( ConstSettingsVisitor & S ) override;
    void PopulateOrExchange(ShuttleGui & S) override;
 
    // AudacityCommand overrides
-   wxString ManualPage() override {return wxT("Extra_Menu:_Scriptables_I#set_clip");};
+   ManualPageID ManualPage() override {return L"Extra_Menu:_Scriptables_I#set_clip";}
    bool ApplyInner( const CommandContext & context, Track * t ) override;
 
 public:
    double mContainsTime;
    int mColour;
    double mT0;
+   wxString mName;
 
 // For tracking optional parameters.
    bool bHasContainsTime;
    bool bHasColour;
    bool bHasT0;
+   bool bHasName;
 };
 
 
