@@ -13,46 +13,43 @@
 
 *//*******************************************************************/
 
-#include "../Audacity.h"
+
 #include "DtmfGen.h"
 #include "LoadEffects.h"
 
-#include "../Experimental.h"
-
 #include <wx/intl.h>
 #include <wx/slider.h>
 #include <wx/valgen.h>
 #include <wx/valtext.h>
 #include <wx/stattext.h>
 
-#include "../Prefs.h"
-#include "../Shuttle.h"
+#include "Prefs.h"
 #include "../ShuttleGui.h"
 #include "../widgets/NumericTextCtrl.h"
 #include "../widgets/valnum.h"
 
 
-enum
+namespace {
+wxString AllSymbols();
+}
+
+const EffectParameterMethods& EffectDtmf::Parameters() const
 {
-   ID_Sequence,
-   ID_Amplitude,
-   ID_Duration,
-   ID_DutyCycle,
-};
-
-// DA: DTMF for Audacity uses a different string.
-#ifdef EXPERIMENTAL_DA
-#define SHORT_APP_NAME "darkaudacity"
-#else
-#define SHORT_APP_NAME "audacity"
-#endif
-
-// Define keys, defaults, minimums, and maximums for the effect parameters
-//
-//     Name       Type        Key               Def                   Min      Max      Scale
-Param( Sequence,  wxString,   wxT("Sequence"),   wxT(SHORT_APP_NAME),  wxT(""), wxT(""), wxT(""));
-Param( DutyCycle, double,     wxT("Duty Cycle"), 55.0,                 0.0,     100.0,   10.0   );
-Param( Amplitude, double,     wxT("Amplitude"),  0.8,                  0.001,   1.0,     1      );
+   static CapturedParameters<EffectDtmf,
+      Sequence, DutyCycle, Amplitude
+   > parameters{
+      [](EffectDtmf &, EffectSettings &es, DtmfSettings &s, bool updating){
+         if (updating) {
+            if (s.dtmfSequence.find_first_not_of(AllSymbols())
+                != wxString::npos)
+               return false;
+            s.Recalculate(es);
+         }
+         return true;
+      },
+   };
+   return parameters;
+}
 
 static const double kFadeInOut = 250.0; // used for fadein/out needed to remove clicking noise
 
@@ -71,6 +68,16 @@ const static wxChar *kSymbols[] =
    wxT("y"), wxT("z")
 };
 
+namespace {
+wxString AllSymbols()
+{
+   wxString symbols;
+   for (unsigned int i = 0; i < WXSIZEOF(kSymbols); ++i)
+      symbols += kSymbols[i];
+   return symbols;
+}
+}
+
 //
 // EffectDtmf
 //
@@ -80,20 +87,8 @@ const ComponentInterfaceSymbol EffectDtmf::Symbol
 
 namespace{ BuiltinEffectsModule::Registration< EffectDtmf > reg; }
 
-BEGIN_EVENT_TABLE(EffectDtmf, wxEvtHandler)
-    EVT_TEXT(ID_Sequence, EffectDtmf::OnSequence)
-    EVT_TEXT(ID_DutyCycle, EffectDtmf::OnAmplitude)
-    EVT_TEXT(ID_Duration, EffectDtmf::OnDuration)
-    EVT_SLIDER(ID_DutyCycle, EffectDtmf::OnDutyCycle)
-END_EVENT_TABLE()
-
 EffectDtmf::EffectDtmf()
 {
-   dtmfDutyCycle = DEF_DutyCycle;
-   dtmfAmplitude = DEF_Amplitude;
-   dtmfSequence = DEF_Sequence;
-   dtmfTone = 0.0;
-   dtmfSilence = 0.0;
 }
 
 EffectDtmf::~EffectDtmf()
@@ -102,45 +97,83 @@ EffectDtmf::~EffectDtmf()
 
 // ComponentInterface implementation
 
-ComponentInterfaceSymbol EffectDtmf::GetSymbol()
+ComponentInterfaceSymbol EffectDtmf::GetSymbol() const
 {
    return Symbol;
 }
 
-TranslatableString EffectDtmf::GetDescription()
+TranslatableString EffectDtmf::GetDescription() const
 {
    return XO("Generates dual-tone multi-frequency (DTMF) tones like those produced by the keypad on telephones");
 }
 
-wxString EffectDtmf::ManualPage()
+ManualPageID EffectDtmf::ManualPage() const
 {
-   return wxT("DTMF_Tones");
+   return L"DTMF_Tones";
 }
 
 // EffectDefinitionInterface implementation
 
-EffectType EffectDtmf::GetType()
+EffectType EffectDtmf::GetType() const
 {
    return EffectTypeGenerate;
 }
 
-// EffectClientInterface implementation
-
-unsigned EffectDtmf::GetAudioOutCount()
+//! Temporary state of the computation
+struct EffectDtmf::Instance
+   : PerTrackEffect::Instance
+   , EffectInstanceWithBlockSize
 {
-   return 1;
-}
+   Instance(const PerTrackEffect &effect, double t0)
+      : PerTrackEffect::Instance{ effect }
+      , mT0{ t0 }
+   {}
 
-bool EffectDtmf::ProcessInitialize(sampleCount WXUNUSED(totalLen), ChannelNames WXUNUSED(chanMap))
+   bool ProcessInitialize(EffectSettings &settings, double sampleRate,
+      ChannelNames chanMap) override;
+   size_t ProcessBlock(EffectSettings &settings,
+      const float *const *inBlock, float *const *outBlock, size_t blockLen)
+   override;
+
+   unsigned GetAudioInCount() const override
+   {
+      return 0;
+   }
+
+   unsigned GetAudioOutCount() const override
+   {
+      return 1;
+   }
+
+   const double mT0;
+   double mSampleRate{};
+
+   sampleCount numSamplesSequence;  // total number of samples to generate
+   sampleCount numSamplesTone;      // number of samples in a tone block
+   sampleCount numSamplesSilence;   // number of samples in a silence block
+   sampleCount diff;                // number of extra samples to redistribute
+   sampleCount numRemaining;        // number of samples left to produce in the current block
+   sampleCount curTonePos;          // position in tone to start the wave
+   bool isTone;                     // true if block is tone, otherwise silence
+   int curSeqPos;                   // index into dtmf tone string
+};
+
+bool EffectDtmf::Instance::ProcessInitialize(
+   EffectSettings &settings, double sampleRate, ChannelNames)
 {
-   if (dtmfNTones <= 0) {   // Bail if no DTFM sequence.
-      ::Effect::MessageBox(
+   mSampleRate = sampleRate;
+
+   auto &dtmfSettings = GetSettings(settings);
+   if (dtmfSettings.dtmfNTones == 0) {   // Bail if no DTFM sequence.
+      // TODO:  don't use mProcessor for this
+      mProcessor.MessageBox(
                XO("DTMF sequence empty.\nCheck ALL settings for this effect."),
          wxICON_ERROR );
 
       return false;
    }
-   double duration = GetDuration();
+
+   double duration = settings.extra.GetDuration();
 
    // all dtmf sequence durations in samples from seconds
    // MJS: Note that mDuration is in seconds but will have been quantised to the units of the TTC.
@@ -153,24 +186,26 @@ bool EffectDtmf::ProcessInitialize(sampleCount WXUNUSED(totalLen), ChannelNames
    numSamplesSequence = nT1 - nT0;  // needs to be exact number of samples selected
 
    //make under-estimates if anything, and then redistribute the few remaining samples
-   numSamplesTone = sampleCount( floor(dtmfTone * mSampleRate) );
-   numSamplesSilence = sampleCount( floor(dtmfSilence * mSampleRate) );
+   numSamplesTone = sampleCount( floor(dtmfSettings.dtmfTone * mSampleRate) );
+   numSamplesSilence = sampleCount( floor(dtmfSettings.dtmfSilence * mSampleRate) );
 
    // recalculate the sum, and spread the difference - due to approximations.
    // Since diff should be in the order of "some" samples, a division (resulting in zero)
    // is not sufficient, so we add the additional remaining samples in each tone/silence block,
    // at least until available.
-   diff = numSamplesSequence - (dtmfNTones*numSamplesTone) - (dtmfNTones-1)*numSamplesSilence;
-   while (diff > 2*dtmfNTones - 1) {   // more than one per thingToBeGenerated
+   diff = numSamplesSequence - (dtmfSettings.dtmfNTones * numSamplesTone)
+      - (dtmfSettings.dtmfNTones - 1) * numSamplesSilence;
+   while (diff > 2 * dtmfSettings.dtmfNTones - 1) {   // more than one per thingToBeGenerated
       // in this case, both numSamplesTone and numSamplesSilence would change, so it makes sense
       //  to recalculate diff here, otherwise just keep the value we already have
 
       // should always be the case that dtmfNTones>1, as if 0, we don't even start processing,
       // and with 1 there is no difference to spread (no silence slot)...
-      wxASSERT(dtmfNTones > 1);
-      numSamplesTone += (diff/(dtmfNTones));
-      numSamplesSilence += (diff/(dtmfNTones-1));
-      diff = numSamplesSequence - (dtmfNTones*numSamplesTone) - (dtmfNTones-1)*numSamplesSilence;
+      wxASSERT(dtmfSettings.dtmfNTones > 1);
+      numSamplesTone += (diff / (dtmfSettings.dtmfNTones));
+      numSamplesSilence += (diff / (dtmfSettings.dtmfNTones - 1));
+      diff = numSamplesSequence - (dtmfSettings.dtmfNTones * numSamplesTone)
+         - (dtmfSettings.dtmfNTones - 1) * numSamplesSilence;
    }
    wxASSERT(diff >= 0);  // should never be negative
 
@@ -181,8 +216,10 @@ bool EffectDtmf::ProcessInitialize(sampleCount WXUNUSED(totalLen), ChannelNames
    return true;
 }
 
-size_t EffectDtmf::ProcessBlock(float **WXUNUSED(inbuf), float **outbuf, size_t size)
+size_t EffectDtmf::Instance::ProcessBlock(EffectSettings &settings,
+   const float *const *, float *const *outbuf, size_t size)
 {
+   auto &dtmfSettings = GetSettings(settings);
    float *buffer = outbuf[0];
    decltype(size) processed = 0;
 
@@ -230,7 +267,10 @@ size_t EffectDtmf::ProcessBlock(float **WXUNUSED(inbuf), float **outbuf, size_t
       if (isTone)
       {
          // generate the tone and append
-         MakeDtmfTone(buffer, len, mSampleRate, dtmfSequence[curSeqPos], curTonePos, numSamplesTone, dtmfAmplitude);
+         assert(curSeqPos < dtmfSettings.dtmfNTones) ;
+         MakeDtmfTone(buffer, len, mSampleRate,
+            dtmfSettings.dtmfSequence[curSeqPos], curTonePos, numSamplesTone,
+            dtmfSettings.dtmfAmplitude);
          curTonePos += len;
       }
       else
@@ -247,88 +287,52 @@ size_t EffectDtmf::ProcessBlock(float **WXUNUSED(inbuf), float **outbuf, size_t
 
    return processed;
 }
-bool EffectDtmf::DefineParams( ShuttleParams & S ){
-   S.SHUTTLE_PARAM( dtmfSequence, Sequence );
-   S.SHUTTLE_PARAM( dtmfDutyCycle, DutyCycle );
-   S.SHUTTLE_PARAM( dtmfAmplitude, Amplitude );
-   return true;
-}
 
-bool EffectDtmf::GetAutomationParameters(CommandParameters & parms)
+// Event handler object
+struct EffectDtmf::Validator
+   : EffectUIValidator
 {
-   parms.Write(KEY_Sequence, dtmfSequence);
-   parms.Write(KEY_DutyCycle, dtmfDutyCycle);
-   parms.Write(KEY_Amplitude, dtmfAmplitude);
+   Validator(EffectUIClientInterface &effect,
+      EffectSettingsAccess &access, const DtmfSettings &settings)
+      : EffectUIValidator{effect, access}
+      // Copy settings
+      , mSettings{settings}
+   {}
+   virtual ~Validator() = default;
 
-   return true;
-}
+   Effect &GetEffect() const { return static_cast<Effect&>(mEffect); }
 
-bool EffectDtmf::SetAutomationParameters(CommandParameters & parms)
+   bool ValidateUI() override;
+   bool UpdateUI() override;
+   void DoUpdateUI();
+
+   void PopulateOrExchange(ShuttleGui & S,
+      const EffectSettings &settings, double projectRate);
+   void OnSequence(wxCommandEvent & evt);
+   void OnDuration(wxCommandEvent & evt);
+   void OnDutyCycle(wxCommandEvent & evt);
+
+   // These settings exist for the lifetime of the validator
+   DtmfSettings mSettings;
+
+   wxTextCtrl *mDtmfSequenceT;
+   wxSlider   *mDtmfDutyCycleS;
+   NumericTextCtrl *mDtmfDurationT;
+   wxStaticText *mDtmfToneT;
+   wxStaticText *mDtmfSilenceT;
+   wxStaticText *mDtmfDutyT;
+};
+
+void EffectDtmf::Validator::PopulateOrExchange(ShuttleGui & S,
+   const EffectSettings &settings, double projectRate)
 {
-   ReadAndVerifyDouble(DutyCycle);
-   ReadAndVerifyDouble(Amplitude);
-   ReadAndVerifyString(Sequence);
+   // Reference to our copy of this effect's special settings
+   auto &dtmfSettings = mSettings;
 
-   wxString symbols;
-   for (unsigned int i = 0; i < WXSIZEOF(kSymbols); i++)
-   {
-      symbols += kSymbols[i];
-   }
+   // Do NOT hold a reference to EffectSettings, just use it to find initial
+   // duration values.  (It came from EffectSettingsAccess so its stable address
+   // can't be relied on.)
 
-   if (Sequence.find_first_not_of(symbols) != wxString::npos)
-   {
-      return false;
-   }
-
-   dtmfDutyCycle = DutyCycle;
-   dtmfAmplitude = Amplitude;
-   dtmfSequence = Sequence;
-   
-   Recalculate();
-
-   return true;
-}
-
-// Effect implementation
-
-bool EffectDtmf::Startup()
-{
-   wxString base = wxT("/Effects/DtmfGen/");
-
-   // Migrate settings from 2.1.0 or before
-
-   // Already migrated, so bail
-   if (gPrefs->Exists(base + wxT("Migrated")))
-   {
-      return true;
-   }
-
-   // Load the old "current" settings
-   if (gPrefs->Exists(base))
-   {
-      gPrefs->Read(base + wxT("String"), &dtmfSequence, wxT(SHORT_APP_NAME));
-      gPrefs->Read(base + wxT("DutyCycle"), &dtmfDutyCycle, 550L);
-      gPrefs->Read(base + wxT("Amplitude"), &dtmfAmplitude, 0.8f);
-
-      SaveUserPreset(GetCurrentSettingsGroup());
-
-      // Do not migrate again
-      gPrefs->Write(base + wxT("Migrated"), true);
-      gPrefs->Flush();
-   }
-
-   return true;
-}
-
-bool EffectDtmf::Init()
-{
-   Recalculate();
-
-   return true;
-}
-
-void EffectDtmf::PopulateOrExchange(ShuttleGui & S)
-{
    // dialog will be passed values from effect
    // Effect retrieves values from saved config
    // Dialog will take care of using them to initialize controls
@@ -339,40 +343,49 @@ void EffectDtmf::PopulateOrExchange(ShuttleGui & S)
    S.AddSpace(0, 5);
    S.StartMultiColumn(2, wxCENTER);
    {
-      mDtmfSequenceT = S.Id(ID_Sequence)
-         .Validator([this]{
-            wxTextValidator vldDtmf(wxFILTER_INCLUDE_CHAR_LIST, &dtmfSequence);
+      mDtmfSequenceT =
+      S
+         .Validator([&dtmfSettings]{
+            wxTextValidator vldDtmf(
+               wxFILTER_INCLUDE_CHAR_LIST, &dtmfSettings.dtmfSequence);
             vldDtmf.SetIncludes(wxArrayString(WXSIZEOF(kSymbols), kSymbols));
             return vldDtmf;
          })
          .AddTextBox(XXO("DTMF &sequence:"), wxT(""), 10);
+      BindTo(*mDtmfSequenceT, wxEVT_TEXT, &Validator::OnSequence);
 
-      S.Id(ID_Amplitude)
+      // A control with no event handler but the validator causes updates
+      // when TransferData functions are called
+      S
          .Validator<FloatingPointValidator<double>>(
-            3, &dtmfAmplitude, NumValidatorStyle::NO_TRAILING_ZEROES,
-            MIN_Amplitude, MAX_Amplitude)
+            3, &dtmfSettings.dtmfAmplitude, NumValidatorStyle::NO_TRAILING_ZEROES,
+            Amplitude.min, Amplitude.max)
          .AddTextBox(XXO("&Amplitude (0-1):"), wxT(""), 10);
 
       S.AddPrompt(XXO("&Duration:"));
+      auto &extra = settings.extra;
       mDtmfDurationT = safenew
-         NumericTextCtrl(S.GetParent(), ID_Duration,
+         NumericTextCtrl(S.GetParent(), wxID_ANY,
                          NumericConverter::TIME,
-                         GetDurationFormat(),
-                         GetDuration(),
-                         mProjectRate,
+                         extra.GetDurationFormat(),
+                         extra.GetDuration(),
+                         projectRate,
                          NumericTextCtrl::Options{}
                             .AutoPos(true));
       S.Name(XO("Duration"))
          .AddWindow(mDtmfDurationT);
+      BindTo(*mDtmfDurationT, wxEVT_TEXT, &Validator::OnDuration);
 
       S.AddFixedText(XO("&Tone/silence ratio:"), false);
-      mDtmfDutyCycleS = S.Id(ID_DutyCycle)
+      mDtmfDutyCycleS =
+      S
          .Style(wxSL_HORIZONTAL | wxEXPAND)
          .MinSize( { -1, -1 } )
          .AddSlider( {},
-                     dtmfDutyCycle * SCL_DutyCycle,
-                     MAX_DutyCycle * SCL_DutyCycle,
-                     MIN_DutyCycle * SCL_DutyCycle);
+                     dtmfSettings.dtmfDutyCycle * DutyCycle.scale,
+                     DutyCycle.max * DutyCycle.scale,
+                     DutyCycle.min * DutyCycle.scale);
+      BindTo(*mDtmfDutyCycleS, wxEVT_SLIDER, &Validator::OnDutyCycle);
    }
    S.EndMultiColumn();
 
@@ -380,75 +393,100 @@ void EffectDtmf::PopulateOrExchange(ShuttleGui & S)
    {
       S.AddFixedText(XO("Duty cycle:"), false);
       mDtmfDutyT =
-         S.AddVariableText(XO("%.1f %%").Format( dtmfDutyCycle ), false);
+      S.AddVariableText(XO("%.1f %%")
+         .Format( dtmfSettings.dtmfDutyCycle ), false);
       
       S.AddFixedText(XO("Tone duration:"), false);
       mDtmfSilenceT =
-         /* i18n-hint milliseconds */
-         S.AddVariableText(XO("%.0f ms").Format( dtmfTone * 1000.0 ), false);
+      /* i18n-hint milliseconds */
+      S.AddVariableText(XO("%.0f ms")
+         .Format( dtmfSettings.dtmfTone * 1000.0 ), false);
 
       S.AddFixedText(XO("Silence duration:"), false);
       mDtmfToneT =
-         /* i18n-hint milliseconds */
-         S.AddVariableText(XO("%0.f ms").Format( dtmfSilence * 1000.0 ), false);
+      /* i18n-hint milliseconds */
+      S.AddVariableText(XO("%0.f ms")
+         .Format( dtmfSettings.dtmfSilence * 1000.0 ), false);
    }
    S.EndMultiColumn();
 }
 
-bool EffectDtmf::TransferDataToWindow()
+// Effect implementation
+
+std::unique_ptr<EffectUIValidator> EffectDtmf::PopulateOrExchange(
+   ShuttleGui & S, EffectInstance &, EffectSettingsAccess &access,
+   const EffectOutputs *)
 {
-   Recalculate();
+   auto &settings = access.Get();
+   auto &dtmfSettings = GetSettings(settings);
+   auto result = std::make_unique<Validator>(*this, access, dtmfSettings);
+   result->PopulateOrExchange(S, settings, mProjectRate);
+   return result;
+}
 
-   if (!mUIParent->TransferDataToWindow())
-   {
-      return false;
-   }
+std::shared_ptr<EffectInstance> EffectDtmf::MakeInstance() const
+{
+   // TODO: don't use Effect::mT0 and Effect::mSampleRate, but use an
+   // EffectContext (that class is not yet defined)
+   return std::make_shared<Instance>(*this, mT0);
+}
 
-   mDtmfDutyCycleS->SetValue(dtmfDutyCycle * SCL_DutyCycle);
+bool EffectDtmf::Validator::UpdateUI()
+{
+   const auto &settings = mAccess.Get();
+   auto &dtmfSettings = mSettings;
 
-   mDtmfDurationT->SetValue(GetDuration());
+   // Copy into our settings
+   mSettings = GetSettings(settings);
 
-   UpdateUI();
+   mDtmfDutyCycleS->SetValue(dtmfSettings.dtmfDutyCycle * DutyCycle.scale);
+
+   mDtmfDurationT->SetValue(settings.extra.GetDuration());
+
+   DoUpdateUI();
 
    return true;
 }
 
-bool EffectDtmf::TransferDataFromWindow()
+bool EffectDtmf::Validator::ValidateUI()
 {
-   if (!mUIParent->Validate() || !mUIParent->TransferDataFromWindow())
-   {
-      return false;
-   }
+   mAccess.ModifySettings([this](EffectSettings &settings){
+      auto &dtmfSettings = mSettings;
+      dtmfSettings.dtmfDutyCycle =
+         (double) mDtmfDutyCycleS->GetValue() / DutyCycle.scale;
+      settings.extra.SetDuration(mDtmfDurationT->GetValue());
 
-   dtmfDutyCycle = (double) mDtmfDutyCycleS->GetValue() / SCL_DutyCycle;
-   SetDuration(mDtmfDurationT->GetValue());
-
-   // recalculate to make sure all values are up-to-date. This is especially
-   // important if the user did not change any values in the dialog
-   Recalculate();
+      // recalculate to make sure all values are up-to-date. This is especially
+      // important if the user did not change any values in the dialog
+      dtmfSettings.Recalculate(settings);
+      return nullptr;
+   });
 
    return true;
 }
 
 // EffectDtmf implementation
 
-void EffectDtmf::Recalculate()
+// Updates dtmfNTones, dtmfTone, dtmfSilence, and sometimes duration
+// They depend on dtmfSequence, dtmfDutyCycle, and duration
+void DtmfSettings::Recalculate(EffectSettings &settings)
 {
+   auto &extra = settings.extra;
    // remember that dtmfDutyCycle is in range (0.0-100.0)
 
-   dtmfNTones = (int) dtmfSequence.length();
+   dtmfNTones = dtmfSequence.length();
 
    if (dtmfNTones==0) {
       // no tones, all zero: don't do anything
       // this should take care of the case where user got an empty
       // dtmf sequence into the generator: track won't be generated
-      SetDuration(0.0);
+      extra.SetDuration(0.0);
       dtmfTone = 0;
       dtmfSilence = 0;
    } else {
       if (dtmfNTones==1) {
         // single tone, as long as the sequence
-          dtmfTone = GetDuration();
+          dtmfTone = extra.GetDuration();
           dtmfSilence = 0;
       } else {
          // Don't be fooled by the fact that you divide the sequence into dtmfNTones:
@@ -456,11 +494,12 @@ void EffectDtmf::Recalculate()
          // Given this, the right thing to do is to divide the sequence duration
          // by dtmfNTones tones and (dtmfNTones-1) silences each sized according to the duty
          // cycle: original division was:
-         // slot=mDuration / (dtmfNTones*(dtmfDutyCycle/MAX_DutyCycle)+(dtmfNTones-1)*(1.0-dtmfDutyCycle/MAX_DutyCycle))
+         // slot=mDuration / (dtmfNTones*(dtmfDutyCycle/DutyCycle.max)+(dtmfNTones-1)*(1.0-dtmfDutyCycle/DutyCycle.max))
          // which can be simplified in the one below.
          // Then just take the part that belongs to tone or silence.
          //
-         double slot = GetDuration() / ((double)dtmfNTones + (dtmfDutyCycle / 100.0) - 1);
+         double slot = extra.GetDuration()
+            / ((double)dtmfNTones + (dtmfDutyCycle / 100.0) - 1);
          dtmfTone = slot * (dtmfDutyCycle / 100.0); // seconds
          dtmfSilence = slot * (1.0 - (dtmfDutyCycle / 100.0)); // seconds
 
@@ -472,6 +511,10 @@ void EffectDtmf::Recalculate()
          // - dtmfNTones-1 silences
       }
    }
+
+   // `this` is the settings copy in the validator
+   // Update the EffectSettings held by the dialog
+   EffectDtmf::GetSettings(settings) = *this;
 }
 
 bool EffectDtmf::MakeDtmfTone(float *buffer, size_t len, float fs, wxChar tone, sampleCount last, sampleCount total, float amplitude)
@@ -600,44 +643,54 @@ bool EffectDtmf::MakeDtmfTone(float *buffer, size_t len, float fs, wxChar tone,
    return true;
 }
 
-void EffectDtmf::UpdateUI(void)
+void EffectDtmf::Validator::DoUpdateUI()
 {
-   mDtmfDutyT->SetLabel(wxString::Format(wxT("%.1f %%"), dtmfDutyCycle));
+   // Update some texts in response to controls
+   auto &dtmfSettings = mSettings;
+
+   mDtmfDutyT
+      ->SetLabel(wxString::Format(wxT("%.1f %%"), dtmfSettings.dtmfDutyCycle));
    mDtmfDutyT->SetName(mDtmfDutyT->GetLabel()); // fix for bug 577 (NVDA/Narrator screen readers do not read static text in dialogs)
 
-   mDtmfSilenceT->SetLabel(wxString::Format(_("%.0f ms"), dtmfTone * 1000.0));
+   mDtmfSilenceT
+      ->SetLabel(wxString::Format(_("%.0f ms"), dtmfSettings.dtmfTone * 1000.0));
    mDtmfSilenceT->SetName(mDtmfSilenceT->GetLabel()); // fix for bug 577 (NVDA/Narrator screen readers do not read static text in dialogs)
 
-   mDtmfToneT->SetLabel(wxString::Format(_("%.0f ms"), dtmfSilence * 1000.0));
+   mDtmfToneT
+      ->SetLabel(wxString::Format(_("%.0f ms"), dtmfSettings.dtmfSilence * 1000.0));
    mDtmfToneT->SetName(mDtmfToneT->GetLabel()); // fix for bug 577 (NVDA/Narrator screen readers do not read static text in dialogs)
 }
 
-void EffectDtmf::OnSequence(wxCommandEvent & WXUNUSED(evt))
+void EffectDtmf::Validator::OnSequence(wxCommandEvent & WXUNUSED(evt))
 {
-   dtmfSequence = mDtmfSequenceT->GetValue();
-   Recalculate();
-   UpdateUI();
+   mAccess.ModifySettings([this](EffectSettings &settings){
+      auto &dtmfSettings = mSettings;
+      dtmfSettings.dtmfSequence = mDtmfSequenceT->GetValue();
+      dtmfSettings.Recalculate(settings);
+      return nullptr;
+   });
+   DoUpdateUI();
 }
 
-void EffectDtmf::OnAmplitude(wxCommandEvent & WXUNUSED(evt))
+void EffectDtmf::Validator::OnDuration(wxCommandEvent & WXUNUSED(evt))
 {
-   if (!mDtmfAmplitudeT->GetValidator()->TransferFromWindow())
-   {
-      return;
-   }
-   Recalculate();
-   UpdateUI();
-}
-void EffectDtmf::OnDuration(wxCommandEvent & WXUNUSED(evt))
-{
-   SetDuration(mDtmfDurationT->GetValue());
-   Recalculate();
-   UpdateUI();
+   mAccess.ModifySettings([this](EffectSettings &settings){
+      auto &dtmfSettings = mSettings;
+      auto &effect = GetEffect();
+      settings.extra.SetDuration(mDtmfDurationT->GetValue());
+      dtmfSettings.Recalculate(settings);
+      return nullptr;
+   });
+   DoUpdateUI();
 }
 
-void EffectDtmf::OnDutyCycle(wxCommandEvent & evt)
+void EffectDtmf::Validator::OnDutyCycle(wxCommandEvent & evt)
 {
-   dtmfDutyCycle = (double) evt.GetInt() / SCL_DutyCycle;
-   Recalculate();
-   UpdateUI();
+   mAccess.ModifySettings([this, &evt](EffectSettings &settings){
+      auto &dtmfSettings = mSettings;
+      dtmfSettings.dtmfDutyCycle = (double) evt.GetInt() / DutyCycle.scale;
+      dtmfSettings.Recalculate(settings);
+      return nullptr;
+   });
+   DoUpdateUI();
 }
