@@ -33,6 +33,7 @@ effects.
 #include "../commands/CommandContext.h"
 #include "../commands/AudacityCommand.h"
 #include "PluginManager.h"
+#include "Track.h"
 
 
 /*******************************************************************************
@@ -299,10 +300,10 @@ bool EffectManager::SetEffectParameters(
       if (eap.HasEntry(wxT("Use Preset")))
       {
          return effect->LoadSettingsFromString(
-            eap.Read(wxT("Use Preset")), settings);
+            eap.Read(wxT("Use Preset")), settings).has_value();
       }
 
-      return effect->LoadSettingsFromString(params, settings);
+      return effect->LoadSettingsFromString(params, settings).has_value();
    }
    AudacityCommand *command = GetAudacityCommand(ID);
    
@@ -333,6 +334,18 @@ bool EffectManager::PromptUser(
 {
    bool result = false;
    if (auto effect = GetEffect(ID)) {
+
+      auto empty = TrackList::Create(nullptr);
+      auto pEffectBase = dynamic_cast<EffectBase*>(effect);
+      if (pEffectBase)
+         // This allows effects to call Init() safely
+         pEffectBase->SetTracks(empty.get());
+      Finally Do([&]{
+         // reverse the side-effect
+         if (pEffectBase)
+            pEffectBase->SetTracks(nullptr);
+      });
+
       std::shared_ptr<EffectInstance> pInstance;
       //! Show the effect dialog, only so that the user can choose settings,
       //! for instance to define a macro.
@@ -803,7 +816,8 @@ void InitializePreset(
       SetConfig(manager, PluginSettings::Private, FactoryDefaultsGroup(),
          InitializedKey, true);
    }
-   manager.LoadUserPreset(CurrentSettingsGroup(), settings);
+   // ignore failure
+   (void) manager.LoadUserPreset(CurrentSettingsGroup(), settings);
 }
 
 std::pair<ComponentInterface *, EffectSettings>
