@@ -13,10 +13,8 @@
 
 *//****************************************************************/
 
-#include "../Audacity.h" // for USE_* macros
-#include "Equalization48x.h"
 
-#include "../Experimental.h"
+#include "Equalization48x.h"
 
 #ifdef EXPERIMENTAL_EQ_SSE_THREADED
 #include "../Project.h"
@@ -28,7 +26,6 @@
 
 #include <wx/setup.h> // for wxUSE_* macros
 
-#include <wx/dcmemory.h>
 #include <wx/event.h>
 #include <wx/string.h>
 
@@ -207,7 +204,7 @@ bool EffectEqualization48x::AllocateBuffersWorkers(int nThreads)
    // this will remove the disparity in data at the intersections of the runs
 
    // The nice magic allocation
-   // megabyte - 3 windows - 4 overlaping buffers - filter 
+   // megabyte - 3 windows - 4 overlapping buffers - filter 
    // 2^20 = 1,048,576 - 3 * 2^14 (16,384) - ((4 * 20) - 3) * 12,384 - 4000 
    // 1,048,576 - 49,152 - 953,568 - 4000 = 41,856 (leftover)
 
@@ -345,10 +342,8 @@ bool EffectEqualization48x::TrackCompare()
    for (auto aTrack :
       mEffectEqualization->inputTracks()->Any< const WaveTrack >()) {
 
-      // Include selected tracks, plus sync-lock selected tracks for Track::All.
-      if (aTrack->GetSelected() ||
-         (// mEffectEqualization->mOutputTracksType == TrackKind::All &&
-          aTrack->IsSyncLockSelected()))
+      // Include selected tracks, plus sync-lock selected tracks
+      if (aTrack->GetSelected() || aTrack->IsSyncLockSelected())
       {
          auto o = mEffectEqualization->mFactory->DuplicateWaveTrack( *aTrack );
          SecondIMap.push_back(aTrack);
@@ -731,7 +726,7 @@ bool EffectEqualization48x::ProcessBuffer4x(BufferInfo *bufferInfo)
 
    auto blockCount=bufferInfo->mBufferLength/mBlockSize;
 
-   __m128 *readBlocks[4]; // some temps so we dont destroy the vars in the struct
+   __m128 *readBlocks[4]; // some temps so we don't destroy the vars in the struct
    __m128 *writeBlocks[4];
    for(int i=0;i<4;i++) {
       readBlocks[i]=(__m128 *)bufferInfo->mBufferSouce[i];
@@ -904,7 +899,7 @@ bool EffectEqualization48x::ProcessOne1x4xThreaded(int count, WaveTrack * t,
 
    if(blockCount<16) // it's not worth 4x processing do a regular process
       return ProcessOne4x(count, t, start, len);
-   if(mThreadCount<=0 || blockCount<256) // dont do it without cores or big data
+   if(mThreadCount<=0 || blockCount<256) // don't do it without cores or big data
       return ProcessOne4x(count, t, start, len);
 
    for(int i=0;i<mThreadCount;i++)
@@ -965,7 +960,7 @@ bool EffectEqualization48x::ProcessOne1x4xThreaded(int count, WaveTrack * t,
             currentSample-=mBlockSize+(mFilterSize>>1);
             mBufferInfo[currentIndex].mBufferStatus=BufferReady; // free for grabbin
             bigBlocksRead++;
-         } else mBufferInfo[currentIndex].mBufferStatus=BufferEmpty; // this is completely unecessary
+         } else mBufferInfo[currentIndex].mBufferStatus=BufferEmpty; // this is completely unnecessary
          currentIndex=(currentIndex+1)%mWorkerDataCount;
       } 
    }
@@ -1036,7 +1031,7 @@ bool EffectEqualization48x::ProcessBuffer8x(BufferInfo *bufferInfo)
 
    auto blockCount=bufferInfo->mBufferLength/mBlockSize;
 
-   __m128 *readBlocks[8]; // some temps so we dont destroy the vars in the struct
+   __m128 *readBlocks[8]; // some temps so we don't destroy the vars in the struct
    __m128 *writeBlocks[8];
    for(int i=0;i<8;i++) {
       readBlocks[i]=(__m128 *)bufferInfo->mBufferSouce[i];
@@ -1205,7 +1200,7 @@ bool EffectEqualization48x::ProcessOne8xThreaded(int count, WaveTrack * t,
 
    if(blockCount<16) // it's not worth 4x processing do a regular process
       return ProcessOne4x(count, t, start, len);
-   if(mThreadCount<=0 || blockCount<256) // dont do it without cores or big data
+   if(mThreadCount<=0 || blockCount<256) // don't do it without cores or big data
       return ProcessOne4x(count, t, start, len);
 
    auto output = t->EmptyCopy();
@@ -1261,7 +1256,7 @@ bool EffectEqualization48x::ProcessOne8xThreaded(int count, WaveTrack * t,
             currentSample-=mBlockSize+(mFilterSize>>1);
             mBufferInfo[currentIndex].mBufferStatus=BufferReady; // free for grabbin
             bigBlocksRead++;
-         } else mBufferInfo[currentIndex].mBufferStatus=BufferEmpty; // this is completely unecessary
+         } else mBufferInfo[currentIndex].mBufferStatus=BufferEmpty; // this is completely unnecessary
          currentIndex=(currentIndex+1)%mWorkerDataCount;
       } 
    }
