@@ -14,32 +14,32 @@
 
 *//*******************************************************************/
 
-#include "../Audacity.h"
+
 #include "Paulstretch.h"
 #include "LoadEffects.h"
 
 #include <algorithm>
 
 #include <math.h>
-#include <float.h>
 
 #include <wx/intl.h>
 #include <wx/valgen.h>
 
-#include "../Shuttle.h"
 #include "../ShuttleGui.h"
-#include "../FFT.h"
+#include "FFT.h"
 #include "../widgets/valnum.h"
 #include "../widgets/AudacityMessageBox.h"
-#include "../Prefs.h"
+#include "Prefs.h"
 
 #include "../WaveTrack.h"
 
-// Define keys, defaults, minimums, and maximums for the effect parameters
-//
-//     Name    Type     Key                     Def      Min      Max      Scale
-Param( Amount, float,   wxT("Stretch Factor"),   10.0,    1.0,     FLT_MAX, 1   );
-Param( Time,   float,   wxT("Time Resolution"),  0.25f,   0.00099f,  FLT_MAX, 1   );
+const EffectParameterMethods& EffectPaulstretch::Parameters() const
+{
+   static CapturedParameters<EffectPaulstretch,
+      Amount, Time
+   > parameters;
+   return parameters;
+}
 
 /// \brief Class that helps EffectPaulStretch.  It does the FFTs and inner loop 
 /// of the effect.
@@ -95,8 +95,7 @@ END_EVENT_TABLE()
 
 EffectPaulstretch::EffectPaulstretch()
 {
-   mAmount = DEF_Amount;
-   mTime_resolution = DEF_Time;
+   Parameters().Reset(*this);
 
    SetLinearEffectFlag(true);
 }
@@ -107,57 +106,32 @@ EffectPaulstretch::~EffectPaulstretch()
 
 // ComponentInterface implementation
 
-ComponentInterfaceSymbol EffectPaulstretch::GetSymbol()
+ComponentInterfaceSymbol EffectPaulstretch::GetSymbol() const
 {
    return Symbol;
 }
 
-TranslatableString EffectPaulstretch::GetDescription()
+TranslatableString EffectPaulstretch::GetDescription() const
 {
    return XO("Paulstretch is only for an extreme time-stretch or \"stasis\" effect");
 }
 
-wxString EffectPaulstretch::ManualPage()
+ManualPageID EffectPaulstretch::ManualPage() const
 {
-   return wxT("Paulstretch");
+   return L"Paulstretch";
 }
 
 // EffectDefinitionInterface implementation
 
-EffectType EffectPaulstretch::GetType()
+EffectType EffectPaulstretch::GetType() const
 {
    return EffectTypeProcess;
 }
 
-// EffectClientInterface implementation
-bool EffectPaulstretch::DefineParams( ShuttleParams & S ){
-   S.SHUTTLE_PARAM( mAmount, Amount );
-   S.SHUTTLE_PARAM( mTime_resolution, Time );
-   return true;
-}
-
-bool EffectPaulstretch::GetAutomationParameters(CommandParameters & parms)
-{
-   parms.WriteFloat(KEY_Amount, mAmount);
-   parms.WriteFloat(KEY_Time, mTime_resolution);
-
-   return true;
-}
-
-bool EffectPaulstretch::SetAutomationParameters(CommandParameters & parms)
-{
-   ReadAndVerifyFloat(Amount);
-   ReadAndVerifyFloat(Time);
-
-   mAmount = Amount;
-   mTime_resolution = Time;
-
-   return true;
-}
-
 // Effect implementation
 
-double EffectPaulstretch::CalcPreviewInputLength(double previewLength)
+double EffectPaulstretch::CalcPreviewInputLength(
+   const EffectSettings &, double previewLength) const
 {
    // FIXME: Preview is currently at the project rate, but should really be
    // at the track rate (bugs 1284 and 852).
@@ -170,7 +144,7 @@ double EffectPaulstretch::CalcPreviewInputLength(double previewLength)
 }
 
 
-bool EffectPaulstretch::Process()
+bool EffectPaulstretch::Process(EffectInstance &, EffectSettings &)
 {
    CopyInputTracks();
    m_t1=mT1;
@@ -196,26 +170,31 @@ bool EffectPaulstretch::Process()
 }
 
 
-void EffectPaulstretch::PopulateOrExchange(ShuttleGui & S)
+std::unique_ptr<EffectUIValidator> EffectPaulstretch::PopulateOrExchange(
+   ShuttleGui & S, EffectInstance &, EffectSettingsAccess &,
+   const EffectOutputs *)
 {
    S.StartMultiColumn(2, wxALIGN_CENTER);
    {
-      S.Validator<FloatingPointValidator<float>>(
-            1, &mAmount, NumValidatorStyle::DEFAULT, MIN_Amount)
+      S
+         .Validator<FloatingPointValidator<float>>(
+            1, &mAmount, NumValidatorStyle::DEFAULT, Amount.min)
          /* i18n-hint: This is how many times longer the sound will be, e.g. applying
           * the effect to a 1-second sample, with the default Stretch Factor of 10.0
           * will give an (approximately) 10 second sound
           */
          .AddTextBox(XXO("&Stretch Factor:"), wxT(""), 10);
 
-      S.Validator<FloatingPointValidator<float>>(
-            3, &mTime_resolution, NumValidatorStyle::ONE_TRAILING_ZERO, MIN_Time)
-         .AddTextBox(XXO("&Time Resolution (seconds):"), wxT(""), 10);
+      S
+         .Validator<FloatingPointValidator<float>>(
+            3, &mTime_resolution, NumValidatorStyle::ONE_TRAILING_ZERO, Time.min)
+         .AddTextBox(XXO("&Time Resolution (seconds):"), L"", 10);
    }
    S.EndMultiColumn();
+   return nullptr;
 };
 
-bool EffectPaulstretch::TransferDataToWindow()
+bool EffectPaulstretch::TransferDataToWindow(const EffectSettings &)
 {
    if (!mUIParent->TransferDataToWindow())
    {
@@ -225,7 +204,7 @@ bool EffectPaulstretch::TransferDataToWindow()
    return true;
 }
 
-bool EffectPaulstretch::TransferDataFromWindow()
+bool EffectPaulstretch::TransferDataFromWindow(EffectSettings &)
 {
    if (!mUIParent->Validate() || !mUIParent->TransferDataFromWindow())
    {
@@ -242,7 +221,7 @@ void EffectPaulstretch::OnText(wxCommandEvent & WXUNUSED(evt))
    EnableApply(mUIParent->TransferDataFromWindow());
 }
 
-size_t EffectPaulstretch::GetBufferSize(double rate)
+size_t EffectPaulstretch::GetBufferSize(double rate) const
 {
    // Audacity's fft requires a power of 2
    float tmp = rate * mTime_resolution / 2.0;
@@ -359,7 +338,7 @@ bool EffectPaulstretch::ProcessOne(WaveTrack *track,double t0,double t1,int coun
          decltype(len) s=0;
 
          while (s < len) {
-            track->Get((samplePtr)bufferptr0, floatSample, start + s, nget);
+            track->GetFloats(bufferptr0, start + s, nget);
             stretch.process(buffer0.get(), nget);
 
             if (first_time) {
@@ -369,7 +348,7 @@ bool EffectPaulstretch::ProcessOne(WaveTrack *track,double t0,double t1,int coun
             s += nget;
 
             if (first_time){//blend the start of the selection
-               track->Get((samplePtr)fade_track_smps.get(), floatSample, start, fade_len);
+               track->GetFloats(fade_track_smps.get(), start, fade_len);
                first_time = false;
                for (size_t i = 0; i < fade_len; i++){
                   float fi = (float)i / (float)fade_len;
@@ -378,7 +357,7 @@ bool EffectPaulstretch::ProcessOne(WaveTrack *track,double t0,double t1,int coun
                }
             }
             if (s >= len){//blend the end of the selection
-               track->Get((samplePtr)fade_track_smps.get(), floatSample, end - fade_len, fade_len);
+               track->GetFloats(fade_track_smps.get(), end - fade_len, fade_len);
                for (size_t i = 0; i < fade_len; i++){
                   float fi = (float)i / (float)fade_len;
                   auto i2 = bufsize / 2 - 1 - i;
@@ -462,7 +441,7 @@ void PaulStretch::process(float *smps, size_t nsmps)
    //get the samples from the pool
    for (size_t i = 0; i < poolsize; i++)
       fft_smps[i] = in_pool[i];
-   WindowFunc(eWinFuncHanning, poolsize, fft_smps.get());
+   WindowFunc(eWinFuncHann, poolsize, fft_smps.get());
 
    RealFFT(poolsize, fft_smps.get(), fft_c.get(), fft_s.get());
 
