@@ -162,7 +162,7 @@ bool EffectPhaser::RealtimeInitialize(EffectSettings &, double)
 }
 
 bool EffectPhaser::RealtimeAddProcessor(
-   EffectSettings &, unsigned, float sampleRate)
+   EffectSettings &, EffectOutputs *, unsigned, float sampleRate)
 {
    EffectPhaserState slave;
 
@@ -191,7 +191,8 @@ size_t EffectPhaser::RealtimeProcess(size_t group, EffectSettings &settings,
 // Effect implementation
 
 std::unique_ptr<EffectUIValidator> EffectPhaser::PopulateOrExchange(
-   ShuttleGui & S, EffectInstance &, EffectSettingsAccess &)
+   ShuttleGui & S, EffectInstance &, EffectSettingsAccess &,
+   const EffectOutputs *)
 {
    S.SetBorder(5);
    S.AddSpace(0, 5);
