@@ -18,6 +18,7 @@
 
 #include <chrono>
 #include <thread>
+#include <condition_variable>
 
 //! Mediator of two-way inter-thread communication of changes of settings
 class RealtimeEffectState::AccessState : public NonInterferingBase {
@@ -25,131 +26,166 @@ public:
    AccessState(const EffectSettingsManager &effect, RealtimeEffectState &state)
       : mEffect{ effect }
       , mState{ state }
-   {
-      Initialize(state.mMainSettings);
-   }
-
-   void Initialize(SettingsAndCounter &settings)
    {
       // Clean initial state of the counter
-      settings.counter = 0;
-      mLastSettings = settings;
-      // Initialize each message buffer with two copies of settings
-      mChannelToMain.Write(ToMainSlot{ settings });
-      mChannelToMain.Write(ToMainSlot{ settings });
-      mChannelFromMain.Write(FromMainSlot{ settings });
-      mChannelFromMain.Write(FromMainSlot{ settings });
+      state.mMainSettings.counter = 0;
+      Initialize(state.mMainSettings.settings,
+         state.mMessage.get(), state.mMovedOutputs.get());
    }
 
-   const SettingsAndCounter &MainRead() {
-      // Main thread clones the object in the std::any, then gives a reference
-      mChannelToMain.Read<ToMainSlot::Reader>(
-         mMainThreadCache, mState.mwInstance.lock());
-      return mMainThreadCache;
+   void Initialize(const EffectSettings &settings,
+      const EffectInstance::Message *pMessage,
+      const EffectOutputs *pOutputs)
+   {
+      mLastSettings = { settings, 0 };
+      // Initialize each message buffer with two copies
+      mChannelToMain.Write(ToMainSlot{ { 0,
+         pOutputs ? pOutputs->Clone() : nullptr } });
+      mChannelToMain.Write(ToMainSlot{ { 0,
+         pOutputs ? pOutputs->Clone() : nullptr } });
+      mChannelFromMain.Write(FromMainSlot{ settings, pMessage });
+      mChannelFromMain.Write(FromMainSlot{ settings, pMessage });
+
+      mMainThreadId = std::this_thread::get_id();
    }
-   void MainWrite(SettingsAndCounter &&settings) {
+
+   void MainRead() {
+      mChannelToMain.Read<ToMainSlot::Reader>(mState.mMovedOutputs.get(),
+         mCounter);
+   }
+   void MainWrite(SettingsAndCounter &&settings,
+      std::unique_ptr<EffectInstance::Message> pMessage) {
       // Main thread may simply swap new content into place
-      mChannelFromMain.Write(std::move(settings));
+      mChannelFromMain.Write(FromMainSlot::Message{
+         std::move(settings.settings), settings.counter, std::move(pMessage) });
    }
-   const SettingsAndCounter &
-   MainWriteThrough(const SettingsAndCounter &settings) {
-      // Main thread assumes worker isn't processing and bypasses the echo
-      return (mMainThreadCache = settings);
+   void MainWrite(SettingsAndCounter::Counter counter,
+      std::unique_ptr<EffectInstance::Message> pMessage) {
+      mChannelFromMain.Write(FromMainSlot::ShortMessage{
+         counter, std::move(pMessage) });
    }
 
-   struct EffectAndSettings{
-      const EffectSettingsManager &effect;
-      const SettingsAndCounter &settings;
+   struct CounterAndOutputs{
+      Response::Counter counter;
+      EffectOutputs *pOutputs{};
    };
    void WorkerRead() {
       // Worker thread avoids memory allocation.  It copies the contents of any
-      mChannelFromMain.Read<FromMainSlot::Reader>(
-         mEffect, mState.mWorkerSettings);
+      mChannelFromMain.Read<FromMainSlot::Reader>(mEffect, mState);
    }
    void WorkerWrite() {
-      // Worker thread avoids memory allocation.  It copies the contents of any
-      mChannelToMain.Write(EffectAndSettings{
-         mEffect, mState.mWorkerSettings });
-   }
 
-   const SettingsAndCounter &MainThreadCache() const { return mMainThreadCache; }
+      {
+         std::unique_lock lk(mLockForCV);
+
+         // Worker thread avoids memory allocation.
+         mChannelToMain.Write(CounterAndOutputs{
+            mState.mWorkerSettings.counter, mState.mOutputs.get() });
+      }
+      mCV.notify_one();
+   }
 
    struct ToMainSlot {
       // For initialization of the channel
       ToMainSlot() = default;
-      explicit ToMainSlot(const SettingsAndCounter &settings)
-         // Copy std::any
-         : mSettings{ settings }
+      explicit ToMainSlot(Response response)
+         : mResponse{ std::move(response) }
       {}
       ToMainSlot &operator=(ToMainSlot &&) = default;
 
       // Worker thread writes the slot
-      ToMainSlot& operator=(EffectAndSettings &&arg) {
-         mSettings.counter = arg.settings.counter;
+      ToMainSlot& operator=(CounterAndOutputs &&arg) {
          // This happens during MessageBuffer's busying of the slot
-         arg.effect.CopySettingsContents(
-            arg.settings.settings, mSettings.settings,
-            SettingsCopyDirection::WorkerToMain);
-         mSettings.settings.extra = arg.settings.settings.extra;
+         mResponse.counter = arg.counter;
+         if (mResponse.pOutputs && arg.pOutputs)
+            mResponse.pOutputs->Assign(std::move(*arg.pOutputs));
          return *this;
       }
 
-      // Main thread doesn't move out of the slot, but copies std::any
-      // and extra fields
-      struct Reader {
-         Reader(ToMainSlot &&slot, SettingsAndCounter &settings,
-            const std::shared_ptr<EffectInstance> pInstance
-         ) {
-            settings.counter = slot.mSettings.counter;
-            if (pInstance)
-               pInstance->AssignSettings(
-                  settings.settings, std::move(slot.mSettings.settings));
-         }
-      };
+      struct Reader { Reader(ToMainSlot &&slot, EffectOutputs *pOutputs,
+         Response::Counter &counter
+      ) {
+         // Main thread is not under the performance constraints of the
+         // worker, but Assign is still used so that
+         // members of underlying vectors or other containers do not
+         // relocate
+         if (pOutputs && slot.mResponse.pOutputs)
+            pOutputs->Assign(std::move(*slot.mResponse.pOutputs));
+         counter = slot.mResponse.counter;
+      } };
 
-      SettingsAndCounter mSettings;
+      Response mResponse;
    };
 
    struct FromMainSlot {
+      struct Message : SettingsAndCounter {
+         std::unique_ptr<EffectInstance::Message> pMessage;
+      };
+      struct ShortMessage {
+         SettingsAndCounter::Counter counter;
+         std::unique_ptr<EffectInstance::Message> pMessage;
+      };
+
       // For initialization of the channel
       FromMainSlot() = default;
-      explicit FromMainSlot(const SettingsAndCounter &settings)
+      explicit FromMainSlot(const EffectSettings &settings,
+         const EffectInstance::Message *pMessage)
          // Copy std::any
-         : mSettings{ settings }
+         : mMessage{ settings, 0, pMessage ? pMessage->Clone() : nullptr }
       {}
       FromMainSlot &operator=(FromMainSlot &&) = default;
 
       // Main thread writes the slot
-      FromMainSlot& operator=(SettingsAndCounter &&settings) {
-         mSettings.swap(settings);
+      FromMainSlot& operator=(Message &&message) {
+         mMessage.SettingsAndCounter::swap(message);
+         if (message.pMessage && mMessage.pMessage)
+            // Merge the incoming message with any still unconsumed message
+            mMessage.pMessage->Merge(std::move(*message.pMessage));
+         return *this;
+      }
+
+      // Main thread writes the slot
+      FromMainSlot& operator=(ShortMessage &&message) {
+         mMessage.counter = message.counter;
+         if (message.pMessage && mMessage.pMessage)
+            // Merge the incoming message with any still unconsumed message
+            mMessage.pMessage->Merge(std::move(*message.pMessage));
          return *this;
       }
 
       // Worker thread reads the slot
       struct Reader { Reader(FromMainSlot &&slot,
-         const EffectSettingsManager &effect, SettingsAndCounter &settings) {
-         if(slot.mSettings.counter == settings.counter)
+         const EffectSettingsManager &effect, RealtimeEffectState &state) {
+         auto &settings = state.mWorkerSettings;
+         if(slot.mMessage.counter == settings.counter)
             return;//copy once
+         settings.counter = slot.mMessage.counter;
 
-         settings.counter = slot.mSettings.counter;
-            // This happens during MessageBuffer's busying of the slot
-            effect.CopySettingsContents(
-               slot.mSettings.settings, settings.settings,
-               SettingsCopyDirection::MainToWorker);
-            settings.settings.extra = slot.mSettings.settings.extra;
+         // This happens during MessageBuffer's busying of the slot
+         effect.CopySettingsContents(
+            slot.mMessage.settings, settings.settings);
+         settings.settings.extra = slot.mMessage.settings.extra;
+         if (slot.mMessage.pMessage && state.mMovedMessage)
+            // Copy the message from the buffer (not a merge)
+            state.mMovedMessage->Assign(std::move(*slot.mMessage.pMessage));
       } };
 
-      SettingsAndCounter mSettings;
+      Message mMessage;
    };
 
    const EffectSettingsManager &mEffect;
    RealtimeEffectState &mState;
 
    MessageBuffer<FromMainSlot> mChannelFromMain;
-   SettingsAndCounter mMainThreadCache;
+   Response::Counter mCounter;
    SettingsAndCounter mLastSettings;
 
    MessageBuffer<ToMainSlot> mChannelToMain;
+
+   std::mutex mLockForCV;
+   std::condition_variable mCV;
+
+   std::thread::id mMainThreadId;
 };
 
 //! Main thread's interface to inter-thread communication of changes of settings
@@ -160,62 +196,107 @@ struct RealtimeEffectState::Access final : EffectSettingsAccess {
    {
    }
    ~Access() override = default;
-   // Try once to detect that the worker thread has echoed the last write
-   static const SettingsAndCounter *FlushAttempt(AccessState &state) {
-      auto &lastSettings = state.mLastSettings;
-      auto pResult = &state.MainRead();
-      if (!state.mState.mInitialized) {
-         // not relying on the other thread to make progress
-         // and no fear of data races
-         state.Initialize(lastSettings);
-         return &state.MainWriteThrough(lastSettings);
-      }
-      else if (!lastSettings.settings.has_value() ||
-         pResult->counter == lastSettings.counter
-      ){
-         // First time test, or echo is completed
-         return pResult;
-      }
-      return nullptr;
-   }
+
+
    const EffectSettings &Get() override {
       if (auto pState = mwState.lock()) {
          if (auto pAccessState = pState->GetAccessState()) {
-            FlushAttempt(*pAccessState); // try once, ignore success
-            auto &lastSettings = pAccessState->mLastSettings;
-            return lastSettings.settings.has_value()
-               ? lastSettings.settings
-               // If no value there yet, then FlushAttempt did MainRead which
-               // has copied the initial value given to the constructor
-               : pAccessState->MainThreadCache().settings;
+            if (pAccessState->mState.mInitialized)
+            {
+               // try once
+               assert(pAccessState->mState.mInitialized);
+               auto& lastSettings = pAccessState->mLastSettings;
+               // Assigns to mCounter
+               pAccessState->MainRead();
+            }
+            else {
+               // Not yet waiting on the other thread's progress
+               // Not necessarily values yet in the state's Settings objects
+            }
+            return pAccessState->mLastSettings.settings;
          }
       }
       // Non-modal dialog may have outlived the RealtimeEffectState
       static EffectSettings empty;
       return empty;
    }
-   void Set(EffectSettings &&settings) override {
-      if (auto pState = mwState.lock())
+   void Set(EffectSettings &&settings, std::unique_ptr<Message> pMessage)
+   override {
+      if (auto pState = mwState.lock()) {
          if (auto pAccessState = pState->GetAccessState()) {
+            if (pMessage && !pAccessState->mState.mInitialized) {
+               // Other thread isn't processing.
+               // Let the instance consume the message directly.
+               if (auto pInstance = pState->mwInstance.lock()) {
+                  auto &stateSettings = pState->mMainSettings.settings;
+                  stateSettings = std::move(settings);
+                  EffectInstance::MessagePackage package{
+                     stateSettings, pMessage.get()
+                  };
+                  pInstance->RealtimeProcessStart(package);
+                  pInstance->RealtimeProcessEnd(stateSettings);
+                  pAccessState->mLastSettings.settings = stateSettings;
+                  return;
+               }
+            }
             auto &lastSettings = pAccessState->mLastSettings;
             // move to remember values here
             lastSettings.settings = std::move(settings);
             ++lastSettings.counter;
             // move a copy to there
-            pAccessState->MainWrite(SettingsAndCounter{ lastSettings });
+            pAccessState->MainWrite(
+               SettingsAndCounter{ lastSettings }, std::move(pMessage));
          }
+      }
+   }
+   void Set(std::unique_ptr<Message> pMessage)
+   override {
+      if (auto pState = mwState.lock()) {
+         if (auto pAccessState = pState->GetAccessState()) {
+            if (pMessage && !pAccessState->mState.mInitialized) {
+               // Other thread isn't processing.
+               // Let the instance consume the message directly.
+               if (auto pInstance = pState->mwInstance.lock()) {
+                  auto &stateSettings = pState->mMainSettings.settings;
+                  EffectInstance::MessagePackage package{
+                     stateSettings, pMessage.get()
+                  };
+                  pInstance->RealtimeProcessStart(package);
+                  pInstance->RealtimeProcessEnd(stateSettings);
+                  // Don't need to update pAccessState->mLastSettings
+                  return;
+               }
+            }
+            auto &lastSettings = pAccessState->mLastSettings;
+            // Don't update settings, but do count
+            ++lastSettings.counter;
+            pAccessState->MainWrite(
+               lastSettings.counter, std::move(pMessage));
+         }
+      }
    }
    void Flush() override {
       if (auto pState = mwState.lock()) {
          if (auto pAccessState = pState->GetAccessState()) {
-            const SettingsAndCounter *pResult{};
-            while (!(pResult = FlushAttempt(*pAccessState))) {
-               // Wait for progress of audio thread
-               using namespace std::chrono;
-               std::this_thread::sleep_for(50ms);
+            assert(pAccessState->mMainThreadId == std::this_thread::get_id());
+            
+            if (pAccessState->mState.mInitialized)
+            {
+               std::unique_lock lk(pAccessState->mLockForCV);
+               pAccessState->mCV.wait(lk,
+                  [&] {
+                        auto& lastSettings = pAccessState->mLastSettings;
+                        pAccessState->MainRead();
+                        return pAccessState->mCounter == lastSettings.counter;
+                      }
+               );
             }
-            pState->mMainSettings.Set(*pResult); // Update the state
-            pAccessState->mLastSettings = *pResult;
+
+            // Update what GetSettings() will return, during play and before
+            // Finalize(), but after it is confirmed that any worker thread has
+            // seen the values given to the last Set().  These values will also
+            // be returned by Get().
+            pState->mMainSettings.Set(pAccessState->mLastSettings);
          }
       }
    }
@@ -270,11 +351,30 @@ const EffectInstanceFactory *RealtimeEffectState::GetEffect()
          mMainSettings.counter = 0;
          mMainSettings.settings = mPlugin->MakeSettings();
          mMainSettings.settings.extra.SetActive(wasActive);
+         mOutputs = mPlugin->MakeOutputs();
+         mMovedOutputs = mPlugin->MakeOutputs();
       }
    }
    return mPlugin;
 }
 
+std::shared_ptr<EffectInstance> RealtimeEffectState::MakeInstance()
+{
+   mMovedMessage.reset();
+   mMessage.reset();
+   auto result = mPlugin->MakeInstance();
+   if (result) {
+      // Allocate presized containers in messages, so later
+      // copies of contents might avoid free store operations
+      mMessage = result->MakeMessage();
+      mMovedMessage = result->MakeMessage();
+      if (auto state = GetAccessState())
+         state->Initialize(mMainSettings.settings,
+            mMessage.get(), mMovedOutputs.get());
+   }
+   return result;
+}
+
 std::shared_ptr<EffectInstance>
 RealtimeEffectState::EnsureInstance(double sampleRate)
 {
@@ -289,7 +389,7 @@ RealtimeEffectState::EnsureInstance(double sampleRate)
 
       //! If there was already an instance, recycle it; else make one here
       if (!pInstance)
-         mwInstance = pInstance = mPlugin->MakeInstance();
+         mwInstance = pInstance = MakeInstance();
       if (!pInstance)
          return {};
 
@@ -310,7 +410,7 @@ std::shared_ptr<EffectInstance> RealtimeEffectState::GetInstance()
    //! If there was already an instance, recycle it; else make one here
    auto pInstance = mwInstance.lock();
    if (!pInstance && mPlugin)
-      mwInstance = pInstance = mPlugin->MakeInstance();
+      mwInstance = pInstance = MakeInstance();
    return pInstance;
 }
 
@@ -368,10 +468,8 @@ RealtimeEffectState::AddTrack(Track &track, unsigned chans, float sampleRate)
    AllocateChannelsToProcessors(chans, numAudioIn, numAudioOut,
    [&](unsigned, unsigned){
       // Add a NEW processor
-      // Pass reference to worker settings, not main -- such as, for connecting
-      // Ladspa ports to the proper addresses.
       if (pInstance->RealtimeAddProcessor(
-         mWorkerSettings.settings, numAudioIn, sampleRate)
+         mWorkerSettings.settings, mOutputs.get(), numAudioIn, sampleRate)
       ) {
          mCurrentProcessor++;
          return true;
@@ -410,11 +508,22 @@ bool RealtimeEffectState::ProcessStart(bool running)
       mLastActive = active;
    }
 
+   bool result = false;
+   if (pInstance) {
+      // Consume messages even if not processing
+      // (issue #3855: plain UI for VST 2 effects)
+
+      // Assuming we are in a processing scope, use the worker settings
+      EffectInstance::MessagePackage package{
+         mWorkerSettings.settings, mMovedMessage.get()
+      };
+      result = pInstance->RealtimeProcessStart(package);
+   }
+
    if (!pInstance || !active)
       return false;
-
-   // Assuming we are in a processing scope, use the worker settings
-   return pInstance->RealtimeProcessStart(mWorkerSettings.settings);
+   else
+      return result;
 }
 
 #define stackAllocate(T, count) static_cast<T*>(alloca(count * sizeof(T)))
@@ -422,7 +531,8 @@ bool RealtimeEffectState::ProcessStart(bool running)
 //! Visit the effect processors that were added in AddTrack
 /*! The iteration over channels in AddTrack and Process must be the same */
 size_t RealtimeEffectState::Process(Track &track, unsigned chans,
-   const float *const *inbuf, float *const *outbuf, size_t numSamples)
+   const float *const *inbuf, float *const *outbuf, float *const dummybuf,
+   size_t numSamples)
 {
    auto pInstance = mwInstance.lock();
    if (!mPlugin || !pInstance || !mLastActive) {
@@ -457,7 +567,7 @@ size_t RealtimeEffectState::Process(Track &track, unsigned chans,
       std::copy(outbuf + ondx, outbuf + ondx + copied, clientOut);
       if (copied < numAudioOut) {
          // Make determinate pointers
-         std::fill(clientOut + copied, clientOut + numAudioOut, nullptr);
+         std::fill(clientOut + copied, clientOut + numAudioOut, dummybuf);
       }
 
       // Inner loop over blocks
@@ -498,9 +608,10 @@ size_t RealtimeEffectState::Process(Track &track, unsigned chans,
 bool RealtimeEffectState::ProcessEnd()
 {
    auto pInstance = mwInstance.lock();
-   bool result = pInstance && IsActive() && mLastActive &&
+   bool result = pInstance &&
       // Assuming we are in a processing scope, use the worker settings
-      pInstance->RealtimeProcessEnd(mWorkerSettings.settings);
+      pInstance->RealtimeProcessEnd(mWorkerSettings.settings) &&
+      IsActive() && mLastActive;
 
    if (auto pAccessState = TestAccessState())
       // Always done, regardless of activity
@@ -528,6 +639,7 @@ void RealtimeEffectState::SetActive(bool active)
    auto access = GetAccess();
    access->ModifySettings([&](EffectSettings &settings) {
       settings.extra.SetActive(active);
+      return nullptr;
    });
    access->Flush();
 
@@ -538,9 +650,6 @@ void RealtimeEffectState::SetActive(bool active)
 
 bool RealtimeEffectState::Finalize() noexcept
 {
-   // This is the main thread cleaning up a state not now used in processing
-   mMainSettings = mWorkerSettings;
-
    mGroups.clear();
    mCurrentProcessor = 0;
 
@@ -548,12 +657,12 @@ bool RealtimeEffectState::Finalize() noexcept
    if (!pInstance)
       return false;
 
+   if (!pInstance->UsesMessages()) {
+      // This is the main thread cleaning up a state not now used in processing
+      mMainSettings = mWorkerSettings;
+   }
+
    auto result = pInstance->RealtimeFinalize(mMainSettings.settings);
-   if(result)
-      //update mLastSettings so that Flush didn't
-      //overwrite what was read from the worker
-      if (auto state = GetAccessState())
-         state->Initialize(mMainSettings);
    mLatency = {};
    mInitialized = false;
    return result;
@@ -672,12 +781,17 @@ void RealtimeEffectState::WriteXML(XMLWriter &xmlFile)
 std::shared_ptr<EffectSettingsAccess> RealtimeEffectState::GetAccess()
 {
    if (!GetEffect())
-      // Effect not found!  Return a dummy
+      // Effect not found!
+      // Return a dummy
       return std::make_shared<Access>();
 
    // Only the main thread assigns to the atomic pointer, here and
    // once only in the lifetime of the state
    if (!GetAccessState())
+   {
+      MakeInstance();
       mpAccessState.emplace(*mPlugin, *this);
+   }
+
    return std::make_shared<Access>(*this);
 }
