@@ -14,7 +14,7 @@
 *//********************************************************************/
 
 
-#include "../Audacity.h"
+
 #include "Reverse.h"
 #include "LoadEffects.h"
 
@@ -23,6 +23,7 @@
 #include <wx/intl.h>
 
 #include "../LabelTrack.h"
+#include "../SyncLock.h"
 #include "../WaveClip.h"
 #include "../WaveTrack.h"
 
@@ -45,31 +46,31 @@ EffectReverse::~EffectReverse()
 
 // ComponentInterface implementation
 
-ComponentInterfaceSymbol EffectReverse::GetSymbol()
+ComponentInterfaceSymbol EffectReverse::GetSymbol() const
 {
    return Symbol;
 }
 
-TranslatableString EffectReverse::GetDescription()
+TranslatableString EffectReverse::GetDescription() const
 {
    return XO("Reverses the selected audio");
 }
 
 // EffectDefinitionInterface implementation
 
-EffectType EffectReverse::GetType()
+EffectType EffectReverse::GetType() const
 {
    return EffectTypeProcess;
 }
 
-bool EffectReverse::IsInteractive()
+bool EffectReverse::IsInteractive() const
 {
    return false;
 }
 
 // Effect implementation
 
-bool EffectReverse::Process()
+bool EffectReverse::Process(EffectInstance &, EffectSettings &)
 {
    //all needed because Reverse should move the labels too
    this->CopyInputTracks(true); // Set up mOutputTracks.
@@ -77,7 +78,7 @@ bool EffectReverse::Process()
    int count = 0;
 
    auto trackRange =
-      mOutputTracks->Any() + &Track::IsSelectedOrSyncLockSelected;
+      mOutputTracks->Any() + &SyncLock::IsSelectedOrSyncLockSelected;
    trackRange.VisitWhile( bGoodResult,
       [&](WaveTrack * track) {
          if (mT1 > mT0) {
@@ -113,8 +114,8 @@ bool EffectReverse::ProcessOneWave(int count, WaveTrack * track, sampleCount sta
    // Beware, the array grows as we loop over it.  Use integer subscripts, not iterators.
    for (size_t ii = 0; ii < clips.size(); ++ii) {
       const auto &clip = clips[ii].get();
-      auto clipStart = clip->GetStartSample();
-      auto clipEnd = clip->GetEndSample();
+      auto clipStart = clip->GetPlayStartSample();
+      auto clipEnd = clip->GetPlayEndSample();
       if (clipStart < start && clipEnd > start && clipEnd <= end) { // the reverse selection begins at the inside of a clip
          double splitTime = track->LongSamplesToTime(start);
          track->SplitAt(splitTime);
@@ -148,8 +149,8 @@ bool EffectReverse::ProcessOneWave(int count, WaveTrack * track, sampleCount sta
    for (i=0; i < clipArray.size(); i++) {
 
       WaveClip *clip = clipArray[i];
-      auto clipStart = clip->GetStartSample();
-      auto clipEnd = clip->GetEndSample();
+      auto clipStart = clip->GetPlayStartSample();
+      auto clipEnd = clip->GetPlayEndSample();
 
       if (clipStart >= start && clipEnd <= end) { // if the clip is inside the selected region
 
@@ -159,7 +160,7 @@ bool EffectReverse::ProcessOneWave(int count, WaveTrack * track, sampleCount sta
          if(checkedFirstClip == false && clipStart > start) {
             checkedFirstClip = true;
             if(i > 0) {
-               if (clipArray[i-1]->GetEndSample() <= start) {
+               if (clipArray[i-1]->GetPlayEndSample() <= start) {
                   currentEnd -= (clipStart - start);
                }
             }
@@ -182,12 +183,12 @@ bool EffectReverse::ProcessOneWave(int count, WaveTrack * track, sampleCount sta
             double offsetStartTime = track->LongSamplesToTime(clipOffsetStart);
             if(i+1 < clipArray.size()) // update currentEnd if there is a clip to process next
             {
-               auto nextClipStart = clipArray[i+1]->GetStartSample();
+               auto nextClipStart = clipArray[i+1]->GetPlayStartSample();
                currentEnd = currentEnd - (clipEnd - clipStart) - (nextClipStart - clipEnd);
             }
 
             revClips.push_back(track->RemoveAndReturnClip(clip)); // detach the clip from track
-            revClips.back()->SetOffset(track->LongSamplesToTime(track->TimeToLongSamples(offsetStartTime))); // align time to a sample and set offset
+            revClips.back()->SetPlayStartTime(track->LongSamplesToTime(track->TimeToLongSamples(offsetStartTime))); // align time to a sample and set offset
          }
       }
       else if (clipStart >= end) { // clip is after the selection region
@@ -201,12 +202,13 @@ bool EffectReverse::ProcessOneWave(int count, WaveTrack * track, sampleCount sta
    // PRL:  I don't think that matters, the sequence of storage of clips in the track
    // is not elsewhere assumed to be by time
    {
-      for (auto it = revClips.rbegin(), revEnd = revClips.rend(); it != revEnd; ++it)
-         track->AddClip(std::move(*it));
+      for (auto it = revClips.rbegin(), revEnd = revClips.rend(); rValue && it != revEnd; ++it)
+         rValue = track->AddClip(*it);
    }
 
    for (auto &clip : otherClips)
-      track->AddClip(std::move(clip));
+      if (!(rValue = track->AddClip(clip)))
+          break;
 
    return rValue;
 }
@@ -231,8 +233,8 @@ bool EffectReverse::ProcessOneClip(int count, WaveTrack *track,
          limitSampleBufferSize( track->GetBestBlockSize(first), len / 2 );
       auto second = first + (len - block);
 
-      track->Get((samplePtr)buffer1.get(), floatSample, first, block);
-      track->Get((samplePtr)buffer2.get(), floatSample, second, block);
+      track->GetFloats(buffer1.get(), first, block);
+      track->GetFloats(buffer2.get(), second, block);
       for (decltype(block) i = 0; i < block; i++) {
          tmp = buffer1[i];
          buffer1[i] = buffer2[block-i-1];
