@@ -9,8 +9,6 @@
 #ifndef __AUDACITY_SCORE_ALIGN_DIALOG__
 #define __AUDACITY_SCORE_ALIGN_DIALOG__
 
-#include "../Experimental.h"
-
 #ifdef EXPERIMENTAL_SCOREALIGN
 
 #include <wx/event.h>
@@ -55,7 +53,6 @@ extern int scorealign(
 
 class wxButton;
 class wxCheckBox;
-class wxString;
 class wxSlider;
 class wxStaticText;
 
