@@ -12,7 +12,7 @@
 
 **********************************************************************/
 
-#include "../Audacity.h" // for USE_* macros
+
 
 #if USE_SOUNDTOUCH
 
@@ -27,16 +27,13 @@ namespace soundtouch { class SoundTouch; }
 
 
 class TimeWarper;
+class NoteTrack;
 class WaveTrack;
 
-class EffectSoundTouch /* not final */ : public Effect
+class EffectSoundTouch /* not final */ : public StatefulEffect
 {
 public:
    
-   // Effect implementation
-
-   void End() override;
-
    // EffectSoundTouch implementation
 
 #ifdef USE_MIDI
@@ -48,9 +45,11 @@ public:
 protected:
    // Effect implementation
 
-   bool ProcessWithTimeWarper(const TimeWarper &warper);
+   using InitFunction = std::function< void(soundtouch::SoundTouch *soundtouch) >;
+   bool ProcessWithTimeWarper(InitFunction initer,
+                              const TimeWarper &warper,
+                              bool preserveLength);
 
-   std::unique_ptr<soundtouch::SoundTouch> mSoundTouch;
    double mCurT0;
    double mCurT1;
 
@@ -59,15 +58,20 @@ private:
 #ifdef USE_MIDI
    bool ProcessNoteTrack(NoteTrack *track, const TimeWarper &warper);
 #endif
-   bool ProcessOne(
+   bool ProcessOne(soundtouch::SoundTouch *pSoundTouch,
       WaveTrack * t, sampleCount start, sampleCount end,
       const TimeWarper &warper);
-   bool ProcessStereo(WaveTrack* leftTrack, WaveTrack* rightTrack,
-                     sampleCount start, sampleCount end,
-                      const TimeWarper &warper);
-   bool ProcessStereoResults(const size_t outputCount,
-                              WaveTrack* outputLeftTrack,
-                              WaveTrack* outputRightTrack);
+   bool ProcessStereo(soundtouch::SoundTouch *pSoundTouch,
+      WaveTrack* leftTrack, WaveTrack* rightTrack,
+      sampleCount start, sampleCount end,
+      const TimeWarper &warper);
+   bool ProcessStereoResults(soundtouch::SoundTouch *pSoundTouch,
+      const size_t outputCount,
+      WaveTrack* outputLeftTrack,
+      WaveTrack* outputRightTrack);
+   void Finalize(WaveTrack* orig, WaveTrack* out, const TimeWarper &warper);
+
+   bool   mPreserveLength;
 
    int    mCurTrackNum;
 
