@@ -18,36 +18,55 @@ doing the second pass over all selected tracks.
 *//*******************************************************************/
 
 
-#include "../Audacity.h"
+
 #include "TwoPassSimpleMono.h"
 
 #include "../WaveTrack.h"
 
-bool EffectTwoPassSimpleMono::Process()
+bool EffectTwoPassSimpleMono::Process(EffectInstance &, EffectSettings &settings)
 {
-    mPass = 0;
-    mSecondPassDisabled = false;
+   mPass = 0;
+   mSecondPassDisabled = false;
 
-    InitPass1();
-    this->CopyInputTracks(); // Set up mOutputTracks.
-    bool bGoodResult = ProcessPass();
+   InitPass1();
+   this->CopyInputTracks(); // Set up mOutputTracks.
 
-    if (bGoodResult && !mSecondPassDisabled)
-    {
-        mPass = 1;
-        if (InitPass2())
-            bGoodResult = ProcessPass();
-    }
+   mWorkTracks = TrackList::Create(
+      const_cast<AudacityProject*>( FindProject() ) );
+   for (auto track : mOutputTracks->Selected< WaveTrack >()) {
+      mWorkTracks->Add(track->EmptyCopy())->ConvertToSampleFormat(floatSample);
+      if( mT0 > 0 )
+         mWorkTracks->back()->InsertSilence(0, mT0);
+   }
 
-    this->ReplaceProcessedTracks(bGoodResult);
-    return bGoodResult;
+   mTrackLists[0] = &mOutputTracks;
+   mTrackLists[1] = mSecondPassDisabled ? &mOutputTracks : &mWorkTracks;
+
+   bool bGoodResult = ProcessPass(settings);
+
+   if (bGoodResult && !mSecondPassDisabled)
+   {
+      mPass = 1;
+      if (InitPass2())
+         bGoodResult = ProcessPass(settings);
+   }
+
+   mWorkTracks->Clear();
+   mWorkTracks.reset();
+
+   this->ReplaceProcessedTracks(bGoodResult);
+   return bGoodResult;
 }
 
-bool EffectTwoPassSimpleMono::ProcessPass()
+bool EffectTwoPassSimpleMono::ProcessPass(EffectSettings &settings)
 {
    //Iterate over each track
    mCurTrackNum = 0;
-   for( auto track : mOutputTracks->Selected< WaveTrack >() ) {
+
+   auto outTracks = (*mTrackLists[1 - mPass])->Selected< WaveTrack >().begin();
+   for( auto track : (*mTrackLists[mPass])->Selected< WaveTrack >() ) {
+      auto outTrack = *outTracks;
+
       //Get start and end times from track
       double trackStart = track->GetStartTime();
       double trackEnd = track->GetEndTime();
@@ -78,11 +97,12 @@ bool EffectTwoPassSimpleMono::ProcessPass()
             return false;
 
          //ProcessOne() (implemented below) processes a single track
-         if (!ProcessOne(track, start, end))
+         if (!ProcessOne(track, outTrack, start, end))
             return false;
       }
 
       mCurTrackNum++;
+      outTracks++;
    }
 
    return true;
@@ -91,7 +111,7 @@ bool EffectTwoPassSimpleMono::ProcessPass()
 
 //ProcessOne() takes a track, transforms it to bunch of buffer-blocks,
 //and executes TwoBufferProcessPass1 or TwoBufferProcessPass2 on these blocks
-bool EffectTwoPassSimpleMono::ProcessOne(WaveTrack * track,
+bool EffectTwoPassSimpleMono::ProcessOne(WaveTrack * track, WaveTrack * outTrack,
                                          sampleCount start, sampleCount end)
 {
    bool ret;
@@ -110,7 +130,7 @@ bool EffectTwoPassSimpleMono::ProcessOne(WaveTrack * track,
       std::min( maxblock, track->GetBestBlockSize(start) ), end - start );
 
    //Get the samples from the track and put them in the buffer
-   track->Get((samplePtr) buffer1.get(), floatSample, start, samples1);
+   track->GetFloats(buffer1.get(), start, samples1);
 
    // Process the first buffer with a NULL previous buffer
    if (mPass == 0)
@@ -132,7 +152,7 @@ bool EffectTwoPassSimpleMono::ProcessOne(WaveTrack * track,
       );
 
       //Get the samples from the track and put them in the buffer
-      track->Get((samplePtr)buffer2.get(), floatSample, s, samples2);
+      track->GetFloats(buffer2.get(), s, samples2);
 
       //Process the buffer.  If it fails, clean up and exit.
       if (mPass == 0)
@@ -145,7 +165,12 @@ bool EffectTwoPassSimpleMono::ProcessOne(WaveTrack * track,
 
       //Processing succeeded. copy the newly-changed samples back
       //onto the track.
-      track->Set((samplePtr)buffer1.get(), floatSample, s - samples1, samples1);
+      if (mSecondPassDisabled || mPass != 0) {
+         outTrack->Set((samplePtr)buffer1.get(), floatSample, s - samples1, samples1);
+      }
+      else {
+         outTrack->Append((samplePtr)buffer1.get(), floatSample, samples1);
+      }
 
       //Increment s one blockfull of samples
       s += samples2;
@@ -181,7 +206,13 @@ bool EffectTwoPassSimpleMono::ProcessOne(WaveTrack * track,
 
    //Processing succeeded. copy the newly-changed samples back
    //onto the track.
-   track->Set((samplePtr)buffer1.get(), floatSample, s - samples1, samples1);
+   if (mSecondPassDisabled || mPass != 0) {
+      outTrack->Set((samplePtr)buffer1.get(), floatSample, s - samples1, samples1);
+   }
+   else {
+      outTrack->Append((samplePtr)buffer1.get(), floatSample, samples1);
+      outTrack->Flush();
+   }
 
    //Return true because the effect processing succeeded.
    return true;
