@@ -120,6 +120,7 @@ bool EffectWahwah::Validator::ValidateUI()
       {
          // pass back the modified settings to the MessageBuffer
          GetSettings(settings) = mSettings;
+         return nullptr;
       }
    );
 
@@ -146,7 +147,7 @@ struct EffectWahwah::Instance
    bool RealtimeInitialize(EffectSettings& settings, double) override;
 
    bool RealtimeAddProcessor(EffectSettings& settings,
-      unsigned numChannels, float sampleRate) override;
+      EffectOutputs *pOutputs, unsigned numChannels, float sampleRate) override;
 
    bool RealtimeFinalize(EffectSettings& settings) noexcept override;
 
@@ -234,7 +235,7 @@ bool EffectWahwah::Instance::RealtimeInitialize(EffectSettings &, double)
 }
 
 bool EffectWahwah::Instance::RealtimeAddProcessor(
-   EffectSettings &settings, unsigned, float sampleRate)
+   EffectSettings &settings, EffectOutputs *, unsigned, float sampleRate)
 {
    EffectWahwahState slave;
 
@@ -263,7 +264,8 @@ size_t EffectWahwah::Instance::RealtimeProcess(size_t group, EffectSettings &set
 // Effect implementation
 
 std::unique_ptr<EffectUIValidator> EffectWahwah::PopulateOrExchange(
-   ShuttleGui & S, EffectInstance &, EffectSettingsAccess &access)
+   ShuttleGui & S, EffectInstance &, EffectSettingsAccess &access,
+   const EffectOutputs *)
 {
    auto& settings = access.Get();
    auto& myEffSettings = GetSettings(settings);
