@@ -12,7 +12,7 @@
 #define __AUDACITY_EXPORT_MULTIPLE__
 
 #include "Export.h"
-#include "../wxFileNameWrapper.h" // member variable
+#include "wxFileNameWrapper.h" // member variable
 
 class wxButton;
 class wxCheckBox;
@@ -29,9 +29,10 @@ class SelectionState;
 class ShuttleGui;
 class Track;
 
-class ExportMultipleDialog final : public wxDialogWrapper
+class AUDACITY_DLL_API ExportMultipleDialog final : public wxDialogWrapper
 {
 public:
+   using ProgressResult = BasicUI::ProgressResult;
 
    ExportMultipleDialog(AudacityProject *parent);
    virtual ~ExportMultipleDialog();
@@ -72,7 +73,7 @@ private:
     * @param t1 End time for export
     * @param tags Metadata to include in the file (if possible).
     */
-   ProgressResult DoExport(std::unique_ptr<ProgressDialog> &pDialog,
+   ProgressResult DoExport(std::unique_ptr<BasicUI::ProgressDialog> &pDialog,
                  unsigned channels,
                  const wxFileName &name,
                  bool selectedOnly,
