@@ -13,10 +13,10 @@
 
 **********************************************************************/
 
-#include "../Audacity.h"
+
 #include "RawAudioGuess.h"
 
-#include "../AudacityException.h"
+#include "AudacityException.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -209,7 +209,12 @@ static void Extract(bool bits16,
    }
 
    if( dataSizeIn < 1 )
-      throw SimpleMessageBoxException{XO("Bad data size")};
+      throw SimpleMessageBoxException{
+         ExceptionType::BadUserAction,
+         XO("Bad data size. Could not import audio"),
+         XO("Warning"), 
+         "Error:_Importing_raw_audio"
+      };
 
    size_t dataSize = (size_t)dataSizeIn;
 
