@@ -1,11 +1,11 @@
 #include "../CommonCommandFlags.h"
-#include "../ProjectHistory.h"
+#include "ProjectHistory.h"
 #include "../ProjectSettings.h"
 #include "../TrackPanelAx.h"
 #include "../ProjectWindow.h"
-#include "../UndoManager.h"
+#include "UndoManager.h"
 #include "../WaveClip.h"
-#include "../ViewInfo.h"
+#include "ViewInfo.h"
 #include "../WaveTrack.h"
 #include "../commands/CommandContext.h"
 #include "../commands/CommandManager.h"
@@ -19,24 +19,20 @@ struct FoundTrack {
    int trackNum{};
    bool channel{};
 
-   TranslatableString ComposeTrackName() const
+   wxString ComposeTrackName() const
    {
-      auto name = waveTrack->GetName();
-      auto shortName = name == waveTrack->GetDefaultName()
-         /* i18n-hint: compose a name identifying an unnamed track by number */
-         ? XO("Track %d").Format( trackNum )
-         : Verbatim(name);
-      auto longName = shortName;
+      /* i18n-hint: The %d is replaced by the number of the track.*/
+      auto shortName = wxString::Format(_("Track %d"), trackNum).Append(" " + waveTrack->GetName());
       if (channel) {
          // TODO: more-than-two-channels-message
          if ( waveTrack->IsLeader() )
          /* i18n-hint: given the name of a track, specify its left channel */
-            longName = XO("%s left").Format(shortName);
+            return XO("%s left").Translation().Format(shortName);
          else
          /* i18n-hint: given the name of a track, specify its right channel */
-            longName = XO("%s right").Format(shortName);
+            return XO("%s right").Translation().Format(shortName);
       }
-      return longName;
+      return shortName;
    }
 };
 
@@ -44,6 +40,7 @@ struct FoundClip : FoundTrack {
    bool found{};
    double startTime{};
    double endTime{};
+   wxString name{};
    int index{};
 };
 
@@ -51,8 +48,10 @@ struct FoundClipBoundary : FoundTrack {
    int nFound{};    // 0, 1, or 2
    double time{};
    int index1{};
+   wxString name1{};
    bool clipStart1{};
    int index2{};
+   wxString name2{};
    bool clipStart2{};
 };
 
@@ -71,8 +70,8 @@ bool TwoChannelsHaveSameBoundaries
    if (left.size() == right.size()) {
       sameClips = true;
       for (unsigned int i = 0; i < left.size(); i++) {
-         if (left[i]->GetStartTime() != right[i]->GetStartTime() ||
-            left[i]->GetEndTime() != right[i]->GetEndTime()) {
+         if (left[i]->GetPlayStartTime() != right[i]->GetPlayStartTime() ||
+            left[i]->GetPlayEndTime() != right[i]->GetPlayEndTime()) {
             sameClips = false;
             break;
          }
@@ -97,8 +96,8 @@ bool ChannelsHaveDifferentClipBoundaries(
    return false;
 }
 
-// When two clips are immediately next to each other, the GetEndTime() of the
-// first clip and the GetStartTime() of the second clip may not be exactly equal
+// When two clips are immediately next to each other, the GetPlayEndTime() of the 
+// first clip and the GetPlayStartTime() of the second clip may not be exactly equal
 // due to rounding errors. When searching for the next/prev start time from a
 // given time, the following function adjusts that given time if necessary to
 // take this into account. If the given time is the end time of the first of two
@@ -110,17 +109,17 @@ double AdjustForFindingStartTimes(
 {
    auto q = std::find_if(clips.begin(), clips.end(),
       [&] (const WaveClip* const& clip) {
-         return clip->GetEndTime() == time; });
+         return clip->GetPlayEndTime() == time; });
    if (q != clips.end() && q + 1 != clips.end() &&
       (*q)->SharesBoundaryWithNextClip(*(q+1))) {
-      time = (*(q+1))->GetStartTime();
+      time = (*(q+1))->GetPlayStartTime();
    }
 
    return time;
 }
 
-// When two clips are immediately next to each other, the GetEndTime() of the
-// first clip and the GetStartTime() of the second clip may not be exactly equal
+// When two clips are immediately next to each other, the GetPlayEndTime() of the
+// first clip and the GetPlayStartTime() of the second clip may not be exactly equal
 // due to rounding errors. When searching for the next/prev end time from a
 // given time, the following function adjusts that given time if necessary to
 // take this into account. If the given time is the start time of the second of
@@ -132,10 +131,10 @@ double AdjustForFindingEndTimes(
 {
    auto q = std::find_if(clips.begin(), clips.end(),
       [&] (const WaveClip* const& clip) {
-         return clip->GetStartTime() == time; });
+         return clip->GetPlayStartTime() == time; });
    if (q != clips.end() && q != clips.begin() &&
       (*(q - 1))->SharesBoundaryWithNextClip(*q)) {
-      time = (*(q-1))->GetEndTime();
+      time = (*(q-1))->GetPlayEndTime();
    }
 
    return time;
@@ -152,38 +151,43 @@ FoundClipBoundary FindNextClipBoundary
 
    auto pStart = std::find_if(clips.begin(), clips.end(),
       [&] (const WaveClip* const& clip) {
-         return clip->GetStartTime() > timeStart; });
+         return clip->GetPlayStartTime() > timeStart; });
    auto pEnd = std::find_if(clips.begin(), clips.end(),
       [&] (const WaveClip* const& clip) {
-         return clip->GetEndTime() > timeEnd; });
+         return clip->GetPlayEndTime() > timeEnd; });
 
    if (pStart != clips.end() && pEnd != clips.end()) {
       if ((*pEnd)->SharesBoundaryWithNextClip(*pStart)) {
          // boundary between two clips which are immediately next to each other.
          result.nFound = 2;
-         result.time = (*pEnd)->GetEndTime();
+         result.time = (*pEnd)->GetPlayEndTime();
          result.index1 = std::distance(clips.begin(), pEnd);
+         result.name1 = (*pEnd)->GetName();
          result.clipStart1 = false;
          result.index2 = std::distance(clips.begin(), pStart);
+         result.name2 = (*pStart)->GetName();
          result.clipStart2 = true;
       }
-      else if ((*pStart)->GetStartTime() < (*pEnd)->GetEndTime()) {
+      else if ((*pStart)->GetPlayStartTime() < (*pEnd)->GetPlayEndTime()) {
          result.nFound = 1;
-         result.time = (*pStart)->GetStartTime();
+         result.time = (*pStart)->GetPlayStartTime();
          result.index1 = std::distance(clips.begin(), pStart);
+         result.name1 = (*pStart)->GetName();
          result.clipStart1 = true;
       }
       else  {
          result.nFound = 1;
-         result.time = (*pEnd)->GetEndTime();
+         result.time = (*pEnd)->GetPlayEndTime();
          result.index1 = std::distance(clips.begin(), pEnd);
+         result.name1 = (*pEnd)->GetName();
          result.clipStart1 = false;
       }
    }
    else if (pEnd != clips.end()) {
       result.nFound = 1;
-      result.time = (*pEnd)->GetEndTime();
+      result.time = (*pEnd)->GetPlayEndTime();
       result.index1 = std::distance(clips.begin(), pEnd);
+      result.name1 = (*pEnd)->GetName();
       result.clipStart1 = false;
    }
 
@@ -200,48 +204,53 @@ FoundClipBoundary FindPrevClipBoundary(const WaveTrack* wt, double time)
 
    auto pStart = std::find_if(clips.rbegin(), clips.rend(),
       [&] (const WaveClip* const& clip) {
-         return clip->GetStartTime() < timeStart; });
+         return clip->GetPlayStartTime() < timeStart; });
    auto pEnd = std::find_if(clips.rbegin(), clips.rend(),
       [&] (const WaveClip* const& clip) {
-         return clip->GetEndTime() < timeEnd; });
+         return clip->GetPlayEndTime() < timeEnd; });
 
    if (pStart != clips.rend() && pEnd != clips.rend()) {
       if ((*pEnd)->SharesBoundaryWithNextClip(*pStart)) {
          // boundary between two clips which are immediately next to each other.
          result.nFound = 2;
-         result.time = (*pStart)->GetStartTime();
+         result.time = (*pStart)->GetPlayStartTime();
          result.index1 =
             static_cast<int>(clips.size()) - 1 -
                std::distance(clips.rbegin(), pStart);
+         result.name1 = (*pStart)->GetName();
          result.clipStart1 = true;
          result.index2 =
             static_cast<int>(clips.size()) - 1 -
                std::distance(clips.rbegin(), pEnd);
+         result.name2 = (*pEnd)->GetName();
          result.clipStart2 = false;
       }
-      else if ((*pStart)->GetStartTime() > (*pEnd)->GetEndTime()) {
+      else if ((*pStart)->GetPlayStartTime() > (*pEnd)->GetPlayEndTime()) {
          result.nFound = 1;
-         result.time = (*pStart)->GetStartTime();
+         result.time = (*pStart)->GetPlayStartTime();
          result.index1 =
             static_cast<int>(clips.size()) - 1 -
                std::distance(clips.rbegin(), pStart);
+         result.name1 = (*pStart)->GetName();
          result.clipStart1 = true;
       }
       else {
          result.nFound = 1;
-         result.time = (*pEnd)->GetEndTime();
+         result.time = (*pEnd)->GetPlayEndTime();
          result.index1 =
             static_cast<int>(clips.size()) - 1 -
                std::distance(clips.rbegin(), pEnd);
+         result.name1 = (*pEnd)->GetName();
          result.clipStart1 = false;
       }
    }
    else if (pStart != clips.rend()) {
       result.nFound = 1;
-      result.time = (*pStart)->GetStartTime();
+      result.time = (*pStart)->GetPlayStartTime();
       result.index1 =
          static_cast<int>(clips.size()) - 1 -
             std::distance(clips.rbegin(), pStart);
+      result.name1 = (*pStart)->GetName();
       result.clipStart1 = true;
    }
 
@@ -322,17 +331,19 @@ TranslatableString ClipBoundaryMessage(
             /* i18n-hint:
                First %s is replaced with the noun "start" or "end"
                identifying one end of a clip,
+               second string is the name of that clip,
                first number gives the position of that clip in a sequence
                of clips,
                last number counts all clips,
                and the last string is the name of the track containing the
                clips.
              */
-            "%s %d of %d clip %s",
-            "%s %d of %d clips %s",
-            2
+            "%s %s, %d of %d clip %s",
+            "%s %s, %d of %d clips %s",
+            3
          )(
             result.clipStart1 ? XO("start") : XO("end"),
+            result.name1,
             result.index1 + 1,
             nClips,
             longName
@@ -341,21 +352,24 @@ TranslatableString ClipBoundaryMessage(
       else {
          str = XP(
             /* i18n-hint:
-               First two %s are each replaced with the noun "start"
+               First and third %s are each replaced with the noun "start"
                or with "end", identifying and end of a clip,
+               second and fourth strings are the names of those clips,
                first and second numbers give the position of those clips in
                a sequence of clips,
                last number counts all clips,
                and the last string is the name of the track containing the
                clips.
              */
-            "%s %d and %s %d of %d clip %s",
-            "%s %d and %s %d of %d clips %s",
-            4
+            "%s %s and %s %s, %d and %d of %d clip %s",
+            "%s %s and %s %s, %d and %d of %d clips %s",
+            6
          )(
             result.clipStart1 ? XO("start") : XO("end"),
-            result.index1 + 1,
+            result.name1,
             result.clipStart2 ? XO("start") : XO("end"),
+            result.name2,
+            result.index1 + 1,
             result.index2 + 1,
             nClips,
             longName
@@ -409,11 +423,12 @@ FoundClip FindNextClip
    {
       auto p = std::find_if(clips.begin(), clips.end(),
          [&] (const WaveClip* const& clip) {
-            return clip->GetStartTime() == t0; });
-      if (p != clips.end() && (*p)->GetEndTime() > t1) {
+            return clip->GetPlayStartTime() == t0; });
+      if (p != clips.end() && (*p)->GetPlayEndTime() > t1) {
          result.found = true;
-         result.startTime = (*p)->GetStartTime();
-         result.endTime = (*p)->GetEndTime();
+         result.startTime = (*p)->GetPlayStartTime();
+         result.endTime = (*p)->GetPlayEndTime();
+         result.name = (*p)->GetName();
          result.index = std::distance(clips.begin(), p);
          return result;
       }
@@ -422,11 +437,12 @@ FoundClip FindNextClip
    {
       auto p = std::find_if(clips.begin(), clips.end(),
          [&] (const WaveClip* const& clip) {
-            return clip->GetStartTime() > t0; });
+            return clip->GetPlayStartTime() > t0; });
       if (p != clips.end()) {
          result.found = true;
-         result.startTime = (*p)->GetStartTime();
-         result.endTime = (*p)->GetEndTime();
+         result.startTime = (*p)->GetPlayStartTime();
+         result.endTime = (*p)->GetPlayEndTime();
+         result.name = (*p)->GetName();
          result.index = std::distance(clips.begin(), p);
          return result;
       }
@@ -449,11 +465,12 @@ FoundClip FindPrevClip
    {
       auto p = std::find_if(clips.begin(), clips.end(),
          [&] (const WaveClip* const& clip) {
-            return clip->GetStartTime() == t0; });
-      if (p != clips.end() && (*p)->GetEndTime() < t1) {
+            return clip->GetPlayStartTime() == t0; });
+      if (p != clips.end() && (*p)->GetPlayEndTime() < t1) {
          result.found = true;
-         result.startTime = (*p)->GetStartTime();
-         result.endTime = (*p)->GetEndTime();
+         result.startTime = (*p)->GetPlayStartTime();
+         result.endTime = (*p)->GetPlayEndTime();
+         result.name = (*p)->GetName();
          result.index = std::distance(clips.begin(), p);
          return result;
       }
@@ -462,11 +479,12 @@ FoundClip FindPrevClip
    {
       auto p = std::find_if(clips.rbegin(), clips.rend(),
          [&] (const WaveClip* const& clip) {
-            return clip->GetStartTime() < t0; });
+            return clip->GetPlayStartTime() < t0; });
       if (p != clips.rend()) {
          result.found = true;
-         result.startTime = (*p)->GetStartTime();
-         result.endTime = (*p)->GetEndTime();
+         result.startTime = (*p)->GetPlayStartTime();
+         result.endTime = (*p)->GetPlayEndTime();
+         result.name = (*p)->GetName();
          result.index =
             static_cast<int>(clips.size()) - 1 -
                std::distance(clips.rbegin(), p);
@@ -581,13 +599,16 @@ void DoSelectClip(AudacityProject &project, bool next)
          auto nClips = result.waveTrack->GetNumClips();
          auto str = XP(
             /* i18n-hint:
-               first number identifies one of a sequence of clips,
-               last number counts the clips,
-               string names a track */
-            "%d of %d clip %s",
-            "%d of %d clips %s",
-            1
+               first string is the name of a clip,
+               first number gives the position of that clip
+               in a sequence of clips,
+               last number counts all clips,
+               last string names a track */
+            "%s, %d of %d clip %s",
+            "%s, %d of %d clips %s",
+            2
          )(
+            result.name,
             result.index + 1,
             nClips,
             longName
@@ -627,64 +648,55 @@ void DoCursorClipBoundary
 }
 
 // This function returns the amount moved.  Possibly 0.0.
-double DoClipMove
-   ( ViewInfo &viewInfo, Track *track,
+double DoClipMove( AudacityProject &project, Track *track,
      TrackList &trackList, bool syncLocked, bool right )
 {
+   auto &viewInfo = ViewInfo::Get(project);
    auto &selectedRegion = viewInfo.selectedRegion;
 
-   // just dealing with clips in wave tracks for the moment. Note tracks??
-   if (track) return track->TypeSwitch<double>( [&]( WaveTrack *wt ) {
+   if (track) {
       ClipMoveState state;
 
       auto t0 = selectedRegion.t0();
 
+      std::unique_ptr<TrackShifter> uShifter;
+
       // Find the first channel that has a clip at time t0
-      for (auto channel : TrackList::Channels(wt) ) {
-         if( nullptr != (state.capturedClip = channel->GetClipAtTime( t0 )) ) {
-            wt = channel;
+      auto hitTestResult = TrackShifter::HitTestResult::Track;
+      for (auto channel : TrackList::Channels(track) ) {
+         uShifter = MakeTrackShifter::Call( *channel, project );
+         if ( (hitTestResult = uShifter->HitTest( t0, viewInfo )) ==
+             TrackShifter::HitTestResult::Miss )
+            uShifter.reset();
+         else
             break;
-         }
       }
 
-      if (state.capturedClip == nullptr)
+      if (!uShifter)
          return 0.0;
-
-      state.capturedClipIsSelection =
-         track->GetSelected() && !selectedRegion.isPoint();
-      state.trackExclusions.clear();
-
-      TimeShiftHandle::CreateListOfCapturedClips(
-         state, viewInfo, *track, trackList, syncLocked, t0 );
-
+      auto pShifter = uShifter.get();
       auto desiredT0 = viewInfo.OffsetTimeByPixels( t0, ( right ? 1 : -1 ) );
-      auto desiredSlideAmount = desiredT0 - t0;
+      auto desiredSlideAmount = pShifter->HintOffsetLarger( desiredT0 - t0 );
 
-      // set it to a sample point, and minimum of 1 sample point
-      if (!right)
-         desiredSlideAmount *= -1;
-      double nSamples = rint(wt->GetRate() * desiredSlideAmount);
-      nSamples = std::max(nSamples, 1.0);
-      desiredSlideAmount = nSamples / wt->GetRate();
-      if (!right)
-         desiredSlideAmount *= -1;
+      state.Init( project, pShifter->GetTrack(), hitTestResult, std::move( uShifter ),
+         t0, viewInfo, trackList, syncLocked );
 
-      state.hSlideAmount = desiredSlideAmount;
-      TimeShiftHandle::DoSlideHorizontal( state, trackList, *track );
+      auto hSlideAmount = state.DoSlideHorizontal( desiredSlideAmount );
+
+      double newT0 = t0 + hSlideAmount;
+      if (hitTestResult != TrackShifter::HitTestResult::Track) {
+         // If necessary, correct for rounding errors. For example,
+         // for a wavetrack, ensure that t0 is still in the clip
+         // which it was within before the move.
+         // (pShifter is still undestroyed in the ClipMoveState.)
+         newT0 = pShifter->AdjustT0(newT0);
+      }
 
-      // update t0 and t1. There is the possibility that the updated
-      // t0 may no longer be within the clip due to rounding errors,
-      // so t0 is adjusted so that it is.
-      double newT0 = t0 + state.hSlideAmount;
-      if (newT0 < state.capturedClip->GetStartTime())
-         newT0 = state.capturedClip->GetStartTime();
-      if (newT0 > state.capturedClip->GetEndTime())
-         newT0 = state.capturedClip->GetEndTime();
       double diff = selectedRegion.duration();
       selectedRegion.setTimes(newT0, newT0 + diff);
 
-      return state.hSlideAmount;
-   } );
+      return hSlideAmount;
+   };
    return 0.0;
 }
 
@@ -706,7 +718,7 @@ void DoClipLeftOrRight
    auto &tracks = TrackList::Get( project );
    auto isSyncLocked = settings.IsSyncLocked();
 
-   auto amount = DoClipMove( viewInfo, trackFocus.Get(),
+   auto amount = DoClipMove( project, trackFocus.Get(),
         tracks, isSyncLocked, right );
 
    window.ScrollIntoView(selectedRegion.t0());
@@ -831,7 +843,7 @@ BaseItemSharedPtr ClipSelectMenu()
 
    static BaseItemSharedPtr menu {
    ( FinderScope{ findCommandHandler },
-   Menu( wxT("Clip"), XXO("Clip B&oundaries"),
+   Menu( wxT("Clip"), XXO("Audi&o Clips"),
       Command( wxT("SelPrevClipBoundaryToCursor"),
          XXO("Pre&vious Clip Boundary to Cursor"),
          FN(OnSelectPrevClipBoundaryToCursor),
@@ -880,23 +892,23 @@ AttachedItem sAttachment2{
    Shared( ClipCursorItems() )
 };
 
-BaseItemSharedPtr ExtraClipCursorItems()
+BaseItemSharedPtr ExtraTimeShiftItems()
 {
    using Options = CommandManager::Options;
    static BaseItemSharedPtr items{
    ( FinderScope{ findCommandHandler },
-   Items( wxT("Clip"),
-      Command( wxT("ClipLeft"), XXO("Clip L&eft"), FN(OnClipLeft),
+   Items( wxT("TimeShift"),
+      Command( wxT("ClipLeft"), XXO("Time Shift &Left"), FN(OnClipLeft),
          TracksExistFlag() | TrackPanelHasFocus(), Options{}.WantKeyUp() ),
-      Command( wxT("ClipRight"), XXO("Clip Rig&ht"), FN(OnClipRight),
+      Command( wxT("ClipRight"), XXO("Time Shift &Right"), FN(OnClipRight),
          TracksExistFlag() | TrackPanelHasFocus(), Options{}.WantKeyUp() )
    ) ) };
    return items;
 }
 
 AttachedItem sAttachment3{
-  { wxT("Optional/Extra/Part2/Cursor"), { OrderingHint::End, {} } },
-  Shared( ExtraClipCursorItems() )
+  { wxT("Optional/Extra/Part1/Edit"), { OrderingHint::End, {} } },
+  Shared( ExtraTimeShiftItems() )
 };
 
 }
