@@ -1,10 +1,11 @@
 #include "../CommonCommandFlags.h"
 #include "../Menus.h"
-#include "../Prefs.h"
-#include "../Project.h"
+#include "Prefs.h"
+#include "Project.h"
+#include "ProjectWindows.h"
 #include "../commands/CommandContext.h"
 #include "../commands/CommandManager.h"
-#include "../toolbars/MixerToolBar.h"
+#include "../toolbars/MeterToolBar.h"
 #include "../toolbars/DeviceToolBar.h"
 
 #include <wx/frame.h>
@@ -26,61 +27,43 @@ struct Handler : CommandHandlerObject {
 void OnOutputGain(const CommandContext &context)
 {
    auto &project = context.project;
-   auto tb = &MixerToolBar::Get( project );
-
-   if (tb) {
-      tb->ShowOutputGainDialog();
-   }
+   auto &tb = MeterToolBar::Get( project, true );
+   tb.ShowOutputGainDialog();
 }
 
 void OnOutputGainInc(const CommandContext &context)
 {
    auto &project = context.project;
-   auto tb = &MixerToolBar::Get( project );
-
-   if (tb) {
-      tb->AdjustOutputGain(1);
-   }
+   auto &tb = MeterToolBar::Get( project, true );
+   tb.AdjustOutputGain(1);
 }
 
 void OnOutputGainDec(const CommandContext &context)
 {
    auto &project = context.project;
-   auto tb = &MixerToolBar::Get( project );
-
-   if (tb) {
-      tb->AdjustOutputGain(-1);
-   }
+   auto &tb = MeterToolBar::Get( project, true );
+   tb.AdjustOutputGain(-1);
 }
 
 void OnInputGain(const CommandContext &context)
 {
    auto &project = context.project;
-   auto tb = &MixerToolBar::Get( project );
-
-   if (tb) {
-      tb->ShowInputGainDialog();
-   }
+   auto &tb = MeterToolBar::Get( project, false );
+   tb.ShowInputGainDialog();
 }
 
 void OnInputGainInc(const CommandContext &context)
 {
    auto &project = context.project;
-   auto tb = &MixerToolBar::Get( project );
-
-   if (tb) {
-      tb->AdjustInputGain(1);
-   }
+   auto &tb = MeterToolBar::Get( project, false );
+   tb.AdjustInputGain(1);
 }
 
 void OnInputGainDec(const CommandContext &context)
 {
    auto &project = context.project;
-   auto tb = &MixerToolBar::Get( project );
-
-   if (tb) {
-      tb->AdjustInputGain(-1);
-   }
+   auto &tb = MeterToolBar::Get( project, false );
+   tb.AdjustInputGain(-1);
 }
 
 void OnInputDevice(const CommandContext &context)
