@@ -1,110 +1,172 @@
-#include "../Audacity.h" // for USE_* macros
-#include "../Experimental.h"
 
-#include "../BatchCommands.h"
+
 #include "../CommonCommandFlags.h"
-#include "../FileNames.h"
+#include "FileNames.h"
 #include "../LabelTrack.h"
-#include "../MissingAliasFileDialog.h"
 #include "../NoteTrack.h"
-#include "../Prefs.h"
+#include "Prefs.h"
 #include "../Printing.h"
-#include "../Project.h"
+#include "Project.h"
+#include "../ProjectFileIO.h"
 #include "../ProjectFileManager.h"
-#include "../ProjectHistory.h"
+#include "ProjectHistory.h"
 #include "../ProjectManager.h"
+#include "../ProjectWindows.h"
 #include "../ProjectWindow.h"
+#include "../SelectFile.h"
 #include "../SelectUtilities.h"
 #include "../TrackPanel.h"
-#include "../ViewInfo.h"
+#include "UndoManager.h"
+#include "ViewInfo.h"
 #include "../WaveTrack.h"
 #include "../commands/CommandContext.h"
 #include "../commands/CommandManager.h"
+#include "../export/ExportMP3.h"
 #include "../export/ExportMultiple.h"
 #include "../import/Import.h"
 #include "../import/ImportMIDI.h"
 #include "../import/ImportRaw.h"
 #include "../widgets/AudacityMessageBox.h"
 #include "../widgets/FileHistory.h"
+#include "../widgets/wxPanelWrapper.h"
 
 #ifdef USE_MIDI
 #include "../import/ImportMIDI.h"
 #endif // USE_MIDI
 
-#include "../ondemand/ODManager.h"
-
+#include <wx/app.h>
 #include <wx/menu.h>
 
 // private helper classes and functions
 namespace {
-void DoExport( AudacityProject &project, const FileExtension & Format )
+
+void DoExport(AudacityProject &project, const FileExtension &format)
 {
    auto &tracks = TrackList::Get( project );
-
+   auto &projectFileIO = ProjectFileIO::Get( project );
+   
    Exporter e{ project };
 
-   MissingAliasFilesDialog::SetShouldShow(true);
    double t0 = 0.0;
    double t1 = tracks.GetEndTime();
+   wxString projectName = project.GetProjectName();
 
    // Prompt for file name and/or extension?
-   bool bPromptingRequired =
-      (project.mBatchMode == 0) || project.GetFileName().empty() ||
-      Format.empty();
-   wxString filename;
+   bool bPromptingRequired = !project.mBatchMode ||
+                             projectName.empty() ||
+                             format.empty();
 
-   if (!bPromptingRequired) {
-
-      // We're in batch mode, and we have an mFileName and Format.
-      wxString extension = Format;
-      extension.MakeLower();
-
-      filename =
-         MacroCommands::BuildCleanFileName(project.GetFileName(), extension);
-
-      // Bug 1854, No warning of file overwrite
-      // (when export is called from Macros).
-      int counter = 0;
-      bPromptingRequired = wxFileExists(filename);
-
-      // We'll try alternative names to avoid overwriting.
-      while ( bPromptingRequired && counter < 100 ) {
-         counter++;
-         wxString number;
-         number.Printf("%03i", counter);
-         // So now the name has a number in it too.
-         filename = MacroCommands::BuildCleanFileName(
-            project.GetFileName() + number, extension);
-         bPromptingRequired = wxFileExists(filename);
-      }
-      // If we've run out of alternative names, we will fall back to prompting
-      // - even if in a macro.
-   }
-
-
-   if (bPromptingRequired)
-   {
+   bool success = false;
+   if (bPromptingRequired) {
       // Do export with prompting.
-      e.SetDefaultFormat(Format);
-      e.Process(false, t0, t1);
+      e.SetDefaultFormat(format);
+      success = e.Process(false, t0, t1);
    }
-   else
-   {
-      FileHistory::Global().Append(filename);
+   else {
+      // We either use a configured output path,
+      // or we use the default documents folder - just as for exports.
+      FilePath pathName = FileNames::FindDefaultPath(FileNames::Operation::MacrosOut);
+
+      if (!FileNames::WritableLocationCheck(pathName, XO("Cannot proceed to export.")))
+      {
+          return;
+      }
+/*
+      // If we've gotten to this point, we are in batch mode, have a file format,
+      // and the project has either been saved or a file has been imported. So, we
+      // want to use the project's path if it has been saved, otherwise use the
+      // initial import path.
+      FilePath pathName = !projectFileIO.IsTemporary() ?
+                           wxPathOnly(projectFileIO.GetFileName()) :
+                           project.GetInitialImportPath();
+*/
+      wxFileName fileName(pathName, projectName, format.Lower());
+
+      // Append the "macro-output" directory to the path
+      const wxString macroDir( "macro-output" );
+      if (fileName.GetDirs().back() != macroDir) {
+         fileName.AppendDir(macroDir);
+      }
+
+      wxString justName = fileName.GetName();
+      wxString extension = fileName.GetExt();
+      FilePath fullPath = fileName.GetFullPath();
+
+      if (wxFileName::FileExists(fileName.GetPath())) {
+         AudacityMessageBox(
+            XO("Cannot create directory '%s'. \n"
+               "File already exists that is not a directory"),
+            Verbatim(fullPath));
+         return;
+      }
+      fileName.Mkdir(0777, wxPATH_MKDIR_FULL); // make sure it exists
+
+      int nChannels = (tracks.Any() - &Track::IsLeader ).empty() ? 1 : 2;
+
       // We're in batch mode, the file does not exist already.
       // We really can proceed without prompting.
-      int nChannels = MacroCommands::IsMono( &project ) ? 1 : 2;
-      e.Process(
+      success = e.Process(
          nChannels,  // numChannels,
-         Format,     // type, 
-         filename,   // filename,
+         format,     // type, 
+         fullPath,   // full path,
          false,      // selectedOnly, 
          t0,         // t0
          t1          // t1
       );
    }
 
+   if (success && !project.mBatchMode) {
+      FileHistory::Global().Append(e.GetAutoExportFileName().GetFullPath());
+   }
 }
+
+void DoImport(const CommandContext &context, bool isRaw)
+{
+   auto &project = context.project;
+   auto &trackFactory = WaveTrackFactory::Get( project );
+   auto &window = ProjectWindow::Get( project );
+
+   auto selectedFiles = ProjectFileManager::ShowOpenDialog(FileNames::Operation::Import);
+   if (selectedFiles.size() == 0) {
+      Importer::SetLastOpenType({});
+      return;
+   }
+
+   // PRL:  This affects FFmpegImportPlugin::Open which resets the preference
+   // to false.  Should it also be set to true on other paths that reach
+   // AudacityProject::Import ?
+   NewImportingSession.Write(false);
+
+   selectedFiles.Sort(FileNames::CompareNoCase);
+
+   auto cleanup = finally( [&] {
+
+      Importer::SetLastOpenType({});
+      window.ZoomAfterImport(nullptr);
+      window.HandleResize(); // Adjust scrollers for NEW track sizes.
+   } );
+
+   for (size_t ff = 0; ff < selectedFiles.size(); ff++) {
+      wxString fileName = selectedFiles[ff];
+
+      FileNames::UpdateDefaultPath(FileNames::Operation::Import, ::wxPathOnly(fileName));
+
+      if (isRaw) {
+         TrackHolders newTracks;
+
+         ::ImportRaw(project, &window, fileName, &trackFactory, newTracks);
+
+         if (newTracks.size() > 0) {
+            ProjectFileManager::Get( project )
+               .AddImportedTracks(fileName, std::move(newTracks));
+         }
+      }
+      else {
+         ProjectFileManager::Get( project ).Import(fileName);
+      }
+   }
+}
+
 }
 
 // Menu handler functions
@@ -143,6 +205,11 @@ void OnClose(const CommandContext &context )
    window.Close();
 }
 
+void OnCompact(const CommandContext &context)
+{
+   ProjectFileManager::Get(context.project).Compact();
+}
+
 void OnSave(const CommandContext &context )
 {
    auto &project = context.project;
@@ -161,18 +228,9 @@ void OnSaveCopy(const CommandContext &context )
 {
    auto &project = context.project;
    auto &projectFileManager = ProjectFileManager::Get( project );
-   projectFileManager.SaveAs(true, true);
+   projectFileManager.SaveCopy();
 }
 
-#ifdef USE_LIBVORBIS
-void OnSaveCompressed(const CommandContext &context)
-{
-   auto &project = context.project;
-   auto &projectFileManager = ProjectFileManager::Get( project );
-   projectFileManager.SaveAs(true);
-}
-#endif
-
 void OnExportMp3(const CommandContext &context)
 {
    auto &project = context.project;
@@ -203,7 +261,6 @@ void OnExportSelection(const CommandContext &context)
    auto &selectedRegion = ViewInfo::Get( project ).selectedRegion;
    Exporter e{ project };
 
-   MissingAliasFilesDialog::SetShouldShow(true);
    e.SetFileDialogTitle( XO("Export Selected Audio") );
    e.Process(true, selectedRegion.t0(),
       selectedRegion.t1());
@@ -227,7 +284,7 @@ void OnExportLabels(const CommandContext &context)
    else
       fName = (*trackRange.rbegin())->GetName();
 
-   fName = FileNames::SelectFile(FileNames::Operation::Export,
+   fName = SelectFile(FileNames::Operation::Export,
       XO("Export Labels As:"),
       wxEmptyString,
       fName,
@@ -276,7 +333,6 @@ void OnExportMultiple(const CommandContext &context)
    auto &project = context.project;
    ExportMultipleDialog em(&project);
 
-   MissingAliasFilesDialog::SetShouldShow(true);
    em.ShowModal();
 }
 
@@ -313,7 +369,7 @@ void OnExportMIDI(const CommandContext &context)
 
       wxString fName;
 
-      fName = FileNames::SelectFile(FileNames::Operation::Export,
+      fName = SelectFile(FileNames::Operation::Export,
          XO("Export MIDI As:"),
          wxEmptyString,
          fName,
@@ -370,54 +426,18 @@ void OnExportMIDI(const CommandContext &context)
 
 void OnImport(const CommandContext &context)
 {
-   auto &project = context.project;
-   auto &window = ProjectWindow::Get( project );
-
-   // An import trigger for the alias missing dialog might not be intuitive, but
-   // this serves to track the file if the users zooms in and such.
-   MissingAliasFilesDialog::SetShouldShow(true);
-
-   auto selectedFiles = ProjectFileManager::ShowOpenDialog();
-   if (selectedFiles.size() == 0) {
-      Importer::SetLastOpenType({});
-      return;
-   }
-
-   // PRL:  This affects FFmpegImportPlugin::Open which resets the preference
-   // to false.  Should it also be set to true on other paths that reach
-   // AudacityProject::Import ?
-   gPrefs->Write(wxT("/NewImportingSession"), true);
-
-   //sort selected files by OD status.  Load non OD first so user can edit asap.
-   //first sort selectedFiles.
-   selectedFiles.Sort(CompareNoCaseFileName);
-   ODManager::Pauser pauser;
-
-   auto cleanup = finally( [&] {
-      Importer::SetLastOpenType({});
-      window.HandleResize(); // Adjust scrollers for NEW track sizes.
-   } );
-
-   for (size_t ff = 0; ff < selectedFiles.size(); ff++) {
-      wxString fileName = selectedFiles[ff];
-
-      FileNames::UpdateDefaultPath(FileNames::Operation::Open, fileName);
-
-      ProjectFileManager::Get( project ).Import(fileName);
-   }
-
-   window.ZoomAfterImport(nullptr);
+   DoImport(context, false);
 }
 
 void OnImportLabels(const CommandContext &context)
 {
    auto &project = context.project;
-   auto &trackFactory = TrackFactory::Get( project );
+   auto &trackFactory = WaveTrackFactory::Get( project );
    auto &tracks = TrackList::Get( project );
    auto &window = ProjectWindow::Get( project );
 
    wxString fileName =
-       FileNames::SelectFile(FileNames::Operation::Open,
+       SelectFile(FileNames::Operation::Open,
          XO("Select a text file containing labels"),
          wxEmptyString,     // Path
          wxT(""),       // Name
@@ -436,7 +456,7 @@ void OnImportLabels(const CommandContext &context)
          return;
       }
 
-      auto newTrack = trackFactory.NewLabelTrack();
+      auto newTrack = std::make_shared<LabelTrack>();
       wxString sTrackName;
       wxFileName::SplitPath(fileName, NULL, NULL, &sTrackName, NULL);
       newTrack->SetName(sTrackName);
@@ -461,7 +481,7 @@ void OnImportMIDI(const CommandContext &context)
    auto &project = context.project;
    auto &window = GetProjectFrame( project );
 
-   wxString fileName = FileNames::SelectFile(FileNames::Operation::Open,
+   wxString fileName = SelectFile(FileNames::Operation::Open,
       XO("Select a MIDI file"),
       wxEmptyString,     // Path
       wxT(""),       // Name
@@ -485,33 +505,7 @@ void OnImportMIDI(const CommandContext &context)
 
 void OnImportRaw(const CommandContext &context)
 {
-   auto &project = context.project;
-   auto &trackFactory = TrackFactory::Get( project );
-   auto &window = ProjectWindow::Get( project );
-
-   wxString fileName =
-       FileNames::SelectFile(FileNames::Operation::Open,
-         XO("Select any uncompressed audio file"),
-         wxEmptyString,     // Path
-         wxT(""),       // Name
-         wxT(""),       // Extension
-         { FileNames::AllFiles },
-         wxRESIZE_BORDER,        // Flags
-         &window);    // Parent
-
-   if (fileName.empty())
-      return;
-
-   TrackHolders newTracks;
-
-   ::ImportRaw(&window, fileName, &trackFactory, newTracks);
-
-   if (newTracks.size() <= 0)
-      return;
-
-   ProjectFileManager::Get( project )
-      .AddImportedTracks(fileName, std::move(newTracks));
-   window.HandleResize(); // Adjust scrollers for NEW track sizes.
+   DoImport(context, true);
 }
 
 void OnPageSetup(const CommandContext &context)
@@ -534,7 +528,12 @@ void OnExit(const CommandContext &WXUNUSED(context) )
 {
    // Simulate the application Exit menu item
    wxCommandEvent evt{ wxEVT_MENU, wxID_EXIT };
-   wxTheApp->AddPendingEvent( evt );
+   wxTheApp->ProcessEvent( evt );
+}
+
+void OnExportFLAC(const CommandContext &context)
+{
+   DoExport(context.project, "FLAC");
 }
 
 }; // struct Handler
@@ -555,6 +554,15 @@ static CommandHandlerObject &findCommandHandler(AudacityProject &) {
 namespace {
 using namespace MenuTable;
 
+#ifdef USE_MIDI
+const ReservedCommandFlag&
+   NoteTracksExistFlag() { static ReservedCommandFlag flag{
+      [](const AudacityProject &project){
+         return !TrackList::Get( project ).Any<const NoteTrack>().empty();
+      }
+   }; return flag; }  //gsw
+#endif
+
 BaseItemSharedPtr FileMenu()
 {
    using Options = CommandManager::Options;
@@ -622,19 +630,20 @@ BaseItemSharedPtr FileMenu()
       Section( "Save",
          Menu( wxT("Save"), XXO("&Save Project"),
             Command( wxT("Save"), XXO("&Save Project"), FN(OnSave),
-               AudioIONotBusyFlag() | UnsavedChangesFlag(), wxT("Ctrl+S") ),
+               AudioIONotBusyFlag(), wxT("Ctrl+S") ),
             Command( wxT("SaveAs"), XXO("Save Project &As..."), FN(OnSaveAs),
                AudioIONotBusyFlag() ),
-            // TODO: The next two items should be disabled if project is empty
-            Command( wxT("SaveCopy"), XXO("Save Lossless Copy of Project..."),
-               FN(OnSaveCopy), AudioIONotBusyFlag() )
-   #ifdef USE_LIBVORBIS
-            ,
-            Command( wxT("SaveCompressed"),
-               XXO("&Save Compressed Copy of Project..."),
-               FN(OnSaveCompressed), AudioIONotBusyFlag() )
-   #endif
-         )
+            Command( wxT("SaveCopy"), XXO("&Backup Project..."), FN(OnSaveCopy),
+               AudioIONotBusyFlag() )
+         )//,
+
+         // Bug 2600: Compact has interactions with undo/history that are bound
+         // to confuse some users.  We don't see a way to recover useful amounts 
+         // of space and not confuse users using undo.
+         // As additional space used by aup3 is 50% or so, perfectly valid
+         // approach to this P1 bug is to not provide the 'Compact' menu item.
+         //Command( wxT("Compact"), XXO("Co&mpact Project"), FN(OnCompact),
+         //   AudioIONotBusyFlag(), wxT("Shift+A") )
       ),
 
       Section( "Import-Export",
@@ -655,8 +664,7 @@ BaseItemSharedPtr FileMenu()
             // Enable Export Selection commands only when there's a selection.
             Command( wxT("ExportSel"), XXO("Expo&rt Selected Audio..."),
                FN(OnExportSelection),
-               AudioIONotBusyFlag() | TimeSelectedFlag() | WaveTracksSelectedFlag(),
-               Options{}.UseStrictFlags() ),
+               AudioIONotBusyFlag() | TimeSelectedFlag() | WaveTracksSelectedFlag() ),
 
             Command( wxT("ExportLabels"), XXO("Export &Labels..."),
                FN(OnExportLabels),
@@ -710,6 +718,36 @@ AttachedItem sAttachment1{
    wxT(""),
    Shared( FileMenu() )
 };
+
+BaseItemSharedPtr HiddenFileMenu()
+{
+   static BaseItemSharedPtr menu
+   {
+      (
+         FinderScope{ findCommandHandler },
+         ConditionalItems( wxT("HiddenFileItems"),
+            []()
+            {
+               // Ensures that these items never appear in a menu, but
+               // are still available to scripting
+               return false;
+            },
+            Menu( wxT("HiddenFileMenu"), XXO("Hidden File Menu"),
+               Command( wxT("ExportFLAC"), XXO("Export as FLAC"),
+                  FN(OnExportFLAC),
+                  AudioIONotBusyFlag() )
+            )
+         )
+      )
+   };
+   return menu;
+}
+
+AttachedItem sAttachment2{
+   wxT(""),
+   Shared( HiddenFileMenu() )
+};
+
 }
 
 #undef FN
