@@ -1,26 +1,29 @@
-#include "../Audacity.h"
-#include "../Experimental.h"
+
 
 #include "../AdornedRulerPanel.h"
 #include "../AudioIO.h"
 #include "../CommonCommandFlags.h"
 #include "../SpectrumAnalyst.h"
-#include "../Prefs.h"
-#include "../Project.h"
+#include "Prefs.h"
+#include "Project.h"
 #include "../ProjectAudioIO.h"
 #include "../ProjectAudioManager.h"
-#include "../ProjectHistory.h"
+#include "ProjectHistory.h"
+#include "ProjectRate.h"
 #include "../ProjectSelectionManager.h"
 #include "../ProjectSettings.h"
 #include "../ProjectWindow.h"
+#include "../ProjectWindows.h"
 #include "../SelectUtilities.h"
-#include "../TimeDialog.h"
+#include "../SyncLock.h"
 #include "../TrackPanel.h"
 #include "../WaveTrack.h"
+#include "../LabelTrack.h"
 #include "../commands/CommandContext.h"
 #include "../commands/CommandManager.h"
 #include "../toolbars/ControlToolBar.h"
 #include "../tracks/ui/SelectHandle.h"
+#include "../tracks/labeltrack/ui/LabelTrackView.h"
 #include "../tracks/playabletrack/wavetrack/ui/WaveTrackView.h"
 #include "../tracks/playabletrack/wavetrack/ui/WaveTrackViewConstants.h"
 
@@ -56,8 +59,7 @@ void DoNextPeakFrequency(AudacityProject &project, bool up)
 double NearestZeroCrossing
 (AudacityProject &project, double t0)
 {
-   const auto &settings = ProjectSettings::Get( project );
-   auto rate = settings.GetRate();
+   auto rate = ProjectRate::Get(project).GetRate();
    auto &tracks = TrackList::Get( project );
 
    // Window is 1/100th of a second.
@@ -71,7 +73,7 @@ double NearestZeroCrossing
       auto s = one->TimeToLongSamples(t0);
       // fillTwo to ensure that missing values are treated as 2, and hence do
       // not get used as zero crossings.
-      one->Get((samplePtr)oneDist.get(), floatSample,
+      one->GetFloats(oneDist.get(),
                s - (int)oneWindowSize/2, oneWindowSize, fillTwo);
 
 
@@ -191,8 +193,8 @@ void SeekWhenAudioActive(double seekStep, wxLongLong &lastSelectionAdjustment)
 double GridMove
 (AudacityProject &project, double t, int minPix)
 {
-   const auto &settings = ProjectSettings::Get( project );
-   auto rate = settings.GetRate();
+   auto &settings = ProjectSettings::Get(project);
+   auto rate = ProjectRate::Get(project).GetRate();
    auto &viewInfo = ViewInfo::Get( project );
    auto format = settings.GetSelectionFormat();
 
@@ -458,6 +460,27 @@ struct Handler
 
 void OnSelectAll(const CommandContext &context)
 {
+   auto& trackPanel = TrackPanel::Get(context.project);
+   auto& tracks = TrackList::Get(context.project);
+   
+   for (auto lt : tracks.Selected< LabelTrack >()) {
+      auto& view = LabelTrackView::Get(*lt);
+      if (view.SelectAllText(context.project)) {
+         trackPanel.Refresh(false);
+         return;
+      }
+   }
+
+   //Presumably, there might be not more than one track
+   //that expects text input
+   for (auto wt : tracks.Any<WaveTrack>()) {
+      auto& view = WaveTrackView::Get(*wt);
+      if (view.SelectAllText(context.project)) {
+         trackPanel.Refresh(false);
+         return;
+      }
+   }
+
    SelectUtilities::DoSelectAll( context.project );
 }
 
@@ -483,8 +506,8 @@ void OnSelectSyncLockSel(const CommandContext &context)
    auto &tracks = TrackList::Get( project );
 
    bool selected = false;
-   for (auto t : tracks.Any()
-         + &Track::IsSyncLockSelected - &Track::IsSelected) {
+   for (auto t : tracks.Any() + &Track::SupportsBasicEditing
+         + &SyncLock::IsSyncLockSelected - &Track::IsSelected) {
       t->SetSelected(true);
       selected = true;
    }
@@ -493,85 +516,16 @@ void OnSelectSyncLockSel(const CommandContext &context)
       ProjectHistory::Get( project ).ModifyState(false);
 }
 
-//this pops up a dialog which allows the left selection to be set.
-//If playing/recording is happening, it sets the left selection at
-//the current play position.
 void OnSetLeftSelection(const CommandContext &context)
 {
-   auto &project = context.project;
-   auto token = ProjectAudioIO::Get( project ).GetAudioIOToken();
-   auto &selectedRegion = ViewInfo::Get( project ).selectedRegion;
-   const auto &settings = ProjectSettings::Get( project );
-   auto &window = GetProjectFrame( project );
-
-   bool bSelChanged = false;
-   auto gAudioIO = AudioIOBase::Get();
-   if ((token > 0) && gAudioIO->IsStreamActive(token))
-   {
-      double indicator = gAudioIO->GetStreamTime();
-      selectedRegion.setT0(indicator, false);
-      bSelChanged = true;
-   }
-   else
-   {
-      auto fmt = settings.GetSelectionFormat();
-      auto rate = settings.GetRate();
-
-      TimeDialog dlg(&window, XO("Set Left Selection Boundary"),
-         fmt, rate, selectedRegion.t0(), XO("Position"));
-
-      if (wxID_OK == dlg.ShowModal())
-      {
-         //Get the value from the dialog
-         selectedRegion.setT0(
-            std::max(0.0, dlg.GetTimeValue()), false);
-         bSelChanged = true;
-      }
-   }
-
-   if (bSelChanged)
-   {
-      ProjectHistory::Get( project ).ModifyState(false);
-   }
+   SelectUtilities::OnSetRegion(context.project,
+      true, true, XO("Set Left Selection Boundary"));
 }
 
 void OnSetRightSelection(const CommandContext &context)
 {
-   auto &project = context.project;
-   auto token = ProjectAudioIO::Get( project ).GetAudioIOToken();
-   auto &selectedRegion = ViewInfo::Get( project ).selectedRegion;
-   const auto &settings = ProjectSettings::Get( project );
-   auto &window = GetProjectFrame( project );
-
-   bool bSelChanged = false;
-   auto gAudioIO = AudioIOBase::Get();
-   if ((token > 0) && gAudioIO->IsStreamActive(token))
-   {
-      double indicator = gAudioIO->GetStreamTime();
-      selectedRegion.setT1(indicator, false);
-      bSelChanged = true;
-   }
-   else
-   {
-      auto fmt = settings.GetSelectionFormat();
-      auto rate = settings.GetRate();
-
-      TimeDialog dlg(&window, XO("Set Right Selection Boundary"),
-         fmt, rate, selectedRegion.t1(), XO("Position"));
-
-      if (wxID_OK == dlg.ShowModal())
-      {
-         //Get the value from the dialog
-         selectedRegion.setT1(
-            std::max(0.0, dlg.GetTimeValue()), false);
-         bSelChanged = true;
-      }
-   }
-
-   if (bSelChanged)
-   {
-      ProjectHistory::Get( project ).ModifyState(false);
-   }
+   SelectUtilities::OnSetRegion(context.project,
+      false, true, XO("Set Right Selection Boundary"));
 }
 
 void OnSelectStartCursor(const CommandContext &context)
@@ -745,7 +699,6 @@ void OnZeroCrossing(const CommandContext &context)
 {
    auto &project = context.project;
    auto &selectedRegion = ViewInfo::Get( project ).selectedRegion;
-   const auto &settings = ProjectSettings::Get( project );
 
    const double t0 = NearestZeroCrossing(project, selectedRegion.t0());
    if (selectedRegion.isPoint())
@@ -753,7 +706,7 @@ void OnZeroCrossing(const CommandContext &context)
    else {
       const double t1 = NearestZeroCrossing(project, selectedRegion.t1());
       // Empty selection is generally not much use, so do not make it if empty.
-      if( fabs( t1 - t0 ) * settings.GetRate() > 1.5 )
+      if( fabs( t1 - t0 ) * ProjectRate::Get(project).GetRate() > 1.5 )
          selectedRegion.setTimes(t0, t1);
    }
 
@@ -866,7 +819,7 @@ void OnCursorTrackStart(const CommandContext &context)
 
    double kWayOverToRight = std::numeric_limits<double>::max();
 
-   auto trackRange = tracks.Selected();
+   auto trackRange = tracks.Selected() + &Track::SupportsBasicEditing;
    if (trackRange.empty())
       // This should have been prevented by command manager
       return;
@@ -892,7 +845,7 @@ void OnCursorTrackEnd(const CommandContext &context)
 
    double kWayOverToLeft = std::numeric_limits<double>::lowest();
 
-   auto trackRange = tracks.Selected();
+   auto trackRange = tracks.Selected() + &Track::SupportsBasicEditing;
    if (trackRange.empty())
       // This should have been prevented by command manager
       return;
@@ -1059,7 +1012,7 @@ BaseItemSharedPtr SelectMenu()
             ,
             Command( wxT("SelSyncLockTracks"), XXO("In All &Sync-Locked Tracks"),
                FN(OnSelectSyncLockSel),
-               TracksSelectedFlag() | IsSyncLockedFlag(),
+               EditableTracksSelectedFlag() | IsSyncLockedFlag(),
                Options{ wxT("Ctrl+Shift+Y"), XO("Select Sync-Locked") } )
    #endif
          ),
@@ -1131,7 +1084,7 @@ BaseItemSharedPtr SelectMenu()
 
       Section( "",
          Command( wxT("ZeroCross"), XXO("At &Zero Crossings"),
-            FN(OnZeroCrossing), TracksSelectedFlag(),
+            FN(OnZeroCrossing), EditableTracksSelectedFlag(),
             Options{ wxT("Z"), XO("Select Zero Crossing") } )
       )
    ) ) };
@@ -1216,11 +1169,11 @@ BaseItemSharedPtr CursorMenu()
 
       Command( wxT("CursTrackStart"), XXO("Track &Start"),
          FN(OnCursorTrackStart),
-         TracksSelectedFlag(),
+         EditableTracksSelectedFlag(),
          Options{ wxT("J"), XO("Cursor to Track Start") } ),
       Command( wxT("CursTrackEnd"), XXO("Track &End"),
          FN(OnCursorTrackEnd),
-         TracksSelectedFlag(),
+         EditableTracksSelectedFlag(),
          Options{ wxT("K"), XO("Cursor to Track End") } ),
 
       Command( wxT("CursProjectStart"), XXO("&Project Start"),
