@@ -1,35 +1,32 @@
-#include "../Audacity.h"
-#include "../Experimental.h"
-
 #include "../CommonCommandFlags.h"
 #include "../LabelTrack.h"
 #include "../Menus.h"
-#include "../MissingAliasFileDialog.h"
-#include "../Mix.h"
+#include "../MixAndRender.h"
 
-#include "../Prefs.h"
-#include "../Project.h"
+#include "Prefs.h"
+#include "Project.h"
 #include "../ProjectAudioIO.h"
-#include "../ProjectHistory.h"
+#include "ProjectHistory.h"
+#include "ProjectRate.h"
 #include "../ProjectSettings.h"
-#include "../PluginManager.h"
-#include "../ProjectStatus.h"
+#include "PluginManager.h"
+#include "ProjectStatus.h"
 #include "../ProjectWindow.h"
 #include "../SelectUtilities.h"
 #include "../ShuttleGui.h"
-#include "../TimeTrack.h"
+#include "../SyncLock.h"
 #include "../TrackPanelAx.h"
 #include "../TrackPanel.h"
 #include "../TrackUtilities.h"
-#include "../UndoManager.h"
+#include "UndoManager.h"
 #include "../WaveClip.h"
-#include "../ViewInfo.h"
+#include "ViewInfo.h"
 #include "../WaveTrack.h"
 #include "../commands/CommandContext.h"
 #include "../commands/CommandManager.h"
 #include "../effects/EffectManager.h"
 #include "../effects/EffectUI.h"
-#include "../prefs/QualityPrefs.h"
+#include "QualitySettings.h"
 #include "../tracks/playabletrack/wavetrack/ui/WaveTrackControls.h"
 #include "../widgets/ASlider.h"
 #include "../widgets/AudacityMessageBox.h"
@@ -50,25 +47,23 @@ namespace {
 void DoMixAndRender
 (AudacityProject &project, bool toNewTrack)
 {
-   const auto &settings = ProjectSettings::Get( project );
    auto &tracks = TrackList::Get( project );
-   auto &trackFactory = TrackFactory::Get( project );
-   auto rate = settings.GetRate();
-   auto defaultFormat = QualityPrefs::SampleFormatChoice();
+   auto &trackFactory = WaveTrackFactory::Get( project );
+   auto rate = ProjectRate::Get(project).GetRate();
+   auto defaultFormat = QualitySettings::SampleFormatChoice();
    auto &trackPanel = TrackPanel::Get( project );
    auto &window = ProjectWindow::Get( project );
 
-   MissingAliasFilesDialog::SetShouldShow(true);
-
+   auto trackRange = tracks.Selected< WaveTrack >();
    WaveTrack::Holder uNewLeft, uNewRight;
-   ::MixAndRender(
-      &tracks, &trackFactory, rate, defaultFormat, 0.0, 0.0, uNewLeft, uNewRight);
+   ::MixAndRender(trackRange.Filter<const WaveTrack>(),
+      Mixer::WarpOptions{ tracks },
+      tracks.MakeUniqueTrackName(_("Mix")),
+      &trackFactory, rate, defaultFormat, 0.0, 0.0, uNewLeft, uNewRight);
 
    if (uNewLeft) {
       // Remove originals, get stats on what tracks were mixed
 
-      auto trackRange = tracks.Selected< WaveTrack >();
-
       // But before removing, determine the first track after the removal
       auto last = *trackRange.rbegin();
       auto insertionPoint = * ++ tracks.Find( last );
@@ -91,10 +86,10 @@ void DoMixAndRender
       auto pNewLeft = tracks.Add( uNewLeft );
       decltype(pNewLeft) pNewRight{};
       if (uNewRight)
-         pNewRight = tracks.Add( uNewRight );
-
-      // Do this only after adding tracks to the list
-      tracks.GroupChannels(*pNewLeft, pNewRight ? 2 : 1);
+      {
+         pNewRight = tracks.Add(uNewRight);
+         tracks.MakeMultiChannelTrack(*pNewLeft, 2, true);
+      }
 
       // If we're just rendering (not mixing), keep the track name the same
       if (selectedCount==1) {
@@ -171,7 +166,7 @@ void DoPanTracks(AudacityProject &project, float PanValue)
    for (auto left : count == 0 ? range : selectedRange )
       left->SetPan( PanValue );
 
-   auto flags = UndoPush::AUTOSAVE;
+   auto flags = UndoPush::NONE;
    ProjectHistory::Get( project )
       /*i18n-hint: One or more audio tracks have been panned*/
       .PushState(XO("Panned audio track(s)"), XO("Pan Track"), flags);
@@ -327,7 +322,8 @@ void DoAlign
 
    if (delta != 0.0) {
       // For a fixed-distance shift move sync-lock selected tracks also.
-      for (auto t : tracks.Any() + &Track::IsSelectedOrSyncLockSelected )
+      for (auto t : tracks.Any()
+           + &SyncLock::IsSelectedOrSyncLockSelected )
          t->SetOffset(t->GetOffset() + delta);
    }
 
@@ -339,6 +335,15 @@ void DoAlign
 
 #ifdef EXPERIMENTAL_SCOREALIGN
 
+#ifdef USE_MIDI
+static const ReservedCommandFlag&
+   NoteTracksSelectedFlag() { static ReservedCommandFlag flag{
+      [](const AudacityProject &project){
+         return !TrackList::Get( project ).Selected<const NoteTrack>().empty();
+      }
+   }; return flag; }  //gsw
+#endif
+
 // rough relative amount of time to compute one
 //    frame of audio or midi, or one cell of matrix, or one iteration
 //    of smoothing, measured on a 1.9GHz Core 2 Duo in 32-bit mode
@@ -362,7 +367,7 @@ class ASAProgress final : public SAProgress {
    long mTotalCells; // how many matrix cells?
    long mCellCount; // how many cells so far?
    long mPrevCellCount; // cell_count last reported with Update()
-   Optional<ProgressDialog> mProgress;
+   std::optional<ProgressDialog> mProgress;
    #ifdef COLLECT_TIMING_DATA
       FILE *mTimeFile;
       wxDateTime mStartTime;
@@ -502,9 +507,9 @@ void DoSortTracks( AudacityProject &project, int flags )
             int ndx;
             for (ndx = 0; ndx < w->GetNumClips(); ndx++) {
                const auto c = w->GetClipByIndex(ndx);
-               if (c->GetNumSamples() == 0)
+               if (c->GetPlaySamplesCount() == 0)
                   continue;
-               stime = std::min(stime, c->GetStartTime());
+               stime = std::min(stime, c->GetPlayStartTime());
             }
             return stime;
          },
@@ -605,105 +610,6 @@ namespace TrackActions {
 
 struct Handler : CommandHandlerObject {
 
-void OnNewWaveTrack(const CommandContext &context)
-{
-   auto &project = context.project;
-   const auto &settings = ProjectSettings::Get( project );
-   auto &tracks = TrackList::Get( project );
-   auto &trackFactory = TrackFactory::Get( project );
-   auto &window = ProjectWindow::Get( project );
-
-   auto defaultFormat = QualityPrefs::SampleFormatChoice();
-
-   auto rate = settings.GetRate();
-
-   auto t = tracks.Add( trackFactory.NewWaveTrack( defaultFormat, rate ) );
-   SelectUtilities::SelectNone( project );
-
-   t->SetSelected(true);
-
-   ProjectHistory::Get( project )
-      .PushState(XO("Created new audio track"), XO("New Track"));
-
-   TrackFocus::Get(project).Set(t);
-   t->EnsureVisible();
-}
-
-void OnNewStereoTrack(const CommandContext &context)
-{
-   auto &project = context.project;
-   const auto &settings = ProjectSettings::Get( project );
-   auto &tracks = TrackList::Get( project );
-   auto &trackFactory = TrackFactory::Get( project );
-   auto &window = ProjectWindow::Get( project );
-
-   auto defaultFormat = QualityPrefs::SampleFormatChoice();
-   auto rate = settings.GetRate();
-
-   SelectUtilities::SelectNone( project );
-
-   auto left = tracks.Add( trackFactory.NewWaveTrack( defaultFormat, rate ) );
-   left->SetSelected(true);
-
-   auto right = tracks.Add( trackFactory.NewWaveTrack( defaultFormat, rate ) );
-   right->SetSelected(true);
-
-   tracks.GroupChannels(*left, 2);
-
-   ProjectHistory::Get( project )
-      .PushState(XO("Created new stereo audio track"), XO("New Track"));
-
-   TrackFocus::Get(project).Set(left);
-   left->EnsureVisible();
-}
-
-void OnNewLabelTrack(const CommandContext &context)
-{
-   auto &project = context.project;
-   auto &tracks = TrackList::Get( project );
-   auto &trackFactory = TrackFactory::Get( project );
-   auto &window = ProjectWindow::Get( project );
-
-   auto t = tracks.Add( trackFactory.NewLabelTrack() );
-
-   SelectUtilities::SelectNone( project );
-
-   t->SetSelected(true);
-
-   ProjectHistory::Get( project )
-      .PushState(XO("Created new label track"), XO("New Track"));
-
-   TrackFocus::Get(project).Set(t);
-   t->EnsureVisible();
-}
-
-void OnNewTimeTrack(const CommandContext &context)
-{
-   auto &project = context.project;
-   auto &tracks = TrackList::Get( project );
-   auto &trackFactory = TrackFactory::Get( project );
-   auto &window = ProjectWindow::Get( project );
-
-   if ( *tracks.Any<TimeTrack>().begin() ) {
-      AudacityMessageBox(
-         XO(
-"This version of Audacity only allows one time track for each project window.") );
-      return;
-   }
-
-   auto t = tracks.AddToHead( trackFactory.NewTimeTrack() );
-
-   SelectUtilities::SelectNone( project );
-
-   t->SetSelected(true);
-
-   ProjectHistory::Get( project )
-      .PushState(XO("Created new time track"), XO("New Track"));
-
-   TrackFocus::Get(project).Set(t);
-   t->EnsureVisible();
-}
-
 void OnStereoToMono(const CommandContext &context)
 {
    EffectUI::DoEffect(
@@ -727,8 +633,7 @@ void OnMixAndRenderToNewTrack(const CommandContext &context)
 void OnResample(const CommandContext &context)
 {
    auto &project = context.project;
-   const auto &settings = ProjectSettings::Get( project );
-   auto projectRate = settings.GetRate();
+   auto projectRate = ProjectRate::Get(project).GetRate();
    auto &tracks = TrackList::Get( project );
    auto &undoManager = UndoManager::Get( project );
    auto &window = ProjectWindow::Get( project );
@@ -802,19 +707,20 @@ void OnResample(const CommandContext &context)
    }
 
    int ndx = 0;
-   auto flags = UndoPush::AUTOSAVE;
+   auto flags = UndoPush::NONE;
    for (auto wt : tracks.Selected< WaveTrack >())
    {
       auto msg = XO("Resampling track %d").Format( ++ndx );
 
-      ProgressDialog progress(XO("Resample"), msg);
+      using namespace BasicUI;
+      auto progress = MakeProgress(XO("Resample"), msg);
 
       // The resampling of a track may be stopped by the user.  This might
       // leave a track with multiple clips in a partially resampled state.
       // But the thrown exception will cause rollback in the application
       // level handler.
 
-       wt->Resample(newRate, &progress);
+       wt->Resample(newRate, progress.get());
 
       // Each time a track is successfully, completely resampled,
       // commit that to the undo stack.  The second and later times,
@@ -965,7 +871,7 @@ void OnScoreAlign(const CommandContext &context)
    }
 
    // Creating the dialog also stores dialog into gScoreAlignDialog so
-   // that it can be delted by CloseScoreAlignDialog() either here or
+   // that it can be deleted by CloseScoreAlignDialog() either here or
    // if the program is quit by the user while the dialog is up.
    ScoreAlignParams params;
 
@@ -1000,9 +906,7 @@ void OnScoreAlign(const CommandContext &context)
       Mixer mix(
          waveTracks,              // const WaveTrackConstArray &inputTracks
          false, // mayThrow -- is this right?
-         Mixer::WarpOptions{
-            *tracks->Any<const TimeTrack >().begin()
-         }, // const WarpOptions &warpOptions
+         Mixer::WarpOptions{ *tracks },
          0.0,                     // double startTime
          endTime,                 // double stopTime
          2,                       // int numOutChannels
@@ -1184,7 +1088,11 @@ void OnTrackMute(const CommandContext &context)
 {
    auto &project = context.project;
 
-   const auto track = TrackFocus::Get( project ).Get();
+   // Use the temporary selection if it is specified, else the track focus
+   auto track = context.temporarySelection.pTrack;
+   if (!track)
+      track = TrackFocus::Get( project ).Get();
+
    if (track) track->TypeSwitch( [&](PlayableTrack *t) {
       TrackUtilities::DoTrackMute(project, t, false);
    });
@@ -1304,16 +1212,7 @@ BaseItemSharedPtr TracksMenu()
    ( FinderScope{ findCommandHandler },
    Menu( wxT("Tracks"), XXO("&Tracks"),
       Section( "Add",
-         Menu( wxT("Add"), XXO("Add &New"),
-            Command( wxT("NewMonoTrack"), XXO("&Mono Track"), FN(OnNewWaveTrack),
-               AudioIONotBusyFlag(), wxT("Ctrl+Shift+N") ),
-            Command( wxT("NewStereoTrack"), XXO("&Stereo Track"),
-               FN(OnNewStereoTrack), AudioIONotBusyFlag() ),
-            Command( wxT("NewLabelTrack"), XXO("&Label Track"),
-               FN(OnNewLabelTrack), AudioIONotBusyFlag() ),
-            Command( wxT("NewTimeTrack"), XXO("&Time Track"),
-               FN(OnNewTimeTrack), AudioIONotBusyFlag() )
-         )
+         Menu( wxT("Add"), XXO("Add &New") )
       ),
 
       //////////////////////////////////////////////////////////////////////////
@@ -1361,9 +1260,9 @@ BaseItemSharedPtr TracksMenu()
             Command( wxT("UnmuteAllTracks"), XXO("&Unmute All Tracks"),
                FN(OnUnmuteAllTracks), TracksExistFlag(), wxT("Ctrl+Shift+U") ),
             Command( wxT("MuteTracks"), XXO("Mut&e Tracks"),
-               FN(OnMuteSelectedTracks), TracksSelectedFlag(), wxT("Ctrl+Alt+U") ),
+               FN(OnMuteSelectedTracks), EditableTracksSelectedFlag(), wxT("Ctrl+Alt+U") ),
             Command( wxT("UnmuteTracks"), XXO("U&nmute Tracks"),
-               FN(OnUnmuteSelectedTracks), TracksSelectedFlag(), wxT("Ctrl+Alt+Shift+U") )
+               FN(OnUnmuteSelectedTracks), EditableTracksSelectedFlag(), wxT("Ctrl+Alt+Shift+U") )
          ),
 
          Menu( wxT("Pan"), XXO("&Pan"),
@@ -1372,13 +1271,13 @@ BaseItemSharedPtr TracksMenu()
             // in the project could very easily be lost unless we
             // require the tracks to be selected.
             Command( wxT("PanLeft"), XXO("&Left"), FN(OnPanLeft),
-               TracksSelectedFlag(),
+               EditableTracksSelectedFlag(),
                Options{}.LongName( XO("Pan Left") ) ),
             Command( wxT("PanRight"), XXO("&Right"), FN(OnPanRight),
-               TracksSelectedFlag(),
+               EditableTracksSelectedFlag(),
                Options{}.LongName( XO("Pan Right") ) ),
             Command( wxT("PanCenter"), XXO("&Center"), FN(OnPanCenter),
-               TracksSelectedFlag(),
+               EditableTracksSelectedFlag(),
                Options{}.LongName( XO("Pan Center") ) )
          )
       ),
@@ -1392,14 +1291,14 @@ BaseItemSharedPtr TracksMenu()
                      { wxT("EndToEnd"),     XXO("&Align End to End") },
                      { wxT("Together"),     XXO("Align &Together") },
                   },
-                  FN(OnAlignNoSync), AudioIONotBusyFlag() | TracksSelectedFlag())
+                  FN(OnAlignNoSync), AudioIONotBusyFlag() | EditableTracksSelectedFlag())
             ),
 
             Section( "",
                // Alignment commands using selection or zero
                CommandGroup(wxT("Align"),
                   alignLabels(),
-                  FN(OnAlign), AudioIONotBusyFlag() | TracksSelectedFlag())
+                  FN(OnAlign), AudioIONotBusyFlag() | EditableTracksSelectedFlag())
             ),
 
             Section( "",
@@ -1416,7 +1315,7 @@ BaseItemSharedPtr TracksMenu()
          // Do we need this sub-menu at all?
          Menu( wxT("MoveSelectionAndTracks"), XO("Move Sele&ction and Tracks"), {
             CommandGroup(wxT("AlignMove"), alignLabels(),
-               FN(OnAlignMoveSel), AudioIONotBusyFlag() | TracksSelectedFlag()),
+               FN(OnAlignMoveSel), AudioIONotBusyFlag() | EditableTracksSelectedFlag()),
          } ),
    #endif
 
