@@ -22,16 +22,16 @@ class ShuttleGui;
 
 #define GUI_PREFS_PLUGIN_SYMBOL ComponentInterfaceSymbol{ XO("GUI") }
 
-class GUIPrefs final : public PrefsPanel
+class AUDACITY_DLL_API GUIPrefs final : public PrefsPanel
 {
  public:
    GUIPrefs(wxWindow * parent, wxWindowID winid);
    ~GUIPrefs();
-   ComponentInterfaceSymbol GetSymbol() override;
-   TranslatableString GetDescription() override;
+   ComponentInterfaceSymbol GetSymbol() const override;
+   TranslatableString GetDescription() const override;
 
    bool Commit() override;
-   wxString HelpPageName() override;
+   ManualPageID HelpPageName() override;
    void PopulateOrExchange(ShuttleGui & S) override;
 
    static void GetRangeChoices(
@@ -40,21 +40,6 @@ class GUIPrefs final : public PrefsPanel
       int *pDefaultRangeIndex = nullptr
    );
 
-   // If no input language given, defaults first to choice in preferences, then
-   // to system language.
-   // Returns the language actually used which is not lang if lang cannot be found.
-   static wxString InitLang( wxString lang = {} );
-
-   // If no input language given, defaults to system language.
-   // Returns the language actually used which is not lang if lang cannot be found.
-   static wxString SetLang( const wxString & lang );
-
-   // Returns the last language code that was set
-   static wxString GetLang();
-   // Returns the last language code that was set
-   // Unlike GetLang, gives en rather than en_GB or en_US for result.
-   static wxString GetLangShort();
-
  private:
    void Populate();
 
@@ -66,9 +51,12 @@ class GUIPrefs final : public PrefsPanel
    int mDefaultRangeIndex;
 };
 
+AUDACITY_DLL_API
 int ShowClippingPrefsID();
+AUDACITY_DLL_API
+int ShowTrackNameInWaveformPrefsID();
 
-extern ChoiceSetting
+extern AUDACITY_DLL_API ChoiceSetting
      GUIManualLocation
 ;
 
