@@ -18,6 +18,8 @@
 #include "PrefsPanel.h"
 
 class wxStaticText;
+class wxTextCtrl;
+class ReadOnlyText;
 class ShuttleGui;
 
 #define LIBRARY_PREFS_PLUGIN_SYMBOL ComponentInterfaceSymbol{ XO("Library") }
@@ -27,11 +29,11 @@ class LibraryPrefs final : public PrefsPanel
  public:
    LibraryPrefs(wxWindow * parent, wxWindowID winid);
    ~LibraryPrefs();
-   ComponentInterfaceSymbol GetSymbol() override;
-   TranslatableString GetDescription() override;
+   ComponentInterfaceSymbol GetSymbol() const override;
+   TranslatableString GetDescription() const override;
 
    bool Commit() override;
-   wxString HelpPageName() override;
+   ManualPageID HelpPageName() override;
    void PopulateOrExchange(ShuttleGui & S) override;
 
  private:
@@ -44,8 +46,8 @@ class LibraryPrefs final : public PrefsPanel
    void OnFFmpegFindButton(wxCommandEvent & e);
    void OnFFmpegDownButton(wxCommandEvent & e);
 
-   wxStaticText *mMP3Version;
-   wxStaticText *mFFmpegVersion;
+   ReadOnlyText *mMP3Version;
+   ReadOnlyText *mFFmpegVersion;
 
    DECLARE_EVENT_TABLE()
 };
