@@ -29,8 +29,8 @@ MousePrefs, QualityPrefs, SpectrumPrefs and ThemePrefs.
 
 #include <functional>
 #include "../widgets/wxPanelWrapper.h" // to inherit
-#include "../include/audacity/ComponentInterface.h"
-#include "../commands/CommandManager.h"
+#include "ComponentInterface.h"
+#include "Registry.h"
 
 /* A few constants for an attempt at semi-uniformity */
 #define PREFS_FONT_SIZE     8
@@ -46,9 +46,29 @@ MousePrefs, QualityPrefs, SpectrumPrefs and ThemePrefs.
 class AudacityProject;
 class ShuttleGui;
 
-class PrefsPanel /* not final */ : public wxPanelWrapper, ComponentInterface
+class AUDACITY_DLL_API PrefsPanel /* not final */
+   : public wxPanelWrapper, ComponentInterface
 {
  public:
+    // An array of PrefsNode specifies the tree of pages in pre-order traversal.
+    struct PrefsNode {
+       using Factory =
+         std::function< PrefsPanel * (
+            wxWindow *parent, wxWindowID winid, AudacityProject *) >;
+       Factory factory;
+       size_t nChildren{ 0 };
+       bool expanded{ false };
+
+       PrefsNode(const Factory &factory_,
+          unsigned nChildren_ = 0,
+          bool expanded_ = true)
+          : factory(factory_), nChildren(nChildren_), expanded(expanded_)
+       {}
+    };
+
+   using Factories = std::vector<PrefsPanel::PrefsNode>;
+   static Factories &DefaultFactories();
+
    // \brief Type alias for factories such as GUIPrefsFactory that produce a
    // PrefsPanel, used by the Preferences dialog in a treebook.
    // The project pointer may be null.  Usually it's not needed because
@@ -60,7 +80,7 @@ class PrefsPanel /* not final */ : public wxPanelWrapper, ComponentInterface
 
    // Typically you make a static object of this type in the .cpp file that
    // also implements the PrefsPanel subclass.
-   struct Registration final
+   struct AUDACITY_DLL_API Registration final
    {
       Registration( const wxString &name, const Factory &factory,
          bool expanded = true,
@@ -82,9 +102,9 @@ class PrefsPanel /* not final */ : public wxPanelWrapper, ComponentInterface
    virtual bool Commit() = 0; // used to be called "Apply"
 
 
-   virtual PluginPath GetPath();
-   virtual VendorSymbol GetVendor();
-   virtual wxString GetVersion();
+   virtual PluginPath GetPath() const override;
+   virtual VendorSymbol GetVendor() const override;
+   virtual wxString GetVersion() const override;
 
    //virtual ComponentInterfaceSymbol GetSymbol();
    //virtual wxString GetDescription();
@@ -95,9 +115,9 @@ class PrefsPanel /* not final */ : public wxPanelWrapper, ComponentInterface
    virtual bool ShowsPreviewButton();
    virtual void PopulateOrExchange( ShuttleGui & WXUNUSED(S) ){};
 
-   // If not empty string, the Help button is added below the panel
-   // Default returns empty string.
-   virtual wxString HelpPageName();
+   //! If not empty string, the Help button is added below the panel
+   /*! Default returns empty string. */
+   virtual ManualPageID HelpPageName();
 
    virtual void Cancel();
 };
