@@ -13,23 +13,70 @@ Paul Licameli
 
 *//*******************************************************************/
 
-#include "../Audacity.h"
+
 #include "SpectrogramSettings.h"
 
-#include "../Experimental.h"
-
-#include "../NumberScale.h"
+#include "AColor.h"
+#include "NumberScale.h"
 
 #include <algorithm>
 
-#include "../FFT.h"
-#include "../Internat.h"
-#include "../Prefs.h"
+#include "FFT.h"
+#include "Prefs.h"
 
 #include <cmath>
 
 #include "../widgets/AudacityMessageBox.h"
 
+IntSetting SpectrumMaxFreq{
+   L"/Spectrum/MaxFreq", 20000 };
+
+namespace {
+// Other settings not yet used outside of this file
+
+// To do: migrate these to ChoiceSetting preferences, which will store an
+// Identifier instead of a number in the preference file.
+IntSetting SpectrumAlgorithm{
+   L"/Spectrum/Algorithm", 0 }; // Default to Frequencies
+IntSetting SpectrumScale{
+   L"/Spectrum/ScaleType", 2 }; // Default to Mel
+IntSetting SpectrumWindowFunction{
+   L"/Spectrum/WindowType", eWinFuncHann };
+
+BoolSetting SpectrumEnableSelection{
+   L"/Spectrum/EnableSpectralSelection", true };
+IntSetting SpectrumFFTSize{
+   L"/Spectrum/FFTSize", 2048 };
+IntSetting SpectrumFrequencyGain{
+   L"/Spectrum/FrequencyGain", 0 };
+IntSetting SpectrumGain{
+   L"/Spectrum/Gain", 20 };
+BoolSetting SpectrumGrayscale{
+   L"/Spectrum/Grayscale", false };
+IntSetting SpectrumMinFreq{
+   L"/Spectrum/MinFreq", 0 };
+IntSetting SpectrumRange{
+   L"/Spectrum/Range", 80 };
+IntSetting SpectrumZeroPaddingFactor{
+   L"/Spectrum/ZeroPaddingFactor", 2 };
+
+#ifdef EXPERIMENTAL_FIND_NOTES
+BoolSetting SpectrumFindNotes{
+   L"/Spectrum/FFTFindNotes", false };
+DoubleSetting SpectrumFindNotesMinA{
+   L"/Spectrum/FindNotesMinA", -30.0 };
+IntSetting SpectrumFindNotesN{
+   L"/Spectrum/FindNotesN", 5 };
+BoolSetting SpectrumFindNotesQuantize{
+   L"/Spectrum/FindNotesQuantize", false };
+#endif //EXPERIMENTAL_FIND_NOTES
+
+#ifdef EXPERIMENTAL_FFT_Y_GRID
+BoolSetting SpectrumYGrid{
+   L"/Spectrum/FFTYGrid", false};
+#endif
+}
+
 SpectrogramSettings::Globals::Globals()
 {
    LoadPrefs();
@@ -38,15 +85,14 @@ SpectrogramSettings::Globals::Globals()
 void SpectrogramSettings::Globals::SavePrefs()
 {
 #ifdef SPECTRAL_SELECTION_GLOBAL_SWITCH
-   gPrefs->Write(wxT("/Spectrum/EnableSpectralSelection"), spectralSelection);
+   SpectrumEnableSelection.Write(spectralSelection);
 #endif
 }
 
 void SpectrogramSettings::Globals::LoadPrefs()
 {
 #ifdef SPECTRAL_SELECTION_GLOBAL_SWITCH
-   spectralSelection
-      = (gPrefs->Read(wxT("/Spectrum/EnableSpectralSelection"), 0L) != 0);
+   spectralSelection = SpectrumEnableSelection.Read();
 #endif
 }
 
@@ -70,10 +116,8 @@ SpectrogramSettings::SpectrogramSettings(const SpectrogramSettings &other)
    , frequencyGain(other.frequencyGain)
    , windowType(other.windowType)
    , windowSize(other.windowSize)
-#ifdef EXPERIMENTAL_ZERO_PADDED_SPECTROGRAMS
    , zeroPaddingFactor(other.zeroPaddingFactor)
-#endif
-   , isGrayscale(other.isGrayscale)
+   , colorScheme(other.colorScheme)
    , scaleType(other.scaleType)
 #ifndef SPECTRAL_SELECTION_GLOBAL_SWITCH
    , spectralSelection(other.spectralSelection)
@@ -107,10 +151,8 @@ SpectrogramSettings &SpectrogramSettings::operator= (const SpectrogramSettings &
       frequencyGain = other.frequencyGain;
       windowType = other.windowType;
       windowSize = other.windowSize;
-#ifdef EXPERIMENTAL_ZERO_PADDED_SPECTROGRAMS
       zeroPaddingFactor = other.zeroPaddingFactor;
-#endif
-      isGrayscale = other.isGrayscale;
+      colorScheme = other.colorScheme;
       scaleType = other.scaleType;
 #ifndef SPECTRAL_SELECTION_GLOBAL_SWITCH
       spectralSelection = other.spectralSelection;
@@ -157,6 +199,47 @@ const EnumValueSymbols &SpectrogramSettings::GetScaleNames()
    return result;
 }
 
+//static
+const EnumValueSymbols &SpectrogramSettings::GetColorSchemeNames()
+{
+   static const EnumValueSymbols result{
+      // Keep in correspondence with enum SpectrogramSettings::ColorScheme:
+      /* i18n-hint: New color scheme for spectrograms */
+      { wxT("SpecColorNew"),     XC("Color (default)",   "spectrum prefs") },
+      /* i18n-hint: Classic color scheme(from theme) for spectrograms */
+      { wxT("SpecColorTheme"),   XC("Color (classic)",   "spectrum prefs") },
+      /* i18n-hint: Grayscale color scheme for spectrograms */
+      { wxT("SpecGrayscale"),    XC("Grayscale",         "spectrum prefs") },
+      /* i18n-hint: Inverse grayscale color scheme for spectrograms */
+      { wxT("SpecInvGrayscale"), XC("Inverse grayscale", "spectrum prefs") },
+   };
+
+   wxASSERT(csNumColorScheme == result.size());
+   static_assert(csNumColorScheme == AColor::colorSchemes, "Broken correspondence");
+
+   return result;
+}
+
+
+void SpectrogramSettings::ColorSchemeEnumSetting::Migrate(wxString &value)
+{
+   // Migrate old grayscale option to Color scheme choice
+   bool isGrayscale = SpectrumGrayscale.Read();
+   if (isGrayscale && !gPrefs->Read(wxT("/Spectrum/ColorScheme"), &value)) {
+      value = GetColorSchemeNames().at(csGrayscale).Internal();
+      Write(value);
+      gPrefs->Flush();
+   }
+}
+
+SpectrogramSettings::ColorSchemeEnumSetting SpectrogramSettings::colorSchemeSetting{
+   wxT("/Spectrum/ColorScheme"),
+   GetColorSchemeNames(),
+   csColorNew, // default to Color(New)
+   { csColorNew, csColorTheme, csGrayscale, csInvGrayscale },
+};
+
+
 //static
 const TranslatableStrings &SpectrogramSettings::GetAlgorithmNames()
 {
@@ -230,6 +313,9 @@ bool SpectrogramSettings::Validate(bool quiet)
       ScaleType(std::max(0,
          std::min((int)(SpectrogramSettings::stNumScaleTypes) - 1,
             (int)(scaleType))));
+   colorScheme = ColorScheme(
+      std::max(0, std::min<int>(csNumColorScheme-1, colorScheme))
+   );
    algorithm = Algorithm(
       std::max(0, std::min((int)(algNumAlgorithms) - 1, (int)(algorithm)))
    );
@@ -241,41 +327,39 @@ bool SpectrogramSettings::Validate(bool quiet)
 
 void SpectrogramSettings::LoadPrefs()
 {
-   minFreq = gPrefs->Read(wxT("/Spectrum/MinFreq"), 0L);
+   minFreq = SpectrumMinFreq.Read();
 
-   maxFreq = gPrefs->Read(wxT("/Spectrum/MaxFreq"), 8000L);
+   maxFreq = SpectrumMaxFreq.Read();
 
-   range = gPrefs->Read(wxT("/Spectrum/Range"), 80L);
-   gain = gPrefs->Read(wxT("/Spectrum/Gain"), 20L);
-   frequencyGain = gPrefs->Read(wxT("/Spectrum/FrequencyGain"), 0L);
+   range = SpectrumRange.Read();
+   gain = SpectrumGain.Read();
+   frequencyGain = SpectrumFrequencyGain.Read();
 
-   windowSize = gPrefs->Read(wxT("/Spectrum/FFTSize"), 1024);
+   windowSize = SpectrumFFTSize.Read();
 
-#ifdef EXPERIMENTAL_ZERO_PADDED_SPECTROGRAMS
-   zeroPaddingFactor = gPrefs->Read(wxT("/Spectrum/ZeroPaddingFactor"), 1);
-#endif
+   zeroPaddingFactor = SpectrumZeroPaddingFactor.Read();
 
-   gPrefs->Read(wxT("/Spectrum/WindowType"), &windowType, eWinFuncHanning);
+   windowType = SpectrumWindowFunction.Read();
 
-   isGrayscale = (gPrefs->Read(wxT("/Spectrum/Grayscale"), 0L) != 0);
+   colorScheme = colorSchemeSetting.ReadEnum();
 
-   scaleType = ScaleType(gPrefs->Read(wxT("/Spectrum/ScaleType"), 0L));
+   scaleType = static_cast<ScaleType>(SpectrumScale.Read());
 
 #ifndef SPECTRAL_SELECTION_GLOBAL_SWITCH
-   spectralSelection = (gPrefs->Read(wxT("/Spectrum/EnableSpectralSelection"), 1L) != 0);
+   spectralSelection = SpectrumEnableSelection.Read();
 #endif
 
-   algorithm = Algorithm(gPrefs->Read(wxT("/Spectrum/Algorithm"), 0L));
+   algorithm = static_cast<Algorithm>(SpectrumAlgorithm.Read());
 
 #ifdef EXPERIMENTAL_FFT_Y_GRID
-   fftYGrid = (gPrefs->Read(wxT("/Spectrum/FFTYGrid"), 0L) != 0);
+   fftYGrid = SpectrumYGrid.Read();
 #endif //EXPERIMENTAL_FFT_Y_GRID
 
 #ifdef EXPERIMENTAL_FIND_NOTES
-   fftFindNotes = (gPrefs->Read(wxT("/Spectrum/FFTFindNotes"), 0L) != 0);
-   findNotesMinA = gPrefs->Read(wxT("/Spectrum/FindNotesMinA"), -30.0);
-   numberOfMaxima = gPrefs->Read(wxT("/Spectrum/FindNotesN"), 5L);
-   findNotesQuantize = (gPrefs->Read(wxT("/Spectrum/FindNotesQuantize"), 0L) != 0);
+   fftFindNotes = SpectrumFindNotes.Read();
+   findNotesMinA = SpectrumFindNotesMinA.Read();
+   numberOfMaxima = SpectrumFindNotesN.Read();
+   findNotesQuantize = SpectrumFindNotesQuantize.Read();
 #endif //EXPERIMENTAL_FIND_NOTES
 
    // Enforce legal values
@@ -286,47 +370,110 @@ void SpectrogramSettings::LoadPrefs()
 
 void SpectrogramSettings::SavePrefs()
 {
-   gPrefs->Write(wxT("/Spectrum/MinFreq"), minFreq);
-   gPrefs->Write(wxT("/Spectrum/MaxFreq"), maxFreq);
+   SpectrumMinFreq.Write(minFreq);
+   SpectrumMaxFreq.Write(maxFreq);
 
    // Nothing wrote these.  They only varied from the linear scale bounds in-session. -- PRL
    // gPrefs->Write(wxT("/SpectrumLog/MaxFreq"), logMinFreq);
    // gPrefs->Write(wxT("/SpectrumLog/MinFreq"), logMaxFreq);
 
-   gPrefs->Write(wxT("/Spectrum/Range"), range);
-   gPrefs->Write(wxT("/Spectrum/Gain"), gain);
-   gPrefs->Write(wxT("/Spectrum/FrequencyGain"), frequencyGain);
+   SpectrumRange.Write(range);
+   SpectrumGain.Write(gain);
+   SpectrumFrequencyGain.Write(frequencyGain);
 
-   gPrefs->Write(wxT("/Spectrum/FFTSize"), windowSize);
+   SpectrumFFTSize.Write(windowSize);
 
-#ifdef EXPERIMENTAL_ZERO_PADDED_SPECTROGRAMS
-   gPrefs->Write(wxT("/Spectrum/ZeroPaddingFactor"), zeroPaddingFactor);
-#endif
+   SpectrumZeroPaddingFactor.Write(zeroPaddingFactor);
 
-   gPrefs->Write(wxT("/Spectrum/WindowType"), windowType);
+   SpectrumWindowFunction.Write(windowType);
 
-   gPrefs->Write(wxT("/Spectrum/Grayscale"), isGrayscale);
+   colorSchemeSetting.WriteEnum(colorScheme);
 
-   gPrefs->Write(wxT("/Spectrum/ScaleType"), (int) scaleType);
+   SpectrumScale.Write(static_cast<int>(scaleType));
 
 #ifndef SPECTRAL_SELECTION_GLOBAL_SWITCH
-   gPrefs->Write(wxT("/Spectrum/EnableSpectralSelection"), spectralSelection);
+   SpectrumEnableSelection.Write(spectralSelection);
 #endif
 
-   gPrefs->Write(wxT("/Spectrum/Algorithm"), (int) algorithm);
+   SpectrumAlgorithm.Write(static_cast<int>(algorithm));
 
 #ifdef EXPERIMENTAL_FFT_Y_GRID
-   gPrefs->Write(wxT("/Spectrum/FFTYGrid"), fftYGrid);
+   SpectrumYGrid.Write(fftYGrid);
 #endif //EXPERIMENTAL_FFT_Y_GRID
 
 #ifdef EXPERIMENTAL_FIND_NOTES
-   gPrefs->Write(wxT("/Spectrum/FFTFindNotes"), fftFindNotes);
-   gPrefs->Write(wxT("/Spectrum/FindNotesMinA"), findNotesMinA);
-   gPrefs->Write(wxT("/Spectrum/FindNotesN"), numberOfMaxima);
-   gPrefs->Write(wxT("/Spectrum/FindNotesQuantize"), findNotesQuantize);
+   SpectrumFindNotes.Write(fftFindNotes);
+   SpectrumFindNotesMinA.Write(findNotesMinA);
+   SpectrumFindNotesN.Write(numberOfMaxima);
+   SpectrumFindNotesQuantize.Write(findNotesQuantize);
 #endif //EXPERIMENTAL_FIND_NOTES
 }
 
+// This is a temporary hack until SpectrogramSettings gets fully integrated
+void SpectrogramSettings::UpdatePrefs()
+{
+   if (minFreq == defaults().minFreq)
+      minFreq = SpectrumMinFreq.Read();
+
+   if (maxFreq == defaults().maxFreq)
+      maxFreq = SpectrumMaxFreq.Read();
+
+   if (range == defaults().range)
+      range = SpectrumRange.Read();
+
+   if (gain == defaults().gain)
+      gain = SpectrumGain.Read();
+
+   if (frequencyGain == defaults().frequencyGain)
+      frequencyGain = SpectrumFrequencyGain.Read();
+
+   if (windowSize == defaults().windowSize)
+      windowSize = SpectrumFFTSize.Read();
+
+   if (zeroPaddingFactor == defaults().zeroPaddingFactor)
+      zeroPaddingFactor = SpectrumZeroPaddingFactor.Read();
+
+   if (windowType == defaults().windowType)
+      windowType = SpectrumWindowFunction.Read();
+
+   if (colorScheme == defaults().colorScheme) {
+      colorScheme = colorSchemeSetting.ReadEnum();
+   }
+
+   if (scaleType == defaults().scaleType)
+      scaleType = static_cast<ScaleType>(SpectrumScale.Read());
+
+#ifndef SPECTRAL_SELECTION_GLOBAL_SWITCH
+   if (spectralSelection == defaults().spectralSelection)
+      spectralSelection = SpectrumEnableSelection.Read();
+#endif
+
+   if (algorithm == defaults().algorithm)
+      algorithm = static_cast<Algorithm>(SpectrumAlgorithm.Read());
+
+#ifdef EXPERIMENTAL_FFT_Y_GRID
+   if (fftYGrid == defaults().fftYGrid)
+      fftYGrid = SpectrumYGrid.Read();
+#endif //EXPERIMENTAL_FFT_Y_GRID
+
+#ifdef EXPERIMENTAL_FIND_NOTES
+   if (fftFindNotes == defaults().fftFindNotes)
+      fftFindNotes = SpectrumFindNotes.Read();
+
+   if (findNotesMinA == defaults().findNotesMinA)
+      findNotesMinA = SpectrumFindNotesMinA.Read();
+
+   if (numberOfMaxima == defaults().numberOfMaxima)
+      numberOfMaxima = SpectrumFindNotesN.Read();
+
+   if (findNotesQuantize == defaults().findNotesQuantize)
+      findNotesQuantize = SpectrumFindNotesQuantize.Read();
+#endif //EXPERIMENTAL_FIND_NOTES
+
+   // Enforce legal values
+   Validate(true);
+}
+
 void SpectrogramSettings::InvalidateCaches()
 {
    DestroyWindows();
@@ -407,8 +554,9 @@ void SpectrogramSettings::CacheWindows() const
    if (hFFT == NULL || window == NULL) {
 
       double scale;
-      const auto fftLen = WindowSize() * ZeroPaddingFactor();
-      const auto padding = (WindowSize() * (zeroPaddingFactor - 1)) / 2;
+      auto factor = ZeroPaddingFactor();
+      const auto fftLen = WindowSize() * factor;
+      const auto padding = (WindowSize() * (factor - 1)) / 2;
 
       hFFT = GetFFT(fftLen);
       RecreateWindow(window, WINDOW, fftLen, padding, windowType, windowSize, scale);
@@ -430,7 +578,6 @@ void SpectrogramSettings::ConvertToEnumeratedWindowSizes()
       size >>= 1, ++logarithm;
    windowSize = std::max(0, std::min(NumWindowSizes - 1, logarithm));
 
-#ifdef EXPERIMENTAL_ZERO_PADDED_SPECTROGRAMS
    // Choices for zero padding begin at 1
    logarithm = 0;
    size = unsigned(zeroPaddingFactor);
@@ -440,15 +587,12 @@ void SpectrogramSettings::ConvertToEnumeratedWindowSizes()
       std::min(LogMaxWindowSize - (windowSize + LogMinWindowSize),
          logarithm
    ));
-#endif
 }
 
 void SpectrogramSettings::ConvertToActualWindowSizes()
 {
    windowSize = 1 << (windowSize + LogMinWindowSize);
-#ifdef EXPERIMENTAL_ZERO_PADDED_SPECTROGRAMS
    zeroPaddingFactor = 1 << zeroPaddingFactor;
-#endif
 }
 
 float SpectrogramSettings::findBin( float frequency, float binUnit ) const
@@ -462,11 +606,11 @@ float SpectrogramSettings::findBin( float frequency, float binUnit ) const
 
 size_t SpectrogramSettings::GetFFTLength() const
 {
-#ifndef EXPERIMENTAL_ZERO_PADDED_SPECTROGRAMS
-   return windowSize;
-#else
+//#ifndef EXPERIMENTAL_ZERO_PADDED_SPECTROGRAMS
+  // return windowSize;
+//#else
    return windowSize * ((algorithm != algPitchEAC) ? zeroPaddingFactor : 1);
-#endif
+//#endif
 }
 
 size_t SpectrogramSettings::NBins() const
