@@ -24,16 +24,16 @@ class ShuttleGui;
 
 #define TRACKS_PREFS_PLUGIN_SYMBOL ComponentInterfaceSymbol{ XO("Tracks") }
 
-class TracksPrefs final : public PrefsPanel
+class AUDACITY_DLL_API TracksPrefs final : public PrefsPanel
 {
  public:
    TracksPrefs(wxWindow * parent, wxWindowID winid);
    ~TracksPrefs();
-   ComponentInterfaceSymbol GetSymbol() override;
-   TranslatableString GetDescription() override;
+   ComponentInterfaceSymbol GetSymbol() const override;
+   TranslatableString GetDescription() const override;
 
    bool Commit() override;
-   wxString HelpPageName() override;
+   ManualPageID HelpPageName() override;
 
    static bool GetPinnedHeadPreference();
    static void SetPinnedHeadPreference(bool value, bool flush = false);
@@ -41,8 +41,6 @@ class TracksPrefs final : public PrefsPanel
    static double GetPinnedHeadPositionPreference();
    static void SetPinnedHeadPositionPreference(double value, bool flush = false);
    
-   static wxString GetDefaultAudioTrackNamePreference();
-
    static WaveTrackViewConstants::Display ViewModeChoice();
    static WaveformSettings::ScaleTypeValues WaveformScaleChoice();
    static WaveTrackViewConstants::SampleDisplay SampleViewChoice();
