@@ -11,16 +11,16 @@ Paul Licameli
 #ifndef __AUDACITY_WAVEFORM_SETTINGS__
 #define __AUDACITY_WAVEFORM_SETTINGS__
 
-#include "../Internat.h" // for TranslatableStrings
+#include "Prefs.h"
 
 class EnumValueSymbols;
 
-class WaveformSettings
+class AUDACITY_DLL_API WaveformSettings : public PrefsListener
 {
 public:
 
    // Singleton for settings that are not per-track
-   class Globals
+   class AUDACITY_DLL_API Globals
    {
    public:
       static Globals &Get();
@@ -47,6 +47,8 @@ public:
    void SavePrefs();
    void Update();
 
+   void UpdatePrefs() override;
+
    void ConvertToEnumeratedDBRange();
    void ConvertToActualDBRange();
    void NextLowerDBRange();
