@@ -30,10 +30,8 @@
 
 *//*******************************************************************/
 
-#include "../Audacity.h" // for USE_* macros
-#include "ControlToolBar.h"
 
-#include "../Experimental.h"
+#include "ControlToolBar.h"
 
 #include <algorithm>
 #include <cfloat>
@@ -56,21 +54,22 @@
 #include <wx/tooltip.h>
 #include <wx/datetime.h>
 
-#include "../AColor.h"
-#include "../AllThemeResources.h"
+#include "AColor.h"
+#include "AllThemeResources.h"
 #include "../AudioIO.h"
-#include "../ImageManipulation.h"
-#include "../Prefs.h"
-#include "../Project.h"
+#include "ImageManipulation.h"
+#include "Prefs.h"
+#include "Project.h"
 #include "../ProjectAudioIO.h"
 #include "../ProjectAudioManager.h"
 #include "../ProjectSettings.h"
-#include "../ProjectStatus.h"
+#include "ProjectStatus.h"
 #include "../ProjectWindow.h"
-#include "../Track.h"
+#include "../SelectUtilities.h"
+#include "Track.h"
+#include "ViewInfo.h"
 #include "../widgets/AButton.h"
-#include "../widgets/ErrorDialog.h"
-#include "../FileNames.h"
+#include "FileNames.h"
 
 #include "../tracks/ui/Scrubbing.h"
 #include "../toolbars/ToolManager.h"
@@ -89,18 +88,22 @@ BEGIN_EVENT_TABLE(ControlToolBar, ToolBar)
    EVT_BUTTON(ID_REW_BUTTON,    ControlToolBar::OnRewind)
    EVT_BUTTON(ID_FF_BUTTON,     ControlToolBar::OnFF)
    EVT_BUTTON(ID_PAUSE_BUTTON,  ControlToolBar::OnPause)
+   EVT_BUTTON(ID_LOOP_BUTTON,   ControlToolBar::OnLoop)
    EVT_IDLE(ControlToolBar::OnIdle)
 END_EVENT_TABLE()
 
 static const TranslatableString
    /* i18n-hint: These are strings for the status bar, and indicate whether Audacity
-   is playing or recording or stopped, and whether it is paused. */
+   is playing or recording or stopped, and whether it is paused;
+   progressive verb form */
      sStatePlay = XO("Playing")
    /* i18n-hint: These are strings for the status bar, and indicate whether Audacity
-   is playing or recording or stopped, and whether it is paused. */
+   is playing or recording or stopped, and whether it is paused;
+   progressive verb form */
    , sStateStop = XO("Stopped")
    /* i18n-hint: These are strings for the status bar, and indicate whether Audacity
-   is playing or recording or stopped, and whether it is paused. */
+   is playing or recording or stopped, and whether it is paused;
+   progressive verb form */
    , sStateRecord = XO("Recording")
 ;
 
@@ -112,7 +115,6 @@ static const TranslatableString
 ControlToolBar::ControlToolBar( AudacityProject &project )
 : ToolBar(project, TransportBarID, XO("Transport"), wxT("Control"))
 {
-   gPrefs->Read(wxT("/GUI/ErgonomicTransportButtons"), &mErgonomicTransportButtons, true);
    mStrLocale = gPrefs->Read(wxT("/Locale/Language"), wxT(""));
 
    mSizer = NULL;
@@ -148,33 +150,6 @@ void ControlToolBar::Create(wxWindow * parent)
    UpdatePrefs();
 }
 
-// This is a convenience function that allows for button creation in
-// MakeButtons() with fewer arguments
-AButton *ControlToolBar::MakeButton(ControlToolBar *pBar,
-                                    teBmps eEnabledUp, teBmps eEnabledDown, teBmps eDisabled,
-                                    int id,
-                                    bool processdownevents,
-                                    const TranslatableString &label)
-{
-   AButton *r = ToolBar::MakeButton(pBar,
-      bmpRecoloredUpLarge, bmpRecoloredDownLarge, bmpRecoloredUpHiliteLarge, bmpRecoloredHiliteLarge,
-      eEnabledUp, eEnabledDown, eDisabled,
-      wxWindowID(id),
-      wxDefaultPosition, processdownevents,
-      theTheme.ImageSize( bmpRecoloredUpLarge ));
-   r->SetLabel(label);
-   enum { deflation =
-#ifdef __WXMAC__
-      6
-#else
-      12
-#endif
-   };
-   r->SetFocusRect( r->GetClientRect().Deflate( deflation, deflation ) );
-
-   return r;
-}
-
 // static
 void ControlToolBar::MakeAlternateImages(AButton &button, int idx,
                                          teBmps eEnabledUp,
@@ -197,7 +172,8 @@ void ControlToolBar::Populate()
 
    mPlay = MakeButton(this, bmpPlay, bmpPlay, bmpPlayDisabled,
       ID_PLAY_BUTTON, true, XO("Play"));
-   MakeAlternateImages(*mPlay, 1, bmpLoop, bmpLoop, bmpLoopDisabled);
+   // 3.1.0 abandoned distinct images for Shift
+   MakeAlternateImages(*mPlay, 1, bmpPlay, bmpPlay, bmpPlayDisabled);
    MakeAlternateImages(*mPlay, 2,
       bmpCutPreview, bmpCutPreview, bmpCutPreviewDisabled);
    MakeAlternateImages(*mPlay, 3,
@@ -229,6 +205,11 @@ void ControlToolBar::Populate()
 
    mRecord->FollowModifierKeys();
 
+   mLoop = MakeButton(this, bmpLoop, bmpLoop, bmpLoopDisabled,
+      ID_LOOP_BUTTON,
+      true, // this makes it a toggle, like the pause button
+      LoopToggleText.Stripped());
+
 #if wxUSE_TOOLTIPS
    RegenerateTooltips();
    wxToolTip::Enable(true);
@@ -250,7 +231,7 @@ void ControlToolBar::RegenerateTooltips()
       {
          case ID_PLAY_BUTTON:
             // Without shift
-            name = wxT("PlayStop");
+            name = wxT("DefaultPlayStop");
             break;
          case ID_RECORD_BUTTON:
             // Without shift
@@ -269,6 +250,9 @@ void ControlToolBar::RegenerateTooltips()
          case ID_REW_BUTTON:
             name = wxT("CursProjectStart");
             break;
+         case ID_LOOP_BUTTON:
+            name = wxT("TogglePlayRegion");
+            break;
       }
       std::vector<ComponentInterfaceSymbol> commands(
          1u, { name, Verbatim( pCtrl->GetLabel() ) } );
@@ -278,7 +262,7 @@ void ControlToolBar::RegenerateTooltips()
       {
          case ID_PLAY_BUTTON:
             // With shift
-            commands.push_back( { wxT("PlayLooped"), XO("Loop Play") } );
+            commands.push_back( { wxT("OncePlayStop"), XO("Play Once") } );
             break;
          case ID_RECORD_BUTTON:
             // With shift
@@ -319,12 +303,6 @@ void ControlToolBar::UpdatePrefs()
    bool updated = false;
    bool active;
 
-   gPrefs->Read( wxT("/GUI/ErgonomicTransportButtons"), &active, true );
-   if( mErgonomicTransportButtons != active )
-   {
-      mErgonomicTransportButtons = active;
-      updated = true;
-   }
    wxString strLocale = gPrefs->Read(wxT("/Locale/Language"), wxT(""));
    if (mStrLocale != strLocale)
    {
@@ -367,37 +345,21 @@ void ControlToolBar::ArrangeButtons()
    // Start with a little extra space
    mSizer->Add( 5, 55 );
 
-   // Add the buttons in order based on ergonomic setting
-   if( mErgonomicTransportButtons )
-   {
-      mPause->MoveBeforeInTabOrder( mRecord );
-      mPlay->MoveBeforeInTabOrder( mRecord );
-      mStop->MoveBeforeInTabOrder( mRecord );
-      mRewind->MoveBeforeInTabOrder( mRecord );
-      mFF->MoveBeforeInTabOrder( mRecord );
+   // Establish correct tab key sequence with mLoop last
+   mPause->MoveBeforeInTabOrder( mLoop );
+   mPlay->MoveBeforeInTabOrder( mLoop );
+   mStop->MoveBeforeInTabOrder( mLoop );
+   mRewind->MoveBeforeInTabOrder( mLoop );
+   mFF->MoveBeforeInTabOrder( mLoop );
+   mRecord->MoveBeforeInTabOrder( mLoop );
 
-      mSizer->Add( mPause,  0, flags, 2 );
-      mSizer->Add( mPlay,   0, flags, 2 );
-      mSizer->Add( mStop,   0, flags, 2 );
-      mSizer->Add( mRewind, 0, flags, 2 );
-      mSizer->Add( mFF,     0, flags, 10 );
-      mSizer->Add( mRecord, 0, flags, 5 );
-   }
-   else
-   {
-      mRewind->MoveBeforeInTabOrder( mFF );
-      mPlay->MoveBeforeInTabOrder( mFF );
-      mRecord->MoveBeforeInTabOrder( mFF );
-      mPause->MoveBeforeInTabOrder( mFF );
-      mStop->MoveBeforeInTabOrder( mFF );
-
-      mSizer->Add( mRewind, 0, flags, 2 );
-      mSizer->Add( mPlay,   0, flags, 2 );
-      mSizer->Add( mRecord, 0, flags, 2 );
-      mSizer->Add( mPause,  0, flags, 2 );
-      mSizer->Add( mStop,   0, flags, 2 );
-      mSizer->Add( mFF,     0, flags, 5 );
-   }
+   mSizer->Add( mPause,  0, flags, 2 );
+   mSizer->Add( mPlay,   0, flags, 2 );
+   mSizer->Add( mStop,   0, flags, 2 );
+   mSizer->Add( mRewind, 0, flags, 2 );
+   mSizer->Add( mFF,     0, flags, 10 );
+   mSizer->Add( mRecord, 0, flags, 10 );
+   mSizer->Add( mLoop, 0, flags, 5 );
 
    // Layout the sizer
    mSizer->Layout();
@@ -416,6 +378,7 @@ void ControlToolBar::ReCreateButtons()
    bool pauseDown = false;
    bool recordDown = false;
    bool recordShift = false;
+   bool loopDown = false;
 
    // ToolBar::ReCreateButtons() will get rid of its sizer and
    // since we've attached our sizer to it, ours will get deleted too
@@ -427,6 +390,7 @@ void ControlToolBar::ReCreateButtons()
       pauseDown = mPause->IsDown();
       recordDown = mRecord->IsDown();
       recordShift = mRecord->WasShiftDown();
+      loopDown = mLoop->IsDown();
       Detach( mSizer );
 
       std::unique_ptr < wxSizer > {mSizer}; // DELETE it
@@ -454,6 +418,9 @@ void ControlToolBar::ReCreateButtons()
       mRecord->PushDown();
    }
 
+   if (loopDown)
+      mLoop->PushDown();
+
    EnableDisableButtons();
 
    RegenerateTooltips();
@@ -496,6 +463,8 @@ void ControlToolBar::EnableDisableButtons()
    mFF->SetEnabled(tracks && (paused || (!playing && !recording)));
 
    mPause->SetEnabled(canStop);
+
+   mLoop->SetEnabled( !recording );
 }
 
 void ControlToolBar::SetPlay(bool down, PlayAppearance appearance)
@@ -576,15 +545,16 @@ void ControlToolBar::OnStop(wxCommandEvent & WXUNUSED(evt))
 
 void ControlToolBar::PlayDefault()
 {
-   // Let control have precedence over shift
+   // Let control-down have precedence over shift state
    const bool cutPreview = mPlay->WasControlDown();
-   const bool looped = !cutPreview &&
-      mPlay->WasShiftDown();
-   ProjectAudioManager::Get( mProject ).PlayCurrentRegion(looped, cutPreview);
+   const bool newDefault = !cutPreview &&
+      !mPlay->WasShiftDown();
+   ProjectAudioManager::Get( mProject )
+      .PlayCurrentRegion(newDefault, cutPreview);
 }
 
+/*! @excsafety{Strong} -- For state of current project's tracks */
 void ControlToolBar::OnRecord(wxCommandEvent &evt)
-// STRONG-GUARANTEE (for state of current project's tracks)
 {
    // TODO: It would be neater if Menu items and Toolbar buttons used the same code for
    // enabling/disabling, and all fell into the same action routines.
@@ -600,10 +570,23 @@ void ControlToolBar::OnPause(wxCommandEvent & WXUNUSED(evt))
    ProjectAudioManager::Get( mProject ).OnPause();
 }
 
+void ControlToolBar::OnLoop(wxCommandEvent & WXUNUSED(evt))
+{
+   // Toggle the state of the play region lock
+   auto &region = ViewInfo::Get(mProject).playRegion;
+   if (region.Active())
+      SelectUtilities::InactivatePlayRegion(mProject);
+   else
+      SelectUtilities::ActivatePlayRegion(mProject);
+}
+
 void ControlToolBar::OnIdle(wxIdleEvent & event)
 {
    event.Skip();
 
+   if (!wxTheApp->IsActive())
+      return;
+
    auto &projectAudioManager = ProjectAudioManager::Get( mProject );
    if ( projectAudioManager.Paused() )
       mPause->PushDown();
@@ -633,11 +616,13 @@ void ControlToolBar::OnIdle(wxIdleEvent & event)
    }
    else {
       mPlay->PushDown();
+      // Choose among alternative appearances of the play button, although
+      // options 0 and 1 became non-distinct in 3.1.0
       mPlay->SetAlternateIdx(
          projectAudioManager.Cutting()
          ? 2
-         : projectAudioManager.Looping()
-            ? 1
+         // : projectAudioManager.Looping()
+            // ? 1
             : 0
       );
    }
@@ -652,6 +637,11 @@ void ControlToolBar::OnIdle(wxIdleEvent & event)
    else
       // push-downs of the stop button are only momentary and always pop up now
       mStop->PopUp();
+
+   if (ViewInfo::Get(mProject).playRegion.Active())
+      mLoop->PushDown();
+   else
+      mLoop->PopUp();
    
    UpdateStatusBar();
    EnableDisableButtons();
