@@ -26,7 +26,7 @@ with changes in the SelectionBar.
 *//*******************************************************************/
 
 
-#include "../Audacity.h"
+
 #include "SelectionBar.h"
 
 #include "SelectionBarListener.h"
@@ -51,17 +51,18 @@ with changes in the SelectionBar.
 #include <wx/statline.h>
 
 
-#include "../widgets/AButton.h"
-#include "../AudioIOBase.h"
-#include "../AColor.h"
+#include "../AudioIO.h"
+#include "AColor.h"
 #include "../KeyboardCapture.h"
-#include "../Prefs.h"
-#include "../Project.h"
+#include "Prefs.h"
+#include "Project.h"
 #include "../ProjectAudioIO.h"
 #include "../ProjectSettings.h"
 #include "../Snap.h"
-#include "../ViewInfo.h"
-#include "../AllThemeResources.h"
+#include "ViewInfo.h"
+#include "QualitySettings.h"
+#include "AllThemeResources.h"
+#include "../widgets/auStaticText.h"
 
 #if wxUSE_ACCESSIBILITY
 #include "../widgets/WindowAccessible.h"
@@ -124,8 +125,7 @@ SelectionBar::SelectionBar( AudacityProject &project )
    // Refer to bug #462 for a scenario where the division-by-zero causes
    // Audacity to fail.
    // We expect mRate to be set from the project later.
-   mRate = (double) gPrefs->Read(wxT("/SamplingRate/DefaultProjectSampleRate"),
-      AudioIOBase::GetOptimalSupportedSampleRate());
+   mRate = (double) QualitySettings::DefaultSampleRate.Read();
 
    // Selection mode of 0 means showing 'start' and 'end' only.
    mSelectionMode = gPrefs->ReadLong(wxT("/SelectionToolbarMode"),  0);
@@ -153,11 +153,14 @@ void SelectionBar::Create(wxWindow * parent)
 }
 
 
-auStaticText * SelectionBar::AddTitle( const wxString & Title, wxSizer * pSizer ){
-   auStaticText * pTitle = safenew auStaticText(this, Title );
+auStaticText * SelectionBar::AddTitle(
+   const TranslatableString & Title, wxSizer * pSizer ){
+   const auto translated = Title.Translation();
+   auStaticText * pTitle = safenew auStaticText(this, translated );
    pTitle->SetBackgroundColour( theTheme.Colour( clrMedium ));
    pTitle->SetForegroundColour( theTheme.Colour( clrTrackPanelText ) );
-   pSizer->Add( pTitle,0, wxALIGN_CENTER_VERTICAL | wxRIGHT,  (Title.length() == 1 ) ? 0:5);
+   pSizer->Add( pTitle, 0, wxEXPAND | wxALIGN_CENTER_VERTICAL | wxRIGHT, 5 );
+
    return pTitle;
 }
 
@@ -186,38 +189,22 @@ void SelectionBar::Populate()
 
    mStartTime = mEndTime = mLengthTime = mCenterTime = mAudioTime = nullptr;
 
-   // This will be inherited by all children:
-   SetFont(wxFont(
-#ifdef __WXMAC__
-                  12
-#else
-                  9
-#endif
-                  ,
-                  wxFONTFAMILY_SWISS, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL));
-
-   wxFlexGridSizer *mainSizer;
-
-   /* we don't actually need a control yet, but we want to use its methods
-    * to do some look-ups, so we'll have to create one. We can't make the
-    * look-ups static because they depend on translations which are done at
-    * runtime */
-
    // Outer sizer has space top and left.
    // Inner sizers have space on right only.
    // This choice makes for a nice border and internal spacing and places clear responsibility
    // on each sizer as to what spacings it creates.
-   Add((mainSizer = safenew wxFlexGridSizer(SIZER_COLS, 1, 1)), 0, wxALIGN_TOP | wxLEFT | wxTOP, 5);
+   wxFlexGridSizer *mainSizer = safenew wxFlexGridSizer(SIZER_COLS, 1, 1);
+   Add(mainSizer, 0, wxALIGN_CENTER_VERTICAL | wxLEFT, 5);
 
    // Top row (mostly labels)
    wxColour clrText =  theTheme.Colour( clrTrackPanelText );
    wxColour clrText2 = *wxBLUE;
-   AddTitle( _("Project Rate (Hz)"), mainSizer );
+   auStaticText *rateLabel = AddTitle( XO("Project Rate (Hz)"), mainSizer );
    AddVLine( mainSizer );
-   AddTitle( _("Snap-To"), mainSizer );
+   auStaticText *snapLabel = AddTitle( XO("Snap-To"), mainSizer );
    AddVLine( mainSizer );
 #ifdef TIME_IN_SELECT_TOOLBAR
-   AddTitle( _("Audio Position"), mainSizer );
+   AddTitle( XO("Audio Position"), mainSizer );
    AddVLine( mainSizer );
 #endif
 
@@ -236,20 +223,13 @@ void SelectionBar::Populate()
       // so that name can be set on a standard control
       mChoice->SetAccessible(safenew WindowAccessible(mChoice));
 #endif
-#ifdef __WXGTK__
-      // Combo boxes are taller on Linux, and if we don't do the following, the selection toolbar will
-      // be three units high.
-      wxSize sz = mChoice->GetBestSize();
-      sz.SetHeight( sz.y-4);
-      mChoice->SetMinSize( sz );
-#endif
-      mainSizer->Add(mChoice, 0, wxALIGN_TOP | wxEXPAND | wxRIGHT, 6);
+      mainSizer->Add(mChoice, 0, wxEXPAND | wxALIGN_TOP | wxRIGHT, 6);
    }
 
    // Bottom row, (mostly time controls)
    mRateBox = safenew wxComboBox(this, RateID,
                              wxT(""),
-                             wxDefaultPosition, wxSize(80, -1));
+                             wxDefaultPosition, wxDefaultSize);
 #if wxUSE_ACCESSIBILITY
    // so that name can be set on a standard control
    mRateBox->SetAccessible(safenew WindowAccessible(mRateBox));
@@ -288,31 +268,16 @@ void SelectionBar::Populate()
                       &SelectionBar::OnFocus,
                       this);
 
-#ifdef __WXGTK__
-   // Combo boxes are taller on Linux, and if we don't do the following, the selection toolbar will
-   // be three units high.
-   wxSize sz = mRateBox->GetBestSize();
-   sz.SetHeight( sz.y-4);
-   mRateBox->SetMinSize( sz );
-#endif
+   mainSizer->Add(mRateBox, 0, wxEXPAND | wxALIGN_TOP | wxRIGHT, 5);
 
-   mainSizer->Add(mRateBox, 0, wxALIGN_TOP | wxRIGHT, 5);
    AddVLine( mainSizer );
 
    mSnapTo = safenew wxChoice(this, SnapToID,
-                          wxDefaultPosition, wxDefaultSize,
-                          SnapManager::GetSnapLabels());
+      wxDefaultPosition, wxDefaultSize,
+      transform_container< wxArrayStringEx >(
+         SnapManager::GetSnapLabels(),
+         std::mem_fn( &TranslatableString::Translation ) ) );
 
-#ifdef __WXGTK__
-   // Combo boxes are taller on Linux, and if we don't do the following, the selection toolbar will
-   // be three units high.
-   sz = mSnapTo->GetBestSize();
-   sz.SetHeight( sz.y-4);
-   mSnapTo->SetMinSize( sz );
-#endif
-
-   mainSizer->Add(mSnapTo,
-                  0, wxALIGN_TOP | wxRIGHT, 5);
 #if wxUSE_ACCESSIBILITY
    // so that name can be set on a standard control
    mSnapTo->SetAccessible(safenew WindowAccessible(mSnapTo));
@@ -328,6 +293,8 @@ void SelectionBar::Populate()
                     &SelectionBar::OnFocus,
                     this);
 
+   mainSizer->Add(mSnapTo, 0, wxEXPAND | wxALIGN_TOP | wxRIGHT, 5);
+
    AddVLine( mainSizer );
 
 #ifdef TIME_IN_SELECT_TOOLBAR
@@ -336,7 +303,7 @@ void SelectionBar::Populate()
    // It works around a wxWidgets-on-Windows RadioButton bug, where tabbing
    // into the radiobutton group jumps to selecting the first item in the 
    // group even if some other item had been selected.
-   // It is an important bug to work around for sceen reader users, who use TAB 
+   // It is an important bug to work around for screen reader users, who use TAB 
    // a lot in navigation.
    // More about the bug here:
    // https://forums.wxwidgets.org/viewtopic.php?t=41120
@@ -353,6 +320,26 @@ void SelectionBar::Populate()
       mainSizer->Add(hSizer.release(), 0, wxALIGN_TOP | wxRIGHT, 0);
    }
 
+#if defined(__WXGTK3__)
+   // Nothing special
+#elif defined(__WXGTK__)
+   // Ensure the font fits inside (hopefully)
+   wxFont font = mChoice->GetFont();
+   font.Scale((double) toolbarSingle / mChoice->GetSize().GetHeight());
+
+   rateLabel->SetFont(font);
+   snapLabel->SetFont(font);
+   mChoice->SetFont(font);
+   mRateBox->SetFont(font);
+   mRateText->SetFont(font);
+   mSnapTo->SetFont(font);
+#endif
+
+   // Make sure they are fully expanded to the longest item
+   mChoice->SetMinSize(wxSize(mChoice->GetBestSize().x, toolbarSingle));
+   mRateBox->SetMinSize(wxSize(mRateBox->GetBestSize().x, toolbarSingle));
+   mSnapTo->SetMinSize(wxSize(mSnapTo->GetBestSize().x, toolbarSingle));
+
    mChoice->MoveBeforeInTabOrder( mStartTime );
    // This shows/hides controls.
    // Do this before layout so that we are sized right.
@@ -360,8 +347,6 @@ void SelectionBar::Populate()
    mainSizer->Layout();
    RegenerateTooltips();
    Layout();
-
-   SetMinSize( GetSizer()->GetMinSize() );
 }
 
 void SelectionBar::UpdatePrefs()
@@ -371,10 +356,20 @@ void SelectionBar::UpdatePrefs()
    // If necessary we can drive the SelectionBar mRate via the Project
    // calling our SetRate().
    // As of 13-Sep-2018, changes to the sample rate pref will only affect 
-   // creation of new projects, not the smaple rate in existing ones.
+   // creation of new projects, not the sample rate in existing ones.
 
+   // This will only change the selection mode during a "Reset Configuration"
+   // action since the read value will be the same during a normal preferences
+   // update.
+   mSelectionMode = gPrefs->ReadLong(wxT("/SelectionToolbarMode"),  0);
+
+   // This will only change the time format during a "Reset Configuration"
+   // action since the read value will be the same during a normal preferences
+   // update.
    wxCommandEvent e;
-   e.SetInt(mStartTime->GetFormatIndex());
+   e.SetString(NumericTextCtrl::LookupFormat(
+               NumericConverter::TIME,
+               gPrefs->Read(wxT("/SelectionFormat"), wxT(""))).Internal());
    OnUpdate(e);
 
    // Set label to pull in language change
@@ -442,12 +437,12 @@ void SelectionBar::ModifySelection(int newDriver, bool done)
    // Here we compute 'i' which combines the identity of the two 
    // driving controls, to use it as an index.
    // The order of the two drivers generally does not matter much,
-   // except that we have want:
+   // except that we want:
    //    start < end
    // and preserve that by adjusting the least dominant driving
    // control.
    int i = mDrive1 + 4 * mDrive2;
-   switch(i){
+   switch(i) {
    case StartTimeID + 4 * EndTimeID:
       if( mEnd < mStart )
          mStart = mEnd;
@@ -457,51 +452,42 @@ void SelectionBar::ModifySelection(int newDriver, bool done)
       mLength = mEnd - mStart;
       mCenter = (mStart+mEnd)/2.0;
       break;
+
    case StartTimeID + 4 * LengthTimeID:
    case StartTimeID * 4 + LengthTimeID:
-      if( mLength < 0 )
-         mLength = 0;
       mEnd = mStart+mLength;
       mCenter = (mStart+mEnd)/2.0;
       break;
-   case StartTimeID + 4 * CenterTimeID:
-      if( mCenter < mStart )
-         mCenter = mStart;
-   case StartTimeID * 4 + CenterTimeID:
-      if( mStart > mCenter )
-         mStart = mCenter;
-      mEnd = mCenter * 2 - mStart;
-      mLength = mStart - mEnd;
-      break;
+
    case EndTimeID + 4 * LengthTimeID:
+      if( mEnd - mLength < 0 )
+         mEnd += (mLength - mEnd);
    case EndTimeID * 4 + LengthTimeID:
-      if( mLength < 0 )
-         mLength = 0;
+      if( mEnd - mLength < 0)
+         mLength -= (mLength - mEnd);
       mStart = mEnd - mLength;
       mCenter = (mStart+mEnd)/2.0;
       break;
-   case EndTimeID + 4 * CenterTimeID:
-      if( mCenter > mEnd )
-         mCenter = mEnd;
-   case EndTimeID * 4 + CenterTimeID:
-      if( mEnd < mCenter )
-         mEnd = mCenter;
-      mStart = mCenter * 2.0 - mEnd;
-      mLength = mEnd - mStart;
-      break;
+
    case LengthTimeID + 4 * CenterTimeID:
+      if( mCenter - (mLength / 2) < 0 )
+         mLength = (mCenter * 2);
    case LengthTimeID * 4 + CenterTimeID:
-      if( mLength < 0 )
-         mLength = 0;
+      if( mCenter - (mLength / 2) < 0 )
+         mCenter = (mLength / 2);
       mStart = mCenter - mLength/2.0;
       mEnd = mCenter + mLength/2.0;
       break;
+
    default:
       // The above should cover all legal combinations of two distinct controls.
       wxFAIL_MSG( "Illegal sequence of selection changes");
    }
 
-   // Places the start-end mrkers on the track panel.
+   // Refresh the controls now
+   ValuesToControls();
+
+   // Places the start-end markers on the track panel.
    mListener->AS_ModifySelection(mStart, mEnd, done);
 }
 
@@ -513,7 +499,6 @@ void SelectionBar::OnChangedTime(wxCommandEvent & event)
 // Called when one of the format drop downs is changed.
 void SelectionBar::OnUpdate(wxCommandEvent &evt)
 {
-   int index = evt.GetInt();
    wxWindow *w = FindFocus();
    NumericTextCtrl ** Ctrls[5] = { &mStartTime, &mEndTime, &mLengthTime, &mCenterTime, &mAudioTime };
    int i;
@@ -524,35 +509,39 @@ void SelectionBar::OnUpdate(wxCommandEvent &evt)
 
    evt.Skip(false);
 
+   auto format = NumericTextCtrl::LookupFormat(NumericConverter::TIME, evt.GetString());
+
    // Save format name before recreating the controls so they resize properly
+   if (mStartTime)
    {
-      auto format = mStartTime->GetBuiltinName(index);
       if (mListener)
          mListener->AS_SetSelectionFormat(format);
    }
 
-   RegenerateTooltips();
-
    // ReCreateButtons() will get rid of our sizers and controls
    // so reset pointers first.
    for( i=0;i<5;i++)
       *Ctrls[i]=NULL;
 
+   mChoice = NULL;
    mRateBox = NULL;
    mRateText = NULL;
+   mSnapTo = NULL;
 
-   ReCreateButtons();
+   ToolBar::ReCreateButtons();
 
    ValuesToControls();
 
-   auto format = mStartTime->GetBuiltinFormat(index);
    for( i=0;i<5;i++)
       if( *Ctrls[i] )
-         (*Ctrls[i])->SetFormatString( format );
+         (*Ctrls[i])->SetFormatName( format );
 
    if( iFocus >=0 )
       if( *Ctrls[iFocus] )
          (*Ctrls[iFocus])->SetFocus();
+
+   RegenerateTooltips();
+
    Updated();
 }
 
@@ -611,7 +600,7 @@ void SelectionBar::OnIdle( wxIdleEvent &evt )
 
    auto &projectAudioIO = ProjectAudioIO::Get( project );
    if ( projectAudioIO.IsAudioActive() ){
-      auto gAudioIO = AudioIOBase::Get();
+      auto gAudioIO = AudioIO::Get();
       audioTime = gAudioIO->GetStreamTime();
    }
    else {
@@ -724,7 +713,7 @@ void SelectionBar::SetSelectionFormat(const NumericFormatSymbol & format)
    // Test first whether changed, to avoid infinite recursion from OnUpdate
    if ( changed ) {
       wxCommandEvent e;
-      e.SetInt(mStartTime->GetFormatIndex());
+      e.SetString(format.Internal());
       OnUpdate(e);
    }
 }
@@ -747,7 +736,9 @@ void SelectionBar::SetRate(double rate)
 
 void SelectionBar::OnRate(wxCommandEvent & WXUNUSED(event))
 {
-   if (mRateBox->GetValue().ToDouble(&mRate) && // is a numeric value
+   auto value = mRateBox->GetValue();
+
+   if (value.ToDouble(&mRate) && // is a numeric value
          (mRate != 0.0))
    {
       NumericTextCtrl ** Ctrls[5] = { &mStartTime, &mEndTime, &mLengthTime, &mCenterTime, &mAudioTime };
@@ -756,6 +747,13 @@ void SelectionBar::OnRate(wxCommandEvent & WXUNUSED(event))
          if( *Ctrls[i] )
             (*Ctrls[i])->SetSampleRate( mRate );
       if (mListener) mListener->AS_SetRate(mRate);
+
+      mLastValidText = value;
+   }
+   else
+   {
+      // Bug 2497 - Undo paste into text box if it's not numeric
+      mRateBox->SetValue(mLastValidText);
    }
 }
 
