@@ -11,8 +11,8 @@
 #ifndef __AUDACITY_SPECTRAL_SELECTION_BAR_LISTENER__
 #define __AUDACITY_SPECTRAL_SELECTION_BAR_LISTENER__
 
-#include "../Audacity.h"
-#include "audacity/Types.h"
+
+#include "Identifier.h"
 
 class AUDACITY_DLL_API SpectralSelectionBarListener /* not final */ {
 
