@@ -518,7 +518,8 @@ static struct DefaultConfigEntry {
    { TransportBarID,         NoBarID,                NoBarID                },
    { ToolsBarID,             TransportBarID,         NoBarID                },
    { EditBarID,              ToolsBarID,             NoBarID                },
-   { AudioSetupBarID,        EditBarID,              NoBarID                },
+   { CutCopyPasteBarID,      EditBarID,              NoBarID                },
+   { AudioSetupBarID,        CutCopyPasteBarID,      NoBarID                },
 #ifdef HAS_AUDIOCOM_UPLOAD
    { ShareAudioBarID,        AudioSetupBarID,        NoBarID                },
    { RecordMeterBarID,       ShareAudioBarID,        NoBarID                },
@@ -549,6 +550,8 @@ static struct DefaultConfigEntry {
    { SpectralSelectionBarID, NoBarID,                NoBarID                },
 };
 
+constexpr ToolBarID HideAfterReset[] = { CutCopyPasteBarID };
+
 // Static member function.
 void ToolManager::OnResetToolBars(const CommandContext &context)
 {
@@ -567,11 +570,12 @@ void ToolManager::Reset()
    {
       int ndx = entry.barID;
       ToolBar *bar = mBars[ ndx ].get();
-
+      
       ToolBarConfiguration::Position position {
          (entry.rightOf == NoBarID) ? nullptr : mBars[ entry.rightOf ].get(),
          (entry.below == NoBarID) ? nullptr : mBars[ entry.below ].get()
       };
+      
       bar->SetSize( 20,20 );
 
       wxWindow *floater;
@@ -669,6 +673,9 @@ void ToolManager::Reset()
       }
 
    }
+
+   for (auto ndx : HideAfterReset)
+      Expose(ndx, false);
    // TODO:??
    // If audio was playing, we stopped the VU meters,
    // It would be nice to show them again, but hardly essential as
@@ -768,6 +775,8 @@ void ToolManager::ReadConfig()
          bShownByDefault = false;
       if( ndx == DeviceBarID )
          bShownByDefault = false;
+      if( ndx == CutCopyPasteBarID )
+         bShownByDefault = false;
       if( ndx == TimeBarID )
          defaultDock = BotDockID;
 
@@ -951,6 +960,19 @@ void ToolManager::ReadConfig()
    wxSystemOptions::SetOption( wxMAC_WINDOW_PLAIN_TRANSITION, mTransition );
 #endif
 
+   // Setup the neighbors according to the
+   // default config
+
+   for (const auto& entry : DefaultConfigTable)
+   {
+      int ndx = entry.barID;
+      ToolBar* bar = mBars[ndx].get();
+
+      bar->SetPreferredNeighbors(
+         static_cast<ToolBarID>(entry.rightOf),
+         static_cast<ToolBarID>(entry.below));
+   }
+
    if (!someFound)
       Reset();
 }
