@@ -13,14 +13,10 @@
 #ifndef __AUDACITY_TRANSCRIPTION_TOOLBAR__
 #define __AUDACITY_TRANSCRIPTION_TOOLBAR__
 
-#include "../Experimental.h"
-
 #include "ToolBar.h"
 
 #include <wx/brush.h> // member variable
 
-#include "audacity/Types.h"
-
 class wxChoice;
 class wxCommandEvent;
 class wxImage;
@@ -30,6 +26,7 @@ class AButton;
 class ASlider;
 class AudacityProject;
 class BoundedEnvelope;
+class sampleCount;
 class WaveTrack;
 
 #ifdef EXPERIMENTAL_VOICE_DETECTION
