@@ -8,19 +8,17 @@ Paul Licameli split from TrackPanel.cpp
 
 **********************************************************************/
 
-#include "../../../../Audacity.h"
-#include "CutlineHandle.h"
 
-#include "../../../../Experimental.h"
+#include "CutlineHandle.h"
 
 #include "../../../../HitTestResult.h"
 #include "../../../../ProjectAudioIO.h"
-#include "../../../../ProjectHistory.h"
+#include "ProjectHistory.h"
 #include "../../../../RefreshCode.h"
 #include "../../../../Snap.h" // for kPixelTolerance
 #include "../../../../TrackPanelMouseEvent.h"
-#include "../../../../UndoManager.h"
-#include "../../../../ViewInfo.h"
+#include "UndoManager.h"
+#include "ViewInfo.h"
 #include "../../../../WaveTrack.h"
 #include "../../../../../images/Cursors.h"
 
@@ -118,6 +116,11 @@ CutlineHandle::~CutlineHandle()
 {
 }
 
+bool CutlineHandle::HandlesRightClick()
+{
+   return true;
+}
+
 UIHandle::Result CutlineHandle::Click
 (const TrackPanelMouseEvent &evt, AudacityProject *pProject)
 {
