@@ -8,17 +8,15 @@ Paul Licameli split from WaveTrackVZoomHandle.cpp
 
 **********************************************************************/
 
-#include "../../../../Audacity.h"
+
 #include "SpectrumVZoomHandle.h"
 
 #include "WaveTrackVZoomHandle.h"
 
-#include "../../../../Experimental.h"
-
 #include "../../../../HitTestResult.h"
-#include "../../../../NumberScale.h"
-#include "../../../../Prefs.h"
-#include "../../../../ProjectHistory.h"
+#include "NumberScale.h"
+#include "Prefs.h"
+#include "ProjectHistory.h"
 #include "../../../../RefreshCode.h"
 #include "../../../../TrackPanelMouseEvent.h"
 #include "../../../../WaveTrack.h"
@@ -39,6 +37,11 @@ void SpectrumVZoomHandle::Enter( bool, AudacityProject* )
 #endif
 }
 
+bool SpectrumVZoomHandle::HandlesRightClick()
+{
+   return true;
+}
+
 UIHandle::Result SpectrumVZoomHandle::Click
 (const TrackPanelMouseEvent &, AudacityProject *)
 {
@@ -142,7 +145,7 @@ void SpectrumVZoomHandle::DoZoom(
       scale = (specSettings.GetScale(min, max));
       const auto fftLength = specSettings.GetFFTLength();
       const float binSize = rate / fftLength;
-      maxFreq = gPrefs->Read(wxT("/Spectrum/MaxFreq"), 8000L);
+      maxFreq = SpectrumMaxFreq.Read();
       // JKC:  Following discussions of Bug 1208 I'm allowing zooming in
       // down to one bin.
       //      const int minBins =
