@@ -8,12 +8,12 @@ Paul Licameli split from TrackPanel.cpp
 
 **********************************************************************/
 
-#include "../../../Audacity.h"
+
 #include "TimeTrackControls.h"
 
 #include "../../../HitTestResult.h"
-#include "../../../Project.h"
-#include "../../../ProjectHistory.h"
+#include "Project.h"
+#include "ProjectHistory.h"
 #include "../../../RefreshCode.h"
 #include "../../../TimeTrack.h"
 #include "../../../widgets/PopupMenuTable.h"
@@ -38,41 +38,17 @@ enum
    OnSetTimeTrackRangeID,
 };
 
-class TimeTrackMenuTable : public PopupMenuTable
-{
-   TimeTrackMenuTable()
-      : PopupMenuTable{ "TimeTrack" }
-   {}
-   DECLARE_POPUP_MENU(TimeTrackMenuTable);
-
-public:
-   static TimeTrackMenuTable &Instance();
-
-private:
-   void InitUserData(void *pUserData) override
-   {
-      mpData = static_cast<CommonTrackControls::InitMenuData*>(pUserData);
-   }
-
-   void DestroyMenu() override
-   {
-      mpData = nullptr;
-   }
-
-   CommonTrackControls::InitMenuData *mpData{};
-
-   void OnSetTimeTrackRange(wxCommandEvent & /*event*/);
-   void OnTimeTrackLin(wxCommandEvent & /*event*/);
-   void OnTimeTrackLog(wxCommandEvent & /*event*/);
-   void OnTimeTrackLogInt(wxCommandEvent & /*event*/);
-};
-
 TimeTrackMenuTable &TimeTrackMenuTable::Instance()
 {
    static TimeTrackMenuTable instance;
    return instance;
 }
 
+void TimeTrackMenuTable::InitUserData(void *pUserData)
+{
+   mpData = static_cast<CommonTrackControls::InitMenuData*>(pUserData);
+}
+
 void TimeTrackMenuTable::OnSetTimeTrackRange(wxCommandEvent & /*event*/)
 {
    TimeTrack *const pTrack = static_cast<TimeTrack*>(mpData->pTrack);
@@ -86,17 +62,19 @@ void TimeTrackMenuTable::OnSetTimeTrackRange(wxCommandEvent & /*event*/)
          _("Lower speed limit"),
          _("Lower speed limit"),
          lower,
-         10,
-         1000);
+         TimeTrackControls::kRangeMin,
+         TimeTrackControls::kRangeMax);
 
       upper = wxGetNumberFromUser(_("Change upper speed limit (%) to:"),
          _("Upper speed limit"),
          _("Upper speed limit"),
          upper,
          lower + 1,
-         1000);
+         TimeTrackControls::kRangeMax);
 
-      if (lower >= 10 && upper <= 1000 && lower < upper) {
+      if (lower >= TimeTrackControls::kRangeMin &&
+          upper <= TimeTrackControls::kRangeMax &&
+          lower < upper) {
          AudacityProject *const project = &mpData->project;
          pTrack->SetRangeLower((double)lower / 100.0);
          pTrack->SetRangeUpper((double)upper / 100.0);
@@ -187,20 +165,17 @@ PopupMenuTable *TimeTrackControls::GetMenuExtension(Track *)
 }
 
 using DoGetTimeTrackControls = DoGetControls::Override< TimeTrack >;
-template<> template<> auto DoGetTimeTrackControls::Implementation() -> Function {
+DEFINE_ATTACHED_VIRTUAL_OVERRIDE(DoGetTimeTrackControls) {
    return [](TimeTrack &track) {
       return std::make_shared<TimeTrackControls>( track.SharedPointer() );
    };
 }
-static DoGetTimeTrackControls registerDoGetTimeTrackControls;
 
 #include "../../ui/TrackView.h"
 
 using GetDefaultTimeTrackHeight = GetDefaultTrackHeight::Override< TimeTrack >;
-template<> template<>
-auto GetDefaultTimeTrackHeight::Implementation() -> Function {
+DEFINE_ATTACHED_VIRTUAL_OVERRIDE(GetDefaultTimeTrackHeight) {
    return [](TimeTrack &) {
       return 100;
    };
 }
-static GetDefaultTimeTrackHeight registerGetDefaultTimeTrackHeight;
