@@ -8,21 +8,22 @@ Paul Licameli split from TrackPanel.cpp
 
 **********************************************************************/
 
-#include "../../Audacity.h"
+
 #include "BackgroundCell.h"
 
-#include "../../AColor.h"
+#include "AColor.h"
 #include "../../HitTestResult.h"
-#include "../../Project.h"
+#include "Project.h"
 #include "../../RefreshCode.h"
 #include "../../SelectionState.h"
-#include "../../Track.h"
+#include "Track.h"
 #include "../../TrackArtist.h"
 #include "../../TrackPanel.h"
+#include "../../TrackPanelConstants.h"
 #include "../../TrackPanelDrawingContext.h"
 #include "../../TrackPanelMouseEvent.h"
 #include "../../UIHandle.h"
-#include "../../ViewInfo.h"
+#include "ViewInfo.h"
 
 #include <wx/cursor.h>
 #include <wx/event.h>
@@ -37,6 +38,9 @@ class BackgroundHandle : public UIHandle
 public:
    BackgroundHandle() {}
 
+   BackgroundHandle(BackgroundHandle&&) = default;
+   BackgroundHandle& operator=(BackgroundHandle&&) = default;
+
    static HitTestPreview HitPreview()
    {
       static wxCursor arrowCursor{ wxCURSOR_ARROW };
@@ -108,12 +112,8 @@ std::vector<UIHandlePtr> BackgroundCell::HitTest
 (const TrackPanelMouseState &,
  const AudacityProject *)
 {
-   std::vector<UIHandlePtr> results;
-   auto result = mHandle.lock();
-   if (!result)
-      result = std::make_shared<BackgroundHandle>();
-   results.push_back(result);
-   return results;
+   auto result = AssignUIHandlePtr(mHandle, std::make_shared<BackgroundHandle>());
+   return { result };
 }
 
 std::shared_ptr<Track> BackgroundCell::DoFindTrack()
@@ -149,3 +149,21 @@ wxRect BackgroundCell::DrawingArea(
    else
       return rect;
 }
+
+auto BackgroundCell::GetMenuItems(
+   const wxRect &, const wxPoint *, AudacityProject * )
+      -> std::vector<MenuItem>
+{
+   // These commands exist in toolbar menus too, but maybe with other labels
+   // TODO: devise a system of registration so that BackgroundCell has no
+   // special knowledge about track sub-types
+   return {
+      { L"NewMonoTrack", XO("Add Mono Track")},
+      { L"NewStereoTrack", XO("Add Stereo Track") },
+      { L"NewLabelTrack", XO("Add Label Track"),  },
+      {},
+      { L"Export", XO("Export Audio..."),  },
+      {},
+      { L"SelectAll", XO("Select All") },
+   };
+}
