@@ -8,10 +8,10 @@ Paul Licameli split from TrackPanel.cpp
 
 **********************************************************************/
 
-#include "../../Audacity.h"
+
 #include "TrackControls.h"
 
-#include "../../Track.h"
+#include "Track.h"
 
 TrackControls::TrackControls( std::shared_ptr<Track> pTrack )
    : CommonTrackCell{ pTrack }
@@ -22,22 +22,22 @@ TrackControls::~TrackControls()
 {
 }
 
+static const AttachedTrackObjects::RegisteredFactory key{
+   []( Track &track ){
+      return DoGetControls::Call( track );
+   }
+};
+
 TrackControls &TrackControls::Get( Track &track )
 {
-   auto pControls =
-      std::static_pointer_cast<TrackControls>( track.GetTrackControls() );
-   if (!pControls)
-      // create on demand
-      track.SetTrackControls( pControls = DoGetControls::Call( track ) );
-   return *pControls;
+   return track.AttachedObjects::Get< TrackControls >( key );
 }
 
 const TrackControls &TrackControls::Get( const Track &track )
 {
-   return Get( const_cast< Track& >( track ) );
+   return Get( const_cast< Track & >( track ) );
 }
 
-template<> auto DoGetControls::Implementation() -> Function {
+DEFINE_ATTACHED_VIRTUAL(DoGetControls) {
    return nullptr;
 }
-static DoGetControls registerDoGetControls;
