@@ -65,8 +65,6 @@ ExpandingToolBar.
 
 #include "ExpandingToolBar.h"
 
-#include "../Experimental.h"
-
 // For compilers that support precompilation, includes "wx/wx.h".
 #include <wx/wxprec.h>
 
@@ -80,10 +78,10 @@ ExpandingToolBar.
 #include <wx/dialog.h>
 
 #include "AButton.h"
-#include "../AllThemeResources.h"
+#include "AllThemeResources.h"
 
 const int kToggleButtonHeight = 8;
-const int kTimerInterval = 50; // every 50 ms -> ~20 updates per second
+const int kMyTimerInterval = 50; // every 50 ms -> ~20 updates per second
 const wxRect kDummyRect = wxRect(-9999, -9999, 0, 0);
 
 enum {
@@ -388,7 +386,7 @@ void ExpandingToolBar::Fit()
    // wxTimers seem to be a little unreliable - sometimes they stop for
    // no good reason, so this "primes" it every now and then...
    mTimer.Stop();
-   mTimer.Start(kTimerInterval);
+   mTimer.Start(kMyTimerInterval);
 }
 
 bool ExpandingToolBar::IsCursorInWindow()
@@ -546,11 +544,10 @@ void ExpandingToolBar::StartMoving()
 //   ImageRoll tgtImageRoll = ImageRoll(ImageRoll::VerticalRoll,
 //                                      tgtImage,
 //                                      magicColor);
-   mTargetPanel = safenew ImageRollPanel(mAreaParent, -1, //tgtImageRoll,
+   mTargetPanel = safenew wxPanelWrapper(mAreaParent, -1, //tgtImageRoll,
                                      wxDefaultPosition,
                                      wxDefaultSize,
                                      wxTRANSPARENT_WINDOW);
-   mTargetPanel->SetLogicalFunction(wxXOR);
    mTargetPanel->SetSize(mDropTarget);
 
    // This gives time for wx to finish redrawing the window that way.
@@ -649,7 +646,7 @@ void ExpandingToolBar::FinishMoving()
 
    mTopLevelParent->Refresh(true);
 
-   mTimer.Start(kTimerInterval);
+   mTimer.Start(kMyTimerInterval);
 }
 
 //
