@@ -34,7 +34,7 @@ class NumericTextCtrl;
 #define GRID_VALUE_TIME wxT("Time")
 #define GRID_VALUE_FREQUENCY wxT("Frequency")
 
-class NumericEditor /* not final */ : public wxGridCellEditor
+class AUDACITY_DLL_API NumericEditor /* not final */ : public wxGridCellEditor
 {
 public:
 
@@ -116,7 +116,8 @@ wxComboBox.
 **************************************************************************/
 #define GRID_VALUE_CHOICE wxT("Choice")
 
-class ChoiceEditor final : public wxGridCellEditor, wxEvtHandler
+class AUDACITY_DLL_API ChoiceEditor final
+   : public wxGridCellEditor, wxEvtHandler
 {
 public:
 
@@ -180,7 +181,7 @@ public:
 \brief wxGrid with support for accessibility.
 **************************************************************************/
 
-class Grid final : public wxGrid
+class AUDACITY_DLL_API Grid final : public wxGrid
 {
 
  public:
@@ -210,6 +211,7 @@ class Grid final : public wxGrid
 
    void OnSetFocus(wxFocusEvent &event);
    void OnSelectCell(wxGridEvent &event);
+   void OnEditorShown(wxGridEvent &event);
    void OnKeyDown(wxKeyEvent &event);
 
  private:
