@@ -18,7 +18,7 @@
 #ifndef __AUDACITY_WIDGETS_PROGRESSDIALOG__
 #define __AUDACITY_WIDGETS_PROGRESSDIALOG__
 
-#include "../Audacity.h"
+
 
 #include <vector>
 #include <wx/defs.h>
@@ -26,17 +26,12 @@
 
 #include "wxPanelWrapper.h" // to inherit
 
+#include "BasicUI.h" // For ProgressResult
+using ProgressResult = BasicUI::ProgressResult;
+
 class wxGauge;
 class wxStaticText;
 
-enum class ProgressResult : unsigned
-{
-   Cancelled = 0, //<! User says that whatever is happening is undesirable and shouldn't have happened at all
-   Success,       //<! User says nothing, everything works fine, continue doing whatever we're doing
-   Failed,        //<! Something has gone wrong, we should stop and cancel everything we did
-   Stopped        //<! Nothing is wrong, but user says we should stop now and leave things as they are now
-};
-
 enum ProgressDialogFlags
 {
    pdlgEmptyFlags = 0x00000000,
@@ -52,7 +47,9 @@ enum ProgressDialogFlags
 /// ProgressDialog Class
 ////////////////////////////////////////////////////////////
 
-class AUDACITY_DLL_API ProgressDialog /* not final */ : public wxDialogWrapper
+class AUDACITY_DLL_API ProgressDialog /* not final */ :
+    public wxDialogWrapper,
+    public BasicUI::ProgressDialog
 {
 public:
    ProgressDialog();
@@ -85,7 +82,9 @@ public:
                int flags = pdlgDefaultFlags,
                const TranslatableString & sRemainingLabelText = {});
 
-   void Reinit();
+   void Reinit() override;
+
+   void SetDialogTitle(const TranslatableString& title) override;
 
 protected:
    bool Create(const TranslatableString & title,
@@ -101,8 +100,12 @@ public:
    ProgressResult Update(wxLongLong current, wxLongLong total, const TranslatableString & message = {});
    ProgressResult Update(wxLongLong_t current, wxLongLong_t total, const TranslatableString & message = {});
    ProgressResult Update(int current, int total, const TranslatableString & message = {});
-
-   void SetMessage(const TranslatableString & message);
+   
+   ProgressResult Poll(
+      unsigned long long numerator, unsigned long long denominator,
+      const TranslatableString& message = {}) override;
+      
+   void SetMessage(const TranslatableString & message) override;
 
 protected:
    wxWindowRef mHadFocus;
